// This class is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 270499
//
//@created	270499  KH
package asgaard.lang;

// Import Skid-Api
import asgaard.skid.*;
import asgaard.utils.log.*;

// Import the java classes used
import java.io.*;

import java.lang.reflect.*;


/**
 * This Class is a special lightweight type of Bean which does not carry any
 * Attributes but it's name (no multilanguage-support). Therefor it is used to hold e.g. String -
 * identifyers used by the System.
 *<p>
 *<b><i>Minimal Usage:</i></b> (The RemoteBroker should be instanciated at this time)
 *<code>  Def d = new Def( <name> );</code><br>
 *<code>  b.flush();</code><br>
 *<p>
 * <i>Remark:</i> A System-Object cannot be modified and carries the created-Timestamp 0
 *<p>
 *<b><i>Remark:</i></b> To inherit this kind of SkidBean modifications within the
 *code of the BeanBroker are nessecary (look for the <CODE>BEANTYPE</CODE> pattern).
 */
public class Def extends LightBean {

  protected String shortName = "SD" + String.valueOf(tsCreated);
  protected String longName;

  /**
   * Generates a empty definition
   */
  public Def() {  }

  /**
   * Generates a new definition with a given name.
   *@param sName the short name of the def
   *@param lName the long name of the def
   *@throws SkidException
   */
  public Def(String sName, String lName) throws SkidException {
    setName( sName );
    setLongname( lName );
  }

  /**
   * Initialises a new Lightweight Bean with a given name and timestamps <b>Do not use manually</b>!!!
   *@param key of the Bean
   *@param calssKey of the Bean
   *@param name the short name of the def
   *@param content the long name of the def
   *@param tsC created - timestamp
   *@param tsM modified - timestamp
   *@throws SkidException
   */
  public void init (int key, int classKey, String name, Object content, long tsC, long tsM) throws SkidException {
    setName( name );
    setLongname( (String) content );
    this.key = key;
    this.cKey = classKey;
    tsCreated = tsC;
    tsModified = tsM;
  }

  /**
   * Returns the name of the Bean which shold be human readable and may be
   * unique in some cases, but this is not strictly required.
   */
  public String getName() {
    return shortName;
  }

  /**
   * Sets the name of the Definition.
   *@param name the value
   */
  public void setName(String name) {
    shortName = name;
  }

  /**
   * Returns the long name of the definition.
   */
  public String getLongname() {
    if (longName == null)
      return getName();
    return longName;
  }

  /**
   * Sets the long name of the definition.
   *
   *@param name a new name
   */
  public void setLongname(String name) {
    longName = name;
    modify(null, null);
  }


  /**

   * Returns the wrapped Object, in this case it's the Longname.
   */
  public Object getObject() {
    return getLongname();
  }

}










