// This class is part of Banshee using Skid
// Banshee is an implementation from the asgaard-project in the sport-domain
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 150699
//
//@created	150699  KH
package asgaard.lang;

import asgaard.skid.*;
import asgaard.utils.unit.*;

/**
 * This class extends Units adding title, prefix, postfix and description to each
 * language-variant of the Unit.
 *<p>
 *<b><i>Remark:</i></b> To inherit this kind of SkidBean modifications within the
 *code of the BeanBroker are nessecary (look for the <CODE>BEANTYPE</CODE> pattern).
 *
 *@see asgaard.utils.unit.Convert
 */
public class Item extends Unit {

  /**
   * Instanciates a new instance <b>Do not use manually</b>.
   */
  public Item() { super(); }

  /**
   * Generates a new Unit with the Default-Values
   *@param sName the identifying (common) Name of the Word
   *@param multiplier the factor
   *@throws SkidException
   */
  public Item(String sName, String lName, Convert unit) throws SkidException {
    super(sName, lName, unit);
  }
  /**
   * Inits a new Item with a given Collection <b>Do not use manually</b>.
   * Only use by the BeanBroker.
   *@param content collection of Words indized by other Def's
   *@throws SkidException if the given Object is no valid Hashtable
   *@param tsC created - timestamp
   *@param tsM modified - timestamp
   */
  public void init(String name, Object content, long tsC, long tsM) throws SkidException {
    super.init(name, content, tsC, tsM);
  }

  /**
   * Extends <code>setName</code> adding title, prefix, postfix and description.
   *@see asgaard.lang.Word.setName
   *@param index is the key of an other Definition-Bean
   *@param name is the default (short) name of the Item
   *@param title is the heading of the item (long version of the word)
   *@param prefix is displayed before the item
   *@param postfix is displayed after the item
   *@param description is an extensive note to the numerous item
   *@throws SkidException if entry is not valid (null)
   */
  public void setItem(int index, String name, String title, String prefix, String postfix, String description) throws SkidException {
    Object[] item = new Object[5];
    item[0] = name;
    item[1] = title;
    item[2] = prefix;
    item[3] = postfix;
    item[4] = description;
    setName(index, item);
  }

  /**
   * Returns the heading of the item
   *@param index is the key of an other Definition-Bean
   *@returns the title
   *@see sgaard.lang.Word.getLongName
   */
  public String getTitle(int index) {
    return getLongName(index);
  }

  /**
   * Returns the prefix of the item
   *@param index is the key of an other Definition-Bean
   *@returns the prefix if any
   */
  public String getPrefix(int index) {
    Object[] item = getArgs( index );
    if (item.length>2)
      return (String) item[2];
    else
      return "";
  }

  /**
   * Returns the postfix of the item
   *@param index is the key of an other Definition-Bean
   *@returns the postfix if any
   */
  public String getPostfix(int index) {
    Object[] item = getArgs( index );
    if (item.length>3)
      return (String) item[3];
    else
      return "";
  }

  /**
   * Returns the description of the item
   *@param index is the key of an other Definition-Bean
   *@returns the description if any
   */
  public String getDescription(int index) {
    Object[] item = getArgs( index );
    if (item.length>4)
      return (String) item[4];
    else
      return "";
  }

}
