// This class is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	2.0 080499
package asgaard.lang;
// Import SkidAPI
import asgaard.skid.*;
import asgaard.utils.log.*;
// Import java classes
import java.util.*;
//import java.rmi.RemoteException;

/**
 * Person is the base implementation for the representation of any human.
 *@see asgaard.sjid.SkidBean
 */
public class Person extends SkidBean {

  transient final private String beanClass = "asgaard.lang.Def";
  transient private Def Nation;        // Key to the Nation - Entry
  transient private Def Sex;           // Key to the Sex
  private int sex = 0;
  private int nat = 0;
  private int SINr = 0;
  private String Title = "";
  private String Lastname = "";
  private String Firstname = "";
  private String Nickname = "";
  private String Initial = "";
  private Date Birthdate;

  /**
   * Creates an instance of the <code>Person</code> class.
   */
  public Person() {
//  public Person() throws RemoteException {
    super();
  }

  /**
   * Get's the update policy defined by the <code>UPDATE_*</code> constants.
   *@see asgaard.skid.RemoteBean.UPDATE_WEAK
   */
  public byte getUpdatePolicy() {
    return BeanBroker.UPDATE_WEAK;
  }

  /**
   * Gets the qualified Name of the Person.
   */
  public String getName() {
    BeanBroker.CALENDAR.setTime( Birthdate );
    return  Firstname + " " +
            Lastname + " " +
            BeanBroker.CALENDAR.get(Calendar.DATE) + "." +
            BeanBroker.CALENDAR.get(Calendar.MONTH) + "." +
            BeanBroker.CALENDAR.get(Calendar.YEAR) + " " +
            String.valueOf( SINr );
  }

  /**
   * Get the Identifier of the Bean which shall be unique worldwide for
   * different Objects (like different Plans with luckily the same Name) and
   * common for identical Objects (like a Person reentered), <b>do not use
   * manually</b>. The method <code>getName</code> is used to create an
   * Identifier.
   *
   * <i>This method usually is used only once at creation time and may produce
   * different results every time.</i>
   *
   *@see asgaard.skid.SkidBean.enforceUniqueness
   *@see asgaard.skid.SkidBean.getName
   */
  public String getIdentifier() {

    StringTokenizer classTok = new StringTokenizer(this.getClass().getName(), ".");

    String className = "x";

    while (classTok.hasMoreTokens()) {

      className = classTok.nextToken();

    }

    return className+":"+getName();

  }

  /**
   * Sets the social-insurance-number of the person.
   *@param SINr the number
   */
  public void setSINr (int SINr) { this.SINr = SINr; }
  /**
   * Sets the Title of the person.
   *@param Title the Title
   */
  public void setTitle (String Title) { this.Title = Title; }
  /**
   * Sets the last name of the person.
   *@param Lastname the last name
   */
  public void setLastname (String Lastname) { this.Lastname = Lastname; }
  /**
   * Sets the first name of the person.
   *@param Firstname the first name
   */
  public void setFirstname (String Firstname) { this.Firstname = Firstname; }
  /**
   * Sets the nickname of the person.
   *@param nickname the nickname
   */
  public void setNickname (String Nickname) { this.Nickname = Nickname; }
  /**
   * Sets the initials of the person.
   *@param Initial the Initials
   */
  public void setInitial (String Initial) { this.Initial = Initial; }
  /**
   * Sets the Birthdate of the person.
   *@param Birthdate the Birthdate
   */
  public void setBirthdate (Date Birthdate) { this.Birthdate = Birthdate; }

  /**
   * Returns the social-insurance-number of the person.
   *return SINr the social-insurance-number
   */
  public int getSINr() { return SINr; }
  /**
   * Returns the title of the person.
   *return the title
   */
  public String getTitle() { return Title; }
  /**
   * Returns the last name of the person.
   *return the last name
   */
  public String getLastname() { return Lastname; }
  /**
   * Returns the first name of the person.
   *return the first name
   */
  public String getFirstname() { return Firstname; }
  /**
   * Returns the nickname of the person.
   *return the nickname
   */
  public String getNickname() { return Nickname; }
  /**
   * Returns the initials of the person.
   *return initials
   */
  public String getInitial() { return Initial; }
  /**
   * Returns the date of birth of the person.
   *return date of birth
   */
  public Date getBirthdate() { return Birthdate; }

  /**
   * Returns the Nation of the Person
   */
  public Def getNation() {
    if (Nation == null)
    try {
      Nation = (Def) this.broker.getBean(nat);
    } catch (Exception e) {
      return null;
    }
    return Nation;
  }

  /**
   * Defines the Nation of the Person

   */
  public boolean setNation(int key) {
    try {
      RemoteBean RB = this.broker.getBean(key);
      if ((RB != null) && (RB instanceof Def)) {
        Nation = (Def) RB;
        nat = key;
        return true;
      } else
        return false;
      } catch (Exception e) {
      return false;
    }
  }

  /**
   * Defines the Nation of the Person
   */
  public boolean setNation(String value) {
    try {
      return setNation(Integer.parseInt( value ));
    } catch (Exception e) {
      setNation(SkidBean.EMPTY_KEY);
      return false;
    }
  }

  /**
   * Returns the Gender of the Person
   */
  public Def getSex() {
    if (Sex == null)
    try {
      Sex = (Def) this.broker.getBean(sex);
    } catch (Exception e) {
      return null;
    }
    return Sex;
  }

  /**
   * Defines the Gender of the Person
   */
  public boolean setSex(int key) {
    try {
      RemoteBean RB = this.broker.getBean(key);
      if ((RB != null) && (RB instanceof Def)) {
        Sex = (Def) RB;
        sex = key;
        return true;
      } else
        return false;
      } catch (Exception e) {
      return false;
    }
  }

  /**
   * Defines the Gender of the Person
   */
  public boolean setSex(String value) {
    try {
      return setSex(Integer.parseInt( value ));
    } catch (Exception e) {
      setSex(SkidBean.EMPTY_KEY);
      return false;
    }
  }


}

