// This class is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 030699
//
//@created	030699  KH
package asgaard.lang;

// Import Java-classes
import java.util.*;

// Import Skid-API
import asgaard.skid.*;
import asgaard.utils.unit.*;

/**
 * This class implements the Mechanism to use different units at a given measure.
 * These Units are stored as instances within Skid and relate on a choosen
 * Base - Unit, e.g. [meter] for Distances (which may also be displayed as [mile]
 * by a unit - instance.
 *<p>
 * The conversation mechanism itselfe is implemented by the classes in the
 * <code>asgaard.utils.unit</code> package using the <code>Convert</code>
 * interface.
 *<p>
 *<b><i>Remark:</i></b> To inherit this kind of SkidBean modifications within the
 *code of the BeanBroker are nessecary (look for the <CODE>BEANTYPE</CODE> pattern).
 *
 *@see asgaard.utils.unit.Convert
 */
public class Unit extends Word {

  private Convert unit;

  /**
   * Instanciates a new unit <b>Do not use manually</b>.
   */
  public Unit() { super(); }

  /**
   * Generates a new Unit with the Default-Values
   *@param sName the identifying (common) Name of the Word
   *@param multiplier the factor
   *@throws SkidException
   */
  public Unit(String sName, String lName, Convert unit) throws SkidException {
    super(sName, lName);
    this.unit = unit;                               // DIRTY TRICK VVVVV
    Hashtable words = (Hashtable) getObject();
    words.clear();
    Object[] item = new Object[3];
    item[0] = sName;
    item[1] = lName;
    item[2] = unit;
    words.put(new Integer( 0 ), item);
  }

  /**
   * Inits a new Unit with a given Collection <b>Do not use manually</b>.
   * Only use by the BeanBroker.
   *@param content collection of Words indized by other Def's
   *@throws SkidException if the given Object is no valid Hashtable
   *@param tsC created - timestamp
   *@param tsM modified - timestamp
   */
  public void init(String name, Object content, long tsC, long tsM) throws SkidException {
    super.init(name, content, tsC, tsM);
    Hashtable words = (Hashtable) getObject();      // DIRTY TRICK VVVVV
    Object[] item = (Object[]) words.get( new Integer( 0 ));
    this.unit = (Convert) item[2];
  }

 /**
  * Returns the conversion object
  *@see asggard.utils.unit.Convert
  *@returns a conversation Object
  */
  public Convert convert() {
    return unit;
  }

  /**
   * Sets the conversation Object.
   *
   *@param factor a multiplier
   */
  public void setConvert(Convert unit) {
    this.unit = unit;
    modify(null, null);
  }

}