// This class is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 030699
//
//@created	030699  KH
package asgaard.lang;

// Import Java-classes
import java.util.*;

// Import Skid-API
import asgaard.skid.*;
/**
 * Same as Definition, but supporting multi-languages for a given word (vocabulary).
 * Is <code>Def</code> used for system settings and basic values, <code>Word</code>
 * is designed for user-access. It's index will be values given by Def's.
 *<p>
 *<b><i>Remark:</i></b> To inherit this kind of SkidBean modifications within the
 *code of the BeanBroker are nessecary (look for the <CODE>BEANTYPE</CODE> pattern).
 */
public class Word extends Def {

  /**
   * The List with the entries
   */
  private Hashtable words = null;

  /**
   * Instanciates a new Word <b>Do not use manually</b>.
   */
  public Word() {
    words = new Hashtable();
  }

  /**
   * Generates a new Word with the Default-Values
   *@param sName the identifying (common) Name of the Word
   *@param lName the default vocabulary
   *@throws SkidException
   */
  public Word(String sName, String lName) throws SkidException {
    super(sName, lName);
    words = new Hashtable();
    setName(0, sName, lName);
    setName( sName );
    setLongname( lName );
  }

  /**
   * Inits a new Word with a given Collection <b>Do not use manually</b>.
   * Only use by the BeanBroker.
   *@param content collection of Words indized by other Def's
   *@throws SkidException if the given Object is no valid Hashtable
   *@param tsC created - timestamp
   *@param tsM modified - timestamp
   */
  public void init(String name, Object content, long tsC, long tsM) throws SkidException {
    if ((content == null) || !(content instanceof Hashtable))
      throw new SkidException("No valid Hashtable");
    words = (Hashtable) content;
    Object[] item = (Object[]) words.get( new Integer( 0 ));
    setName( (String) item[0] );
    setLongname( (String) item[1] );
    key = getKey();
    tsCreated = tsC;
    tsModified = tsM;
    setAge();
  }

  /**
   * Adds a word to the collection, using <code>index</code> to differ between different
   * languages/context keys.
   *@param index is the key of an other Definition-Bean
   *@param sWord is the default (short) version of the word
   *@throws SkidException if entry is not valid (null)
   */
  public void setName(int index, String sWord) throws SkidException {
    Object[] item = new Object[1];
    item[0] = sWord;
    setName(index, item);
  }

  /**
   * Adds a word to the collection, using <code>index</code> to differ between different
   * languages/context keys.
   *@param index is the key of an other Definition-Bean
   *@param sWord is the default (short) version of the word
   *@param lWord is the long version of the word
   *@throws SkidException if entry is not valid (null)
   */
  public void setName(int index, String sWord, String lWord) throws SkidException {
    Object[] item = new Object[2];
    item[0] = sWord;
    item[1] = lWord;
    setName(index, item);
  }

  /**
   * Adds a word to the collection, using <code>index</code> to differ between different
   * languages/context keys.
   *@param index is the key of an other Definition-Bean
   *@param word is an array of 2 Entry: Short- and Long version of the word
   *@throws SkidException if the array is not valid (<1 Strings)
   */
  protected void setName(int index, Object[] word) throws SkidException {
    if ((word == null) || (word.length<1) || (word[0] == null))
      throw new SkidException("Word not valid.");
    words.put(new Integer( index ), word);
    modify(null, null);
  }

  /**
   * Returns the word in short form and the asked language if available, if not
   * the default-word is returned.
   *@param index is the key of an other Definition-Bean
   *@returns the short word
   */
  public String getName(int index) {
    Object[] item = (Object[]) words.get( new Integer( index ));
    if (item == null)
      return getName();
    else
      return (String) item[0];
  }

  /**
   * Returns the word in long form and the asked language if available, if not
   * the default-word is returned.
   *@param index is the key of an other Definition-Bean
   *@returns the long word
   */
  public String getLongName(int index) {
    Object[] item = (Object[]) words.get( new Integer( index ));
    if ((item == null) || (item.length<2) || (item[1] == null))
      return getName();
    else
      return (String) item[1];
  }

  /**
   * Returns all Objects stored concerned to an searched key.
   *@param index is the key of an other Definition-Bean
   *@returns the short word
   */
  public Object[] getArgs(int index) {
    return (Object[]) words.get( new Integer( index ));
  }

  /**
   * Returns the wrapped Object, in this case it's the collection of word's
   *@return a Hashtable
   */
  public Object getObject() {
    return words;
  }

}