// This is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	2.0 010499
//
//@created	110398  KH
//@modified	300398  KH Connection implemented
//@modified	310398  KH stored procedures
//@modified 130598  KH DataObjectPropertys, neues Speicherkonzept
//@modified 210598  KH Workaround BLOB-Bug mit HexInputStream
//@modified 260299  KH + *crypto*
//@modified 100399  KH ~ rename to BeanBroker (former name DataObjectCollection)
//@modified 080499  KH ~ new Beans
package asgaard.skid;
// Import the JDBC classes
import java.sql.*;

// Import the java classes used
import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import java.rmi.*;

// Import SkidAPI
import asgaard.skid.*;

import asgaard.lang.*;

import asgaard.utils.*;

import asgaard.utils.log.*;

import asgaard.utils.test.*;

import asgaard.utils.cryptography.*;

/**
 * <p>BeanBroker is the transparent objectstorage of "Skid".
 * It is th object-oriented interface of the database and maintain all
 * databean-object-instances at runtime within a common access functionality.
 * CREATE, FIND, UPDATE is done completely transparent, the used database is also hidden.
 *<p>
 * <b><i>Important:</i></b> <code>public static BeanBroker THIS</code> has to be defined
 * within each VM to guarantie regular behavior within Skid!
 *<p>
 * To become persistent an Object has to implement <code>RemoteBean</code>.
 *
 *@see asgaard.skid.SkidBean
 */
public final class BeanBroker implements RemoteBroker, TestIt {

    /**
     * Global pointer to the BeanBroker (one per VM)
     */
    public static RemoteBroker THIS = null;

    /**
     * String with the ip-Adress of the Host
     */
    public static String HOST = null;

    /**
     * Not used for Search.
     */
    public static final byte FILTER_NONE = 0;
    /**
     * exact the same value "="
     */
    public static final byte FILTER_EQ = 1;
    /**
     * greater ">"
     */
    public static final byte FILTER_GR = 2;
    /**
     * greater or equal ">="
     */
    public static final byte FILTER_GR_EQ = 3;
    /**
     * smaller "<"
     */
    public static final byte FILTER_SM = 4;
    /**
     * smaller or equal "<="
     */
    public static final byte FILTER_SM_EQ = 5;
    /**
     * not equal "!="
     */
    public static final byte FILTER_NOT_EQ = 6;

  /**
   * No information for the UNDO-mechanism is stored
   */
  public static final int HISTORY_NONE = -1;
  /**
   * All information for the UNDO-mechanism is stored.
   */
  public static final int HISTORY_ALL = 0;
  /**
   * No Updates are done.
   */
  public static final byte UPDATE_NONE = -1;
  /**
   * All modifications are done, no difference whether there are changes made or not.
   */
  public static final byte UPDATE_ALL = 0;
  /**
   * Only if there is no newer Bean at the Broker an update is made.
   */
  public static final byte UPDATE_WEAK = 1;
  /**
   * Only if the Broker-Version of the Bean has not changed in meantime.
   */
  public static final byte UPDATE_STRONG = 2;
  /**
   * Beantypes: Holds Meta-Information about the class
   */
  public static final byte TYPE_CLASS = 0;
  /**
   * Beantypes: A SkidBean, made persistand by complete storage through serialisation
   */
  public static final byte TYPE_BEAN = 1;
  /**
   * Beantypes: A lightweight-Bean made persistance storing only parts of itselfe
   */
  public static final byte TYPE_LIGHTWEIGHT = 2;
  /**
   * Beantypes: A heavyweight-Bean it's content is stored in an extra table
   */
  public static final byte TYPE_HEAVYWEIGHT = 3;
  /**
   * Name of the Dumpfile
   */
  public static String Dump_Fiele_Name = "SKIDDUMP.SQL";

  public final static GregorianCalendar CALENDAR = new GregorianCalendar();

  public final static int ID_HIGH = (int) 0x00100000;
  public final static int ID_FAIL = -1;
  public final static int PUBLIC_ID = 1;
  private int broker_id = ID_FAIL;
  private int broker_id2 = ID_FAIL;
  private int running_broker_nr = 0;
  private int running_broker_nr2 = 0;

    private final static byte[] key = {   // Key to encrypt passwords
        (byte)0xe4, (byte)0x3a, (byte)0x32, (byte)0x90,
				(byte)0xb8, (byte)0xef, (byte)0x01, (byte)0x62,
				(byte)0xa7, (byte)0x0e, (byte)0x5b, (byte)0x92,
				(byte)0xc4, (byte)0x68, (byte)0x22, (byte)0x36};
    private final static IDEA crypt = new IDEA();
    private static final String tab = "0123456789ABCDEF";

    private String driverClass;
    private String connStr;
    private String connUser;
    private String connPwd;
    private String localDB;
    private boolean runLocal;
    private ResourceBundle sRes;

    private Connection  conn = null;	    // Connection zur Datenbank
    private byte log_level = 0;           // Which Statements are created
    protected LogClient log = null;       // Log-Client
    final static boolean debug = Skid.debug;
    final static boolean create_db = false;

    private boolean keyValid = false;	    // Is the Connection valid?
					                                // If not, there are no write-Ops possible

    /**
     * Database Maintainanceobject (Backup, Restore, ...)
     *@see asgaard.Skid.SkidMaintain
     */
    public SkidMaintain maintain;         // Database Maintainanceobject

    private Hashtable	beanCache;                    // Chache with SkidBeans
    private Hashtable linkCache;                    // Chache with Relations
    private static Hashtable classKeyList = new Hashtable();  // List with existing Classes
    private static Hashtable classNameList = new Hashtable();  // List with existing Classes
    private static Hashtable classKeyList2 = null;
    private static Hashtable classNameList2 = null;

    private BeanBrokerCall insertBean;  // does the Database INSERT
    private BeanBrokerCall updateBean;  // does the Database UPDATE
    private BeanBrokerCall selectBean;  // does the Database SELECT
    private BeanBrokerCall selectClass; // does the Database SELECT
    private BeanBrokerCall insertClass; // does the Database INSERT
    private BeanBrokerCall insertLink;  // does the Database INSERT
    private BeanBrokerCall updateLink;  // does the Database UPDATE
    private BeanBrokerCall selectLink;  // does the Database SELECT
    private BeanBrokerCall selectLink2; // does the Database SELECT
    private BeanBrokerCall createKey;   // does the Database CREATE for a new Bean-Key table
    private BeanBrokerCall deleteKey;   // does the Database DELETE in a Bean-Key table
    private BeanBrokerCall insertKey;   // does the Database INSERT in a Bean-Key table
    private BeanBrokerCall selectKey;   // does the Database SELECT in a Bean-Key table
    private BeanBrokerCall dropKey;     // DROPS a Bean-Key table
    private BeanBrokerCall insertEvent; // does the Database INSERT
    private BeanBrokerCall updateEvent; // does the Database UPDATE
    private BeanBrokerCall deleteEvent; // does the Database DELETE
    private BeanBrokerCall selectEvent; // does the Database SELECT

    /**
     * Encrypts an password for Database connection with the built in key.
     *
     *@see asgaard.utils.cryptography.IDEA
     *@param  clearPwd  clear password
     *@return encrypted password
     */
    public final static String encryptPwd(String clearPwd) {
      if (clearPwd.equals("")) return "";
      try {
        byte[] inbuf = clearPwd.getBytes();             // using asgaard.utils.cryptography.IDEA algotithm
	  	  crypt.setKey( key );
        crypt.encrypt(inbuf);
        byte[] enc = crypt.encryptFinish();
        String hex = "";
        int j;
        for (int i=0; i < enc.length; i++) {
          j = (enc[i] + 256) % 256;
          hex += tab.charAt( j >>> 4 );                //SHR 4
          hex += tab.charAt( j & 15 );                 //Mask LSB
        }
        return hex;
      } catch (Exception e) {
        return "";
      }
    }

    /**
     * Creating Object-Broker and Establishing connection to the database.
     * Decrypting password for establishing connection if available.
     *
     *@param   driverClass zu ladende Treiberklasse
     *@param   connStr     Connection-String um an der DB anzumelden
     *@param   connUser    Username
     *@param   connPwd     Pawort
     *@param   output	    Anzeige
     *@param  log       LogClient
     *@param  loglevel  which statements shall be created
     *@param	nr	serialnumber
     *@param	xKey	encrypted key in HEX-Format, empty keys are invalid
     *@see DataObjectCollection#DataObjectCollection
     *@see asgaard.utils.cryptography.IDEA
     */
    public BeanBroker (String driverClass, String connStr, String connUser, String connPwd, String localDB, LogClient log, byte loglevel, boolean runLocal, ResourceBundle sRes, long nr, String xKey) throws Exception {
	    super();
      THIS = this;
      this.log = log;
      if (log != null)
        log_level = loglevel;
      try {
        certifyClient(nr, xKey);
      } catch (Exception e) {
        broker_id = (int) nr;
      }
	    if (conn == null) 				                                            // open the connection to the database
        connectDatabase (driverClass, connStr, connUser, connPwd, localDB, runLocal, sRes);
      if (!keyValid) {
        broker_id = ID_FAIL;
        certifyClient(nr, xKey);
      }
      maintain = new SkidMaintain( conn, sRes );	                          // Create a Maintenance-Object
      if (debug) try {
        localDB = checkDir(localDB);
        System.out.println("Database dump to " + localDB + Dump_Fiele_Name);
        maintain.dump(new FileWriter( localDB + Dump_Fiele_Name));
      } catch (Exception e) {
        System.err.println("Dump failed: " + e.toString());
      }
	    beanCache = new Hashtable();
	    linkCache = new Hashtable();
    }

    /**
     * Returns the current LogLevel
     */
    public byte getLogLevel() {
      return log_level;
    }
    /**
     * Checks a directory and creates it if it does not exist
     */
    private String checkDir(String path) {
        if (!path.endsWith(File.separator))
          path += File.separator;
        File testdir = new File(path);                               // Check if directory exists
        if (! testdir.exists()) {
          testdir.mkdir();
          logMsg( this.getClass().getName(), "db_mkdir", LogMsg.LOG_REMARK, true,
                         "Local database directory ("+path+") created.");
        }
        return path;
    }

    /**
     * Resonnecting to the database after a timeout
     */
    protected Connection reconnect() throws Exception {
      Driver D = (Driver)Class.forName(driverClass).newInstance(); 	    // Load the JDBC driver
      if (runLocal) {                                                   // FLATFILE DB
        Properties dir = new Properties();
        localDB = checkDir(localDB);
        dir.put("Directory", localDB);                                  // set DB
	      conn = DriverManager.getConnection (connStr, dir);              // establishing the connection
      } else {                                                          // REMOTE DB
        String clearPwd = "";
        if ((connPwd.length()>0)) {                                   // DECRYPTING THE PASSWORD
          byte[] inbuf = new byte[connPwd.length()/2];
          for (int i=0;i < inbuf.length; i++) {
            inbuf[i] = (byte) (tab.indexOf(connPwd.charAt(i*2))*16 +
                       tab.indexOf(connPwd.charAt(i*2+1)));             // using asgaard.utils.cryptography.IDEA algotithm
              }
		      crypt.setKey( key );
          crypt.decrypt(inbuf);
          clearPwd = new String(crypt.decryptFinish());
        }
	      conn = DriverManager.getConnection (connStr,connUser,clearPwd); // establishing the connection
      }
      LogClient cLog = null;
      if (log_level >= LogMsg.LOG_BACKUP)
        cLog = log;
      BeanBrokerCall.setConnection(conn, this, sRes);
      return conn;
    }
    /**
     * Does the initialisation of the Broker with the database content
     *@returns if succesfull
     */
    private boolean init() {
	    try {						                          // Hallo DB?
  	    Statement stmt = conn.createStatement();// Create a statement
		    ResultSet rset = stmt.executeQuery (java.text.MessageFormat.format(sRes.getString("selectRunningNr"), new Object[] {String.valueOf(ID_HIGH*broker_id),String.valueOf(ID_HIGH*(broker_id+1))}));
        while (rset.next()) {                   // Check stored classes
          int nr = rset.getInt(1);
          if (running_broker_nr < nr)
            running_broker_nr = nr;
        }
        running_broker_nr = running_broker_nr / BeanBroker.ID_HIGH;
        classKeyList.clear();
        classNameList.clear();
		    rset = stmt.executeQuery (sRes.getString("selectBeanClasses"));
        while (rset.next()) {                   // Check stored classes
          Integer k = new Integer( rset.getInt(1));
          String name = rset.getString(2);
          BeanKeyCall classAdapter = new BeanKeyCall(k.intValue(), name, rset.getByte(3));
          boolean keyExist = existClass( k );
          if (! keyExist) {
            classAdapter.setTableExist();
            classKeyList.put( k, classAdapter );
            classNameList.put( name, classAdapter );
          }
          if (debug) logMsg( this.getClass().getName(), "connectDatabase#0", LogMsg.LOG_DEBUG, ! keyExist, name);
        }
        return true;
	    } catch (Exception e) {
        return false;
      }
    }

    /**
     * Does the connection to the database and creates it if nessecary.
     */
    protected void connectDatabase (String driverClass, String connStr, String connUser, String connPwd, String localDB, boolean runLocal, ResourceBundle sRes) throws Exception {
      this.driverClass = driverClass;
      this.connStr = connStr;
      this.connUser = connUser;
      this.connPwd = connPwd;
      this.localDB = localDB;
      this.runLocal = runLocal;
      this.sRes = sRes;
      conn = reconnect();
      boolean do_setup = true;
      if (!create_db)
        do_setup = ! init();
      if (do_setup) {
		    Skid.out("dbSetup");
        try {
          try {
            Enumeration keys = classKeyList.keys();
            while (keys.hasMoreElements()) {
              String[] args = new String[] { ((BeanKeyCall) classKeyList.get(keys.nextElement())).getTableName() };
              dropKey.exec(args, "setup.dropKeys#" + (String) args[0]);
            }
        } catch (Exception ex) { }
		      SkidSetup.createDataBase(sRes, conn);		// Do the DB-Setup
        } catch (SQLException f) {                // failed: create statement
          System.err.println(LogMsg.LOG_ERROR + " Database setup failed: " + f.toString());
          logMsg( this.getClass().getName(), "connectDatabase#1", LogMsg.LOG_ERROR, true,
                     "Database setup failed: ("+connStr+") Driver ("+driverClass+")\n\n"+
                     f.toString());
          throw f;
        }
        if (init())
          logMsg( this.getClass().getName(), "connectDatabase#2", LogMsg.LOG_WARNING, true,
                     "Database ("+connStr+") Driver ("+driverClass+") created.");
        else throw new SkidException("BeanBroker initialisation failed.");
      }
      try {                                   // Set up the STATEMENTS
        selectClass = new BeanBrokerCall(sRes.getString("callSelectClass"), sRes.getString("SelectClass"), null, true);
        insertClass = new BeanBrokerCall(sRes.getString("callInsertClass"), sRes.getString("InsertClass"), null, false);
        insertBean = new BeanBrokerCall(sRes.getString("callInsertBean"), sRes.getString("InsertBean"), null, false);
        updateBean = new BeanBrokerCall(sRes.getString("callUpdateBean"), sRes.getString("UpdateBean"), null, false);
        selectBean = new BeanBrokerCall(sRes.getString("callSelectBean"), sRes.getString("SelectBean"), null, true);
        insertLink = new BeanBrokerCall(sRes.getString("callInsertLink"), sRes.getString("InsertLink"), null, false);
        updateLink = new BeanBrokerCall(sRes.getString("callUpdateLink"), sRes.getString("UpdateLink"), null, false);
        selectLink = new BeanBrokerCall(sRes.getString("callSelectLink"), sRes.getString("SelectLink"), null, true);
        selectLink2 = new BeanBrokerCall(sRes.getString("callSelectLink2"), sRes.getString("SelectLink2"), sRes.getString("createLINK"), true);
        createKey  = new BeanBrokerCall(sRes.getString("callCreateKey"), sRes.getString("CreateKey"), null, false);
        deleteKey  = new BeanBrokerCall(sRes.getString("callDeleteKey"), sRes.getString("DeleteKey"), null, false);
        insertKey  = new BeanBrokerCall(sRes.getString("callInsertKey"), sRes.getString("InsertKey"), null, false);
        selectKey  = new BeanBrokerCall(sRes.getString("callSelectKey"), sRes.getString("SelectKey"), null, true);
        dropKey  = new BeanBrokerCall(sRes.getString("callDropKey"), sRes.getString("DropKey"), null, false);
        BeanKeyCall.setCalls(createKey, deleteKey, insertKey, selectKey, dropKey);
        insertEvent = new BeanBrokerCall(sRes.getString("callInsertEvent"), sRes.getString("InsertEvent"), null, false);
        updateEvent = new BeanBrokerCall(sRes.getString("callUpdateEvent"), sRes.getString("UpdateEvent"), null, false);
        deleteEvent = new BeanBrokerCall(sRes.getString("callDeleteEvent"), sRes.getString("DeleteEvent"), sRes.getString("createEVENT"), false);
        selectEvent = new BeanBrokerCall(sRes.getString("callSelectEvent"), sRes.getString("SelectEvent"), sRes.getString("createEVENT"), true);
      } catch (SQLException g) {              // failed: create statement
        System.err.println(LogMsg.LOG_ERROR + " Database setup failed: " + g.toString());
        logMsg( this.getClass().getName(), "connectDatabase#3", LogMsg.LOG_ERROR, true,
                   "Database setup failed: ("+connStr+") Driver ("+driverClass+") "+
                   g.toString());
        throw g;
      }
    }

    /**
     * returns a new key for the Bean incounting the running score.
     * Don't waste them!
     *@throws SkidException if all the key's are already in use
     */
    private int getNewRunningNr() throws SkidException {
      running_broker_nr++;
      if (running_broker_nr >= BeanBroker.ID_HIGH)
        throw new SkidException("All keys for this Broker-ID already in use!");
      return running_broker_nr + broker_id * BeanBroker.ID_HIGH;
    }

    /**
     * Returns whether a Class is registered by the Broker or not.
     */
    public static boolean existClass(Integer classKey) {
      return classKeyList.containsKey( classKey );
    }
    /**
     * Class-Key of the Bean, differ between different classes.
     *@param classname the name of the class
     */
    public int getClassKey(String classname) {
      BeanKeyCall classAdapter = (BeanKeyCall) classNameList.get(classname);
      if (debug) logMsg( this.getClass().getName(), "getClassKey#0", LogMsg.LOG_DEBUG, true, classname+ " found="+String.valueOf(classAdapter != null));
      Integer k = null;
      if (classAdapter == null)                 // check if new
      try {
        selectClass.exec(new Object[] {classname}, "getClassKey#1");
        if (selectClass.next()) {
          k = new Integer(selectClass.getInt(1));
          classAdapter = new BeanKeyCall(k.intValue(), classname, (byte) selectClass.getInt(3));
        } else {
          byte beantype = ((RemoteBean) Class.forName(classname).newInstance()).getBeanType();
          int p = getNewRunningNr();
          k = new Integer(p);
          insertClass.exec(new Object[] {String.valueOf(p), classname, String.valueOf(beantype)}, "getClassKey#2");
          classAdapter = new BeanKeyCall(p, classname, beantype);
        }
        boolean keyExist = existClass( k );
        if (! keyExist) {
          classKeyList.put( k, classAdapter );
          classNameList.put( classname, classAdapter );
        }
        if (debug) logMsg( this.getClass().getName(), "getClassKey#3", LogMsg.LOG_DEBUG, k!=null, String.valueOf(classAdapter)+ " classKey="+String.valueOf(k));
        return k.intValue();
      } catch (Exception e) {
        if (debug) logMsg( this.getClass().getName(), "getClassKey#4", LogMsg.LOG_DEBUG, false, e.toString());
        return ID_FAIL;
      } else
        if (debug) logMsg( this.getClass().getName(), "getClassKey#5", LogMsg.LOG_DEBUG, true, String.valueOf(classAdapter)+ " classKey="+String.valueOf(classAdapter.getBeanClassKey()));
        return classAdapter.getBeanClassKey();
    }

    /**
     * Class-Key of the Bean, differ between different classes.
     *@param aBean a Bean asked for the class-id
     */
    public int getClassKey(RemoteBean aBean) {
      return getClassKey(aBean.getClass().getName());
    }

    /**
     * Class-Key of the Bean, differ between different classes.
     *@param aClass the Bean-Class asked for the class-id
     */
    public int getClassKey(Class aClass) {
      return getClassKey(aClass.getName());
    }

    /**
     * Sets the serial-number, checks the corresponding key and certifies the client.
     * Without a valid key there are no writing operations possible.
     *@param	nr	serialnumber
     *@param	xKey	encrypted key in HEX-Format, empty keys are invalid
     */
    private final boolean certifyClient( long nr, String xKey) throws Exception {
      keyValid = false;
      if ((xKey != null) && (xKey.length()>0)) {                  // DECRYPTING THE KEY
        String clearPwd = "";
        byte[] inbuf = new byte[xKey.length()/2];
        for (int i=0;i < inbuf.length; i++) {
          inbuf[i] = (byte) (tab.indexOf(xKey.charAt(i*2))*16 +
                      tab.indexOf(xKey.charAt(i*2+1)));           // using asgaard.utils.cryptography.IDEA algotithm
        }
		    crypt.setKey( key );
        crypt.decrypt(inbuf);
        clearPwd = new String("_"+String.valueOf(nr)+"_"+crypt.decryptFinish());
  	    Statement stmt = conn.createStatement ( );
        // This statement is intended to be here, just to increase security a little bit.
	      ResultSet rset = stmt.executeQuery("select k from cKey where (nr = " + String.valueOf(nr)+") ");
	      rset.next();
//XXX !!! PASSWORTENTSCHLSSELUNG HAUT NICHT HIN !!! XXX
//        if (clearPwd.hashCode() == rset.getLong( 1 )) {
          broker_id = (int) nr;
          keyValid = true;
//        }
      }
  	  return keyValid;
    }

    /**
     * Puts the Bean into the Chache and saves it <b>do not use manually</b>.
     * Checks the Timestamps according to a update-policy given by the Bean and
     * saves al modifications permanently.
     * <b><i>To enforce a Bean's saving use the Bean's saving method.</i> <code>flush</code></b>
     *<p>
     * <i>Remark:</i> A System-Object cannot be modified and carries the created-Timestamp 0
     *
     *@see asgaard.skid.RemoteBean#saveflush
     *@param aObj the calling Bean
     *@return null if the update was not possible or [key, classkey and modified if successfull]
     *@throws Exception if an error occoured during the writeback.
     */
    public Object[] putBean(RemoteBean aObj) throws Exception {
      try {
        if ((aObj == null) || (broker_id == ID_FAIL))           // No Object present
          return null;
        if (! aObj.isChanged())                                 // Nothing to do
          return null;
        if (aObj.isNew())
          aObj.init(getNewRunningNr(), getClassKey(aObj), this);     // Set the keys
        else
          aObj.init(aObj.getKey(), aObj.getClassKey(), this);
        if (debug) logMsg( this.getClass().getName(), "putBean#0", LogMsg.LOG_DEBUG, true, "isNew="+String.valueOf(aObj.isNew()));
        RemoteBean oObj = getBean( aObj.getKey() );             // Search for the Bean
        boolean newBean = oObj == null;
        if (! newBean) {
          if (! ((newBean) || (oObj == aObj))) {
            switch (aObj.getUpdatePolicy()) {
              case UPDATE_NONE : return null;
              case UPDATE_ALL : break;
              case UPDATE_WEAK :
                if (oObj.getModified() > aObj.getModified())    // WEAK
                  return null; break;                          // exit if ts(old) > ts(new)
              default :
                if (oObj.getAge() != aObj.getAge())             // STRONG
                  return null; break;                          // exit if ts(old) != ts(new)
            }
          }
/*----------------------------------------------------------------------------------------------
// SOME KIND OF ERROR HANDLING IF THE HASH-KEY FAILS AND OVERRIDES AN OLDER BUT DIFFERENT OBJECT
        if (getClassKey( oObj ) != getClassKey( aObj ))
          logMsg(this.getClass().getName(), "putBean#1", LogMsg.LOG_ERROR, true,
              "Class overwrite! old cid=" + String.valueOf(getClassKey( oObj ))+
                            ", new cid="+String.valueOf(getClassKey( aObj )), oObj);
        if (oObj.getCreated() != aObj.getCreated())
          logMsg(this.getClass().getName(), "putBean#2", LogMsg.LOG_WARNING, true,
              "Class overwrite! old created=" + String.valueOf(oObj.getCreated())+
                            ", new created="+String.valueOf(aObj.getCreated()), oObj);
// SOME KIND OF ERROR HANDLING IF THE HASH-KEY FAILS AND OVERRIDES AN OLDER BUT DIFFERENT OBJECT
//----------------------------------------------------------------------------------------------*/
        }
        if (aObj.isReadOnly())                                  // write-protected or System Object
          return null;
        if (oObj != aObj)
          beanCache.put( new Integer(aObj.getKey()), aObj );    // to the chache
        Object[] args = new Object[8];
        args[0] = String.valueOf( aObj.getKey() );
        args[1] = String.valueOf( getClassKey( aObj ));
        args[2] = String.valueOf( aObj.getCreated() );
        args[3] = String.valueOf( aObj.getModified() );
        args[4] = aObj.getIdentifier();
        args[5] = aObj.getName();
        args[6] = String.valueOf( aObj.getBeanType());
        boolean isLightWeight = false;
        switch (aObj.getBeanType()) {
          case BeanBroker.TYPE_LIGHTWEIGHT: case BeanBroker.TYPE_CLASS: {
            isLightWeight = true;
            args[7] = aObj.invoke("getObject", new Object[0], false);
            break;
          }
          default: {                                            // normal Obj.
            args[7] = aObj;
            break;
          }
        }
        if (! newBean)                                          // to the database
          updateBean.exec( args, "BeanBroker.putBean#3" );
        else
          try {
            insertBean.exec( args, "BeanBroker.putBean#4" );
          } catch (Exception e) {
            if (debug) logMsg( this.getClass().getName(), "putBean#4a", LogMsg.LOG_DEBUG, false, e.toString());
            updateBean.exec( args, "BeanBroker.putBean#5" );
          }
        try {
          aObj.setAge();                                          // KEYS
          if (!isLightWeight)
            getBeanKeyCall( aObj.getClassKey() ).updateKeys( aObj );
        } catch (Exception f) {
            logMsg( aObj.getClass().getName(), "BeanBroker.putBean#6", LogMsg.LOG_ERROR, false,
                       "updateKeys failed: " + f.toString());
        }
        return new Object[] {new Integer(aObj.getKey()), new Integer(aObj.getClassKey()), new Long(aObj.getModified())};
      } catch (Exception g) {
          if (debug) logMsg( this.getClass().getName(), "putBean#7", LogMsg.LOG_DEBUG, false, g.toString());
          throw g;
      }
    }

    /**
     * Returns the Key-Call-Adapter for a asked Class (represented by it's key).
     *@param classkey the id of the class
     *@throws Exception if the key's couldn't be mapped
     */
    public BeanKeyCall getBeanKeyCall(Class aClass) throws Exception {
      return getBeanKeyCall( getClassKey( aClass ));
    }

    /**
     * Returns the Key-Call-Adapter for a asked Class (represented by it's key).
     *@param classkey the id of the class
     *@throws Exception if the key's couldn't be mapped
     */
    public BeanKeyCall getBeanKeyCall(int pClass) throws Exception {
      return (BeanKeyCall) classKeyList.get( new Integer( pClass ));
    }

    /**
     * Searches for a Bean by a given key.
     *@param aClass the searched class
     *@param paramName list of the parameter to search for
     *@param searchValues values to fit
     *@param search-operator
     *@return a set of Bean as result of the search
     *@throws Exception if anything fails
     */
    public int[] getBeanKey(String aClass, String[] paramName, Object[] searchValue, byte[] searchOp) throws SkidException {
      if ((aClass==null) ||(paramName==null) || (searchValue==null) ||  (searchOp==null))
        throw new SkidException ("Invalid parameter: (null)");
      if ((paramName.length != searchValue.length) ||  (searchOp.length != searchValue.length))
        throw new SkidException ("Invalid parameter size");
      if (debug) logMsg( this.getClass().getName(), "getBeanKey#0", LogMsg.LOG_DEBUG, aClass!=null, aClass);
      try {
        RemoteBean aBean = (RemoteBean) Class.forName(aClass).newInstance();
        SkidBeanFilter F = aBean.getFilter();
        for (int i=0;i<paramName.length;i++) {
          if (debug) logMsg( this.getClass().getName(), "getBeanKey#1", LogMsg.LOG_DEBUG, true, "TRY:"+paramName[i]+"~"+String.valueOf(searchValue[i]));
          F.setFilterEntry(paramName[i], searchValue[i], searchOp[i] );
        }
        int[] k = F.getResultKeys();
        if (debug) logMsg( this.getClass().getName(), "getBeanKey#2", LogMsg.LOG_DEBUG, k!=null, String.valueOf(k));
        return k;

      } catch (Exception e) {
          logMsg( this.getClass().getName(), "getBeanKey#3", LogMsg.LOG_ERROR, false, e.toString());
          return new int[0];
      }
    }

    /**
     * Searches for a Bean by a given key.
     *@param key the primary key of the object
     *@return the searched Bean if the key exist
     *@throws Exception if anything fails
     */
    public RemoteBean getBean(int key) throws Exception {
      if ((key == 0)  || (broker_id == ID_FAIL))
        return null;
      RemoteBean oObj = (RemoteBean) beanCache.get(new Integer(key));
      if (oObj == null) {
        Object[] args = new Object[1];
        args[0] = String.valueOf( key );
        selectBean.exec(args, "BeanBroker.getBean#1");
        try {
          if (selectBean.next()) {
            switch (selectBean.getInt( 7 )) {
              case BeanBroker.TYPE_CLASS: {
                Def classBean = new Def();
                classBean.init(selectBean.getInt( 1 ), selectBean.getInt( 2 ), selectBean.getString(6), selectBean.getString(8), selectBean.getLong(3), selectBean.getLong(3));
                classBean.setAge();
                oObj = classBean;
                break;
              }
              case BeanBroker.TYPE_LIGHTWEIGHT: {
                BeanKeyCall classAdapter = getBeanKeyCall( selectBean.getInt( 2 ));
                if (debug) logMsg( this.getClass().getName(), "getBean#2", LogMsg.LOG_DEBUG, classAdapter!=null, "LIGHTWEIGHT BEAN");
                LightBean lightweightBean = (LightBean) Class.forName(classAdapter.getBeanClassName()).newInstance();
                lightweightBean.init(selectBean.getInt( 1 ), selectBean.getInt( 2 ), selectBean.getString(6), selectBean.getString(8), selectBean.getLong(3), selectBean.getLong(3));
                lightweightBean.setAge();
                oObj = lightweightBean;
                break; }
              default: {                                                // a "real" object ...
                if (debug) logMsg( this.getClass().getName(), "getBean#3", LogMsg.LOG_DEBUG, true, "SKIDBEAN");
                oObj = (RemoteBean) selectBean.getObject(8, true);
                break;
              }
            }
            if (debug) logMsg( this.getClass().getName(), "getBean#4", LogMsg.LOG_DEBUG, oObj!=null, String.valueOf(oObj));
            if (oObj != null)
              beanCache.put( new Integer(oObj.getKey()), oObj );        // to the chache
          }
        } catch (Exception e) {
          if (debug) logMsg( this.getClass().getName(), "getBean#5", LogMsg.LOG_DEBUG, false, e.toString());
          throw e;
        }
      }
      return oObj;
    }

    /**
     * Searches for Beans by a given set of keys. Simply calles the primitive getBean method.
     *@param key the primary key of the object
     *@return the searched Bean if the key exist
     *@throws Exception if anything fails
     */
    public RemoteBean[] getBean(int[] key) throws Exception {
      RemoteBean[] oObj = new RemoteBean[key.length];
      for (int i=0;i<key.length;i++)
        oObj[i] = getBean(key[i]);
      return oObj;
    }

    /**
     * Returns whether an Bean is available within the Broker or not.
     *@param key the unique key of the Bean
     *@return true for an existing Bean with the given key
     */
    public boolean beanExist(int key) {
      try {
        return (getBean(key) != null);
      } catch (Exception e) {
        if (debug) {
          System.err.println(e.toString());
          e.printStackTrace();
        }
        return false;
      }
    }

    /**
     * Adds a new Relation between Beans.
     *@param link one relation
     *@return wether an update is possible or not.
     *@throws Exception if the key is not valid or anything else fails
     */
    public boolean putLink(BeanLink link) throws Exception {
      if ((link == null)  || (broker_id == ID_FAIL))
        return false;
      if (! link.isChanged())                               // Nothing to do
        return false;
      BeanLink oObj = getLink(link.From, link.To, link.By); // search for link
      if (! ((oObj == null) || (oObj == link))) {
        switch (link.update_policy) {
          case UPDATE_ALL : break;
          case UPDATE_WEAK :
            if (oObj.Modified > link.Modified)              // WEAK
              return false; break;                          // exit if ts(old) > ts(new)
          default :
            if (oObj.Age != link.Age)                       // STRONG
              return false; break;                          // exit if ts(old) != ts(new)
        }
        if ((link.To == 0) ||                               // check if keys exist
            (! beanExist(link.From)) ||
            (! beanExist(link.To)) ||
            (! beanExist(link.By)) )
          throw new SkidException("Key not valid");
      }
      if (oObj != link)
        linkCache.put(link.getKey(), link);
      Object[] args = new Object[9];                        // create Keyparts
      args[0] = String.valueOf( link.From );
      args[1] = String.valueOf( link.Order );
      args[2] = String.valueOf( link.To );
      args[3] = String.valueOf( link.By );
      args[4] = String.valueOf( link.Created );
      args[5] = String.valueOf( link.Modified );
      args[6] = String.valueOf( link.Begin );
      args[7] = String.valueOf( link.End );
      args[8] = String.valueOf( link.State );
      if (oObj != null)                                     // to the database
        updateLink.exec( args, "BeanLink.put#1" );
      else
        try {
          insertLink.exec( args, "BeanLink.put#2" );
        } catch (Exception e) {
          updateLink.exec( args, "BeanLink.put#3" );
        }
      link.setAge();
      return true;
    }

    /**
     * Searches for a specific Link using it's primary key.
     *@param from key of the origin Bean
     *@param to key of the target Bean
     *@param  by key of the link-Bean (kind of Link)
     *@return the link if found
     *@throws Exception if anything goes wrong.
     */
    public BeanLink getLink(int from, int to, int by) throws Exception {
      BeanLink aLink = (BeanLink) linkCache.get(BeanLink.getKey(from, to, by));
      if (aLink == null) {
        Object[] args = new Object[9];                  // create Keyparts
        args[0] = String.valueOf( from );
        args[1] = null;
        args[2] = String.valueOf( to );
        args[3] = String.valueOf( by );
        args[4] = null;
        args[5] = null;
        args[6] = null;
        args[7] = null;
        args[8] = null;
        selectLink.exec(args, "BeanLink.get#1");
        if (selectLink.next()) {
          aLink = new BeanLink ( selectLink.getInt(1),
                                  selectLink.getInt(2),
                                  selectLink.getInt(3),
                                  selectLink.getInt(4),
                                  selectLink.getInt(5),
                                  selectLink.getInt(6),
                                  selectLink.getInt(7),
                                  selectLink.getInt(8),
                                  selectLink.getInt(9));
          linkCache.put( aLink.getKey(), aLink);
        }
      }
      return aLink;
    }

    /**
     * Searches for a set of Links using the filter - capabilities of
     * <code>BeanLink</code>.
     *@param filter the BeanLink used as Filter
     *@return the links if any found
     *@throws Exception if anything goes wrong.
     */
    public BeanLink[] getLink(BeanLinkFilter filter) throws Exception {
      Object[] args = new Object[1];                  // create the where clause
      String where = selectLink2.getFilterClause( (byte) 0, filter.useFrom, new Integer(filter.From), "");
      where = selectLink2.getFilterClause( (byte) 1, filter.useOrder, new Integer(filter.Order), where);
      where = selectLink2.getFilterClause( (byte) 2, filter.useOrder, new Integer(filter.To), where);
      where = selectLink2.getFilterClause( (byte) 3, filter.useOrder, new Integer(filter.By), where);
      where = selectLink2.getFilterClause( (byte) 4, filter.useOrder, new Long(filter.Created), where);
      where = selectLink2.getFilterClause( (byte) 5, filter.useOrder, new Long(filter.Modified), where);
      where = selectLink2.getFilterClause( (byte) 6, filter.useOrder, new Long(filter.Begin), where);
      where = selectLink2.getFilterClause( (byte) 7, filter.useOrder, new Long(filter.End), where);
      where = selectLink2.getFilterClause( (byte) 8, filter.useOrder, new Integer(filter.State), where);

      if (where.equals(""))
        where = selectLink2.getZeroClause((byte) 1);
      args[0] = where;
      selectLink2.exec(args, "BeanLink.get#2");
      Vector v = new Vector();
      while (selectLink2.next()) {
        BeanLink aLink = (BeanLink) linkCache.get(BeanLink.getKey(selectLink2.getInt(1),
                                                                  selectLink2.getInt(3),
                                                                  selectLink2.getInt(4)));
        if (aLink == null) {
          aLink = new BeanLink ( selectLink2.getInt(1),
                                  selectLink2.getInt(2),
                                  selectLink2.getInt(3),
                                  selectLink2.getInt(4),
                                  selectLink2.getInt(5),
                                  selectLink2.getInt(6),
                                  selectLink2.getInt(7),
                                  selectLink2.getInt(8),
                                  selectLink2.getInt(9));
          linkCache.put( aLink.getKey(), aLink);
        }
        v.addElement( aLink);
      }
      BeanLink[] res = new BeanLink[v.size()];
      for (int i=0;i<v.size();i++)
        res[i] = (BeanLink) v.elementAt(i);
      return res;
    }

    /**
     * Returns if a specific Key exists, hiding all Exceptions.
     *@param from key of the origin Bean
     *@param to key of the target Bean
     *@param  by key of the link-Bean (kind of Link)
     *@return the link if found
     */
    public boolean linkExist(int from, int to, int by) {
      try {
        return (getLink(from, to, by) != null);
      } catch (Exception e) {
        if (debug) {
          System.err.println(e.toString());
          e.printStackTrace();
        }
        return false;
      }
    }

    /**
     * Adding an Event and replicating it to all suscribed Objects.
     *@param producer of the event (not <code>null</code>)
     *@param event the kond of the event (not <code>null</code>)
     *@throws Exception if it doesen't work
     */
    public void addEvent(RemoteBean producer, RemoteBean event) throws Exception {
      Object[] args = new Object[1];
      String clause = selectEvent.getFilterClause((byte) 0, FILTER_EQ, new Integer( producer.getKey() ), "");
      clause = selectEvent.getFilterClause((byte) 2, FILTER_EQ, new Integer( event.getKey() ), clause);
      clause = selectEvent.getFilterClause((byte) 3, FILTER_EQ, new Integer( SkidEvent.EVENT_PATTERN ), clause);
      args[0] = clause;
      selectEvent.exec(args, "BeanBroker.addEvent#1");
      String clause0 = String.valueOf( producer.getKey());
      String clause1 = BeanBrokerCall.addListClause(String.valueOf( event.getKey() ));
      clause1 += BeanBrokerCall.addListClause(String.valueOf( SkidEvent.EVENT_UNREAD ));
      clause1 += BeanBrokerCall.addListClause(String.valueOf( System.currentTimeMillis() ));
      while (selectEvent.next()) {
        args[0] = clause0 + BeanBrokerCall.addListClause( selectEvent.getString( 2 )) + clause1;
        insertEvent.exec(args, "BeanBroker.addEvent#2");
      }
    }

    /**
     * Changing the State of an specific event.
     *@param producer of the event (not <code>null</code>)
     *@param event the kond of the event (not <code>null</code>)
     *@param consumer of the event (not <code>null</code>)
     *@param fromState ols State of the event
     *@param toState ols State of the event
     *@throws Exception if it doesen't work
     */
    public void setEvent(RemoteBean producer, RemoteBean event, RemoteBean consumer, byte fromState, byte toState) throws Exception {
      Object[] args = new Object[5];
      args[0] = String.valueOf( toState );
      args[1] = String.valueOf( producer.getKey() );
      args[2] = String.valueOf( consumer.getKey() );
      args[3] = String.valueOf( event.getKey() );
      args[4] = String.valueOf( fromState );
      updateEvent.exec(args, "BeanBroker.setEvent#1");
    }

    /**
     * Adding an Event-Pattern and suscribing this type of event.
     *@param producer of the event (not <code>null</code>)
     *@param event the kond of the event (not <code>null</code>)
     *@param consumer of the event (not <code>null</code>)
     *@throws Exception if it doesen't work
     */
    public void suscribeEvent(RemoteBean producer, RemoteBean event, RemoteBean consumer) throws Exception {
      setEvent(producer, event, consumer, SkidEvent.EVENT_PATTERN, SkidEvent.EVENT_DELETED);
      Object[] args = new Object[1];
      String clause = String.valueOf( producer.getKey() );
      clause += insertEvent.addListClause(String.valueOf( consumer.getKey() ));
      clause += insertEvent.addListClause(String.valueOf( event.getKey() ));
      clause += insertEvent.addListClause(String.valueOf( SkidEvent.EVENT_PATTERN ));
      clause += BeanBrokerCall.addListClause(String.valueOf( System.currentTimeMillis() ));
      args[0] = clause;
      insertEvent.exec(args, "BeanBroker.addEventPattern#1");
    }

    /**
     * Deletes all marked Events
     *@throws Exception
     */
    protected void delEvent() throws Exception {
      Object[] args = new Object[1];
      args[0] = selectEvent.getFilterClause((byte) 3, FILTER_EQ, new Integer( SkidEvent.EVENT_DELETED ), "");
      deleteEvent.exec(args, "BeanBroker.delEvent#1");
    }

    /**
     * Deletes all events older than the given time
     *@param deadline oldest event allowed
     *@throws Exception
     */
    protected void delEvent(long deadline) throws Exception {
      Object[] args = new Object[1];
      args[0] = selectEvent.getFilterClause((byte) 4, FILTER_SM, new Long( deadline ), "");
      deleteEvent.exec(args, "BeanBroker.delEvent#2");
    }

    /**
     * Returns the Events for a consumer object which are newer than the given time
     *@param consumer of the events
     *@param newer than the given time
     *@returns a set of events
     *@throws Exception
     */
    public SkidEvent[] getEvent(RemoteBean consumer, long newer) throws Exception {
      Vector res = new Vector();
      Object[] args = new Object[1];
      String clause = selectEvent.getFilterClause((byte) 1, FILTER_EQ, new Integer( consumer.getKey() ), "");
      clause = selectEvent.getFilterClause((byte) 4, FILTER_GR, new Long( newer ), clause);
      clause = selectEvent.getFilterClause((byte) 3, FILTER_GR, new Integer( SkidEvent.EVENT_PATTERN ), clause);
      clause = selectEvent.getFilterClause((byte) 3, FILTER_SM, new Integer( SkidEvent.EVENT_DELETED ), clause);
      args[0] = clause;
      selectEvent.exec(args, "BeanBroker.getEvent#1");
      while (selectEvent.next()) {
        res.addElement(new SkidEvent( selectEvent.getInt( 1 ),
                                      selectEvent.getInt( 2 ),
                                      selectEvent.getInt( 3 ),
                                      (byte) selectEvent.getInt( 4 ),
                                      selectEvent.getLong( 5 ) ));
      }
      SkidEvent[] evSet = new SkidEvent[res.size()];
      for (int i=0;i<evSet.length;i++) {
        evSet[i] = (SkidEvent) res.elementAt( i );
      }
      return evSet;
    }

    /**
     * Checks if the given username exists or not
     *@param	userName name of the user
     *@param	encr the encrypted password (optional)
     *@returns the user's status if the username exists and else null
     *@exception    RemoteException	if RMI doesen't work
     *@exception    SQLException the db doesen't work
     */
    public final String checkUser( String userName, String encr ) throws SkidException, RemoteException, SQLException {
      if ((userName == null) || (userName.length() < 1))
        throw new SkidException("An empty username is not valid!");
      Statement stmt = conn.createStatement();
      // This statement is intended to be here, just to increase security a little bit.
      ResultSet rset = null;
      if (encr == null)
        rset = stmt.executeQuery("select * from user where (name = '" + userName+"') ");
      else
        rset = stmt.executeQuery("select * from user where (name = '" + userName+"') and (encr = '"+ encr+"') ");
      stmt = null;
      if (rset.next()) {
        return rset.getString(3);
      } else {
        return null;
      }
    }

    /**
     * Excanges an user with an authorised object (hardcoded)
     */
    public final Object getUser(Object o, String name, String encr) throws SkidException, RemoteException, SQLException {
      boolean retry = true;
      while (retry)
      try {
        retry = false;
        if (o instanceof String[]) {
          Statement stmt = conn.createStatement();
          String[] s = (String[]) o;
          if (s.length < 10)
            return null;
          String x = checkUser( name, encr );
          if ((x == null) || ( !s[0].equals(name) && (x.indexOf( UserSession.STATUS_ADMIN ) < 0)))
            return null;
          String query = "select * from user where (name = '" + s[0]+"') ";
          ResultSet rset = stmt.executeQuery(query);
          boolean ok = false;
          if (rset.next()) {
            s[1] = rset.getString(2);
            s[2] = rset.getString(3);
            s[3] = rset.getString(4);
            s[4] = rset.getString(5);
            s[5] = rset.getString(6);
            s[6] = rset.getString(7);
            s[7] = rset.getString(8);
            s[8] = rset.getString(9);
            s[9] = rset.getString(10);
            ok = true;
          }
          if (debug) logMsg( this.getClass().getName(), "getUser#0", LogMsg.LOG_DEBUG, ok, query);
          return s;
        } else
          return null;
      } catch (SQLException e) {
        try {
          reconnect();
        } catch (Exception f) { return null; }
      }
      return null;
    }

    /**
     * Excanges an user with an authorised object (hardcoded)
     */
    public final Object getUser(String name, String encr) throws SkidException, RemoteException, SQLException {
      try {
        Statement stmt = conn.createStatement();
        String x = checkUser( name, encr );
        if ((x == null) || (x.indexOf( UserSession.STATUS_ADMIN ) < 0)) {
          if (debug) logMsg( this.getClass().getName(), "getUser#1", LogMsg.LOG_DEBUG, x!=null, String.valueOf(x));
          return null;
        }
        ResultSet rset = stmt.executeQuery("select name from user ");
        Vector v = new Vector();
        while (rset.next())
          v.addElement( rset.getString(1) );
        if (debug) logMsg( this.getClass().getName(), "getUser#2", LogMsg.LOG_DEBUG, true, String.valueOf(v.size()));
        return v;
      } catch (SQLException e) {
        if (debug) logMsg( this.getClass().getName(), "getUser#3", LogMsg.LOG_DEBUG, false, e.toString());
        return null;
      }
    }

    /**
     * Excanges an user with an authorised object (hardcoded)
     */
    public final boolean putUser(Object o) throws SkidException, RemoteException, SQLException {
//    public final boolean putUser(Object o, String name, String encr) throws SkidException, RemoteException, SQLException {
      byte retry = 0;
      String query = "";
      while (retry < 1)
      try {
        if (o instanceof String[]) {
          Statement stmt = conn.createStatement();
          String[] s = (String[]) o;
          if (s.length < 10) {
            if (debug) logMsg( this.getClass().getName(), "putUser#1", LogMsg.LOG_DEBUG, false, String.valueOf(s.length));
            return false;
          }
//XXX Therefor a User may change himselfe - that is a security-hole.
//          if (! (s[0].equalsIgnoreCase(name) && (s[1].equals(encr)))) {
//            String x = checkUser( name, encr );
//            if ((x == null) || (x.indexOf( UserSession.STATUS_ADMIN ) < 0 )) {
//              if (debug) logMsg( this.getClass().getName(), "putUser#2", LogMsg.LOG_DEBUG, x!=null, String.valueOf(x));
//              return false;
//            }
//          }
          if (checkUser( s[0], null ) != null) {     // User does exist
            query = "DELETE FROM user WHERE name='"+s[0]+"' ";
            stmt.executeUpdate(query);
          }
          query = "INSERT INTO user VALUES ( '" + s[0] +
                                              "', '" + s[1] +
                                              "', '" + s[2] +
                                              "', " + s[3] +
                                              ", '" + s[4] +
                                              "', '" + s[5] +
                                              "', '" + s[6] +
                                              "', " +  s[7] +
                                              ", " +  s[8] +
                                              ", " +  s[9] + " ) ";
          stmt.executeUpdate(query);
          logMsg( this.getClass().getName(), "putUser#3", LogMsg.LOG_BACKUP, true, query);
          return true;
        } else {
          if (debug) logMsg( this.getClass().getName(), "putUser#4", LogMsg.LOG_DEBUG, o!=null, String.valueOf(o));
        }
      } catch (SQLException e) {
        if (debug) logMsg( this.getClass().getName(), "putUser#5", LogMsg.LOG_DEBUG, false, query + "\n" + e.toString());
        try {
          reconnect();
          retry++;
        } catch (Exception f) {
          if (debug) logMsg( this.getClass().getName(), "putUser#6", LogMsg.LOG_DEBUG, false, f.toString());
        }
      }
      return false;
    }

  /**
   * Invokes the called method of the filterinterface. The BeanKeyCall-Objects are cached by

   * the BeanBroker and available via this one-method interface

   *
   *@param beanClass the class which the asked BeanKeyCall object is for
   *@param name the name of the Method
   *@param args the Arguments
   *@return a result returned (if any)
   *@see java.lang.reflect.Method#invoke
   *@throws IllegalAccessException if the underlying method is inaccessible.

   *@throws IllegalArgumentException if the number of actual and formal parameters differ, or if an unwrapping conversion fails.
   *@throws InvocationTargetException if the underlying method throws an exception.
   *@throws: NullPointerException if the specified object is null.
   *@throws NoSuchMethodException if there is no Method with this name
   */
  public Object invokeCall(Class beanClass, String name, Object args[], String[] classnames) throws Exception {

    BeanKeyCall call = getBeanKeyCall(beanClass);       // look for the call

    if (args == null)

      args = new Object[0];

    Class[] params = new Class[args.length];

    for (int i=0;i<args.length;i++) {

      if ((classnames == null) || (classnames.length <= i))

        params[i] = args[i].getClass();

      else

        params[i] = Test.forName(classnames[i], false);

    }

    if ((name == null) || (name.length() == 0))

      return call.getClass().getConstructor(params).newInstance(args);

    else

      return call.getClass().getMethod(name, params).invoke(call, args);

  }

  /**
   * Returns the Test-Script for this Class
   */
  public Object[][] getTestScript() {
    return Test.concatScripts(new Object[] { TestSkid.TESTSCRIPT_BEANBROKER_INTERFACE, TestSkid.TESTSCRIPT_BEANBROKER_FINAL });
  }

  /**
   *  Switches the test - mode on / off. All changes at the Database between on and off
   * will be deleted.
   */
  public void testMode(boolean on) {
    if (on) {
      broker_id2 = broker_id;
      running_broker_nr2 = running_broker_nr;
      broker_id = PUBLIC_ID;
      running_broker_nr = 0;
      classKeyList2 = classKeyList;
      classKeyList = new Hashtable();
      classNameList2 = classNameList;
      classNameList = new Hashtable();
      try { init(); } catch (Exception e) { }
      logMsg( this.getClass().getName(), "testMode#1", LogMsg.LOG_REMARK, true, "TESTMODE ON");
    } else {
      broker_id = broker_id2;
      running_broker_nr = running_broker_nr2;
      broker_id2 = ID_FAIL;
      running_broker_nr2 = 0;
      classKeyList = classKeyList2;
      classKeyList2 = null;
      classNameList = classNameList2;
      classNameList2 = null;
      String del = "";
      boolean ok = false;
      try {
        del = "Beans";
  	  	conn.createStatement().executeUpdate (sRes.getString("deleteTestBeans"));
        del = "Links";
  	  	conn.createStatement().executeUpdate (sRes.getString("deleteTestLinks"));
        del = "Events";
  	  	conn.createStatement().executeUpdate (sRes.getString("deleteTestEvents"));
        System.out.println("Database cleaned.");
        ok = true;
      } catch (Exception e) {
        System.out.println("Could not clean Databasetable ["+del+"]: " + e.toString());
      }
      logMsg( this.getClass().getName(), "testMode#2", LogMsg.LOG_REMARK, ok, "TESTMODE OFF");
    }
  }
  /**
   *  Performs some Tests on the living Object
   */
  public boolean testIt(Object local_handle) {
    testMode(true);
    boolean ret = Test.testIt(this.getClass().getName(), getTestScript(), local_handle);
    testMode(false);
    return ret;
  }

  /**
   * Puts a Message to the LOG-Server if available and returns whether it was sent or not
   *@see asgaard.utils.log.LogMsg
   *@param server     The calling Java-Serverclass
   *@param point      The Method / place where the statement was created
   *@param typ        The Type of the LOG-Statement
   *@param ok         Was the ServicePoint passed successfuly
   *@param msg        Human redable Message of the Statement
   */
  public boolean logMsg(String server, String point, byte typ, boolean ok, String msg) {
    if ((debug || (log_level >= typ)) && (log != null)) {
      log.put( server, point, typ, ok,msg);
      return true;
    } else
      return false;
  }

  /**
   * Puts a Message to the LOG-Server if available and returns whether it was sent or not
   *@see asgaard.utils.log.LogMsg
   *@param server     The calling Java-Serverclass
   *@param point      The Method / place where the statement was created
   *@param typ        The Type of the LOG-Statement
   *@param ok         Was the ServicePoint passed successfuly
   *@param msg        Human redable Message of the Statement
   */
  public boolean logMsg(String server, String point, byte typ, boolean ok, String msg, Object bean) {
    if ((debug || (log_level >= typ)) && (log != null)) {
      log.put( server, point, typ, ok, msg, bean);
      return true;
    } else
      return false;
  }

}

















