// This is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 060599
//
//@created	060599  KH
package asgaard.skid;
// Import used asgaard-parts
import asgaard.utils.log.*;
// Import the SQL-classes
import java.sql.*;
// Import used Java classes
import java.util.*;
import java.lang.reflect.*;

/**
 * This class maps the keys of an Bean to the Database.
 *@see asgaard.skid.SkidBean
 *@see asgaard.skid.BeanBrokerCall
 */
public class BeanKeyCall {
  public static final byte REKURSE = -2;
  public static final byte BEAN = -1;
  public static final byte OTHER = 0;
  public static final byte BYTE = 1;
  public static final byte SHORT = 2;
  public static final byte INT = 3;
  public static final byte LONG = 4;
  public static final byte FLOAT = 5;
  public static final byte DOUBLE = 6;
  public static final byte DATE = 7;
  public static final byte BOOLEAN = 8;

  private static final boolean debug = true;

  private static BeanBrokerCall createKey;
  private static BeanBrokerCall deleteKey;
  private static BeanBrokerCall insertKey;
  protected static BeanBrokerCall selectKey;
  protected static BeanBrokerCall dropKey;

  private BeanKeyCall parentClassCall = null;

  private int p = BeanBroker.ID_FAIL;
  private String name = null;
  private byte beantype = BeanBroker.ID_FAIL;
  private boolean tableExist = false;
  private Class beanclass = null;

  protected String tableName = null;
  protected boolean isBean = false;
  protected boolean isRoot = true;
  protected int superCount = 0;
  protected Method[] methods;
  protected int[] fieldnr;
  protected byte[] types;
  protected String[] fieldNames;
  protected boolean[] array;
  protected BeanKeyCall[] rekurse;

  /**
   * Sets the handles to the call-Statements
   *@param create call of the database
   *@param delete call of the database
   *@param insert call of the database
   *@param select call of the database
   */
  public static void setCalls (BeanBrokerCall create, BeanBrokerCall delete, BeanBrokerCall insert, BeanBrokerCall select, BeanBrokerCall drop) {
    createKey = create;
    deleteKey = delete;
    insertKey = insert;
    selectKey = select;
    dropKey = drop;
  }

  /**
   * Creates an empty adapter between a Bean(class) and the Database
   */
  public BeanKeyCall(int p, String name, byte beantype ) throws ClassNotFoundException {
    super();
    this.p = p;
    this.name = name;
    setBeanclass(Class.forName(name));
    try {
      setTableName(BeanBrokerCall.getTableName( p )); // AFTER setBeanClass !!!
    } catch (Exception ex) { }
    this.beantype = beantype;
  }

  /**
   * Creates an adapter between a Bean(class) and the Database and the
   * corresponding physical table with the Keys.
   *@param aClass the new Class to map
   *@throws Exception if the keys couldn't be adapted
   */
  public BeanKeyCall(Class aClass) throws Exception {
    this(BeanBroker.THIS.getClassKey( aClass ), aClass.getName(), ((RemoteBean) aClass.newInstance()).getBeanType() );
    analyzeBean(beanclass);
  }


  private void setBeanclass(Class aClass) {
    if (beanclass == null)
      beanclass = aClass;
  }
  private void setTableName(String name) {
    if (tableName == null)
      tableName = name;
  }
  /**
   * shortcut: Tells that the table currently exists
   */
  void setTableExist() {
    tableExist = true;
  }

  /**
   * Creates an adapter between a Bean(class) and the Database and the
   * corresponding physical table with the Keys.
   *@param aClass the new Class to map
   *@param tablename name of the table
   *@throws Exception if the keys couldn't be adapted
   */
  private BeanKeyCall(Class aClass, String tablename) throws Exception {
    setBeanclass(aClass);
    setTableName(tablename);
    analyzeBean(beanclass); // AFTER set*
  }

  /**
   * Returns the class-key of the to this adapter corresponding Bean-class
   */
  public int getBeanClassKey() {
    return p;
  }

  /**
   * Returns the class-name of the to this adapter corresponding Bean-class
   */
  public String getBeanClassName() {
    return name;
  }

  /**
   * Returns the beantype of the to this adapter corresponding Bean-class
   */
  public int getBeanClassType() {
    return beantype;
  }

  /**
   * Returns the set of key-names of the adapted Bean (if any).
   *@return an array of String
   */
  public String[] getFieldNames() {
    return fieldNames;
  }

  /**
   * Returns the Index of a given property-name
   *@returns the fieldindex or -1 if seek failed
   *@param fieldname the name of the property (without "get")
   */
  public Integer getFieldIdx(String fieldname) {
    int i = 0;
    while ((i < fieldNames.length) && (! fieldNames[i].equalsIgnoreCase(fieldname)))
      i++;
    if (fieldNames[i].equalsIgnoreCase(fieldname))
      return new Integer(i);
    else
      return new Integer(-1);
  }

  /**
   * Returns the FieldNr to a corresponding index.
   *@param index the searched index
   *@returns the fieldnumber
   */
  public Integer getFieldNr(Integer index) {
    return new Integer(fieldnr[index.intValue()]);
  }

  /**
   * Returns the name of the Table
   */
  public String getTableName() {
    return tableName;
  }
  
  /**
   * Returns if this class is the RemoteBean - rootclass
   *@return true if the class is the root
   */
  public Boolean isRoot() {
    return new Boolean(this.isRoot);
  }

  /**
   * Returns if this class implements the RemoteBean
   *@return true if the class implements the interface
   */
  public Boolean isBean() {
    return new Boolean(this.isBean);
  }

  /**
   * Returns the number of the accessible properties.
   * (-1 if there was no Bean analyzed)
   */
  public Integer getFieldCount() {
    if (methods == null)
      return new Integer(-1);
    else
      return new Integer(methods.length);
  }

  /**
   * Get Keyset <b>Do not use manually</b>
   *@param a correct clause for the SELECT statement
   *@see asgaard.skid.SkidBeanFilter#getResultKeys
   *@returns a key-collection
   *@throws SQLException
   */
  public Vector getResultKeys(String clause) throws SQLException {
    Vector res = new Vector();
    Object[] args = new Object[2];
    args[0] = tableName;
    args[1] = clause;
    selectKey.exec(args, "BeanKeyCall.getResultKeys#1");
    while (selectKey.next())
      res.addElement(new Integer(selectKey.getInt(1)));
    return res;
  }

  /**
   * Checks, if the KEY-Table exists.
   */
  private boolean checkTableExist() {
    if (tableExist)
      return true;
    try {
      try {
        Object[] args = new Object[2];
        args[0] = tableName;
        args[1] = BeanBrokerCall.getKeyNullClause();
        selectKey.exec(args, "BeanKeyCall.checkTable#1");
        tableExist = true;
      } catch (SQLException e) {   }
    } finally {
      return tableExist;
    }
  }

  /**
   * Updates the Keys of an Bean.
   *@param aBean the Bean which shall be updated
   *@throws Exception if the update was not successful
   *@return if there are any Keys to enter
   */
  public boolean updateKeys(RemoteBean aBean) throws Exception {
    setTableName(BeanBrokerCall.getTableName( aBean.getClassKey() ));
    return updateKeys( aBean, String.valueOf(aBean.getKey()), 0);
  }

  /**
   * Updates the Keys of an Object.
   *@param aObj the Object which shall be updated
   *@param key the id of the Object
   *@param fieldno number of the Filed (bean = 0)
   *@throws Exception if the update was not successful
   *@return if there are any Keys to enter
   */
  private boolean updateKeys(Object aObj, String key, int fieldno) throws Exception {
    if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "updateKeys#0", LogMsg.LOG_DEBUG, true, String.valueOf(aObj) ); } catch (Exception ex) { }
    if (!tableExist)
      analyzeBean(aObj.getClass());                   // do we know the Bean?
    if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "updateKeys#1", LogMsg.LOG_DEBUG, true, "parentcall="+String.valueOf(parentClassCall) ); } catch (Exception ex) { }
    if ((parentClassCall != null) && (aObj instanceof RemoteBean) && (parentClassCall != this))
      parentClassCall.updateKeys((RemoteBean) aObj);                      // to the parent-methods
    if (methods.length < 1)
      return false;                                                       // nothing to do
    Object[] args = new Object[2];
    args[0] = tableName;
    if (fieldno > 0)
      args[1] = key + BeanBrokerCall.getSecondaryKeyClause( fieldno );
    else
      args[1] = key;
    String insStr = new String( key );
    if (fieldno > 0)
      insStr += BeanBrokerCall.addListClause( String.valueOf( fieldno ));
    Object[] none = new Object[0];
    deleteKey.exec(args, "BeanKeyCall.deleteKeys#" + (String) args[0]);   // del old
    int rowCount = 1;
    int currentRow = 0;
    String[] values = new String[methods.length];
    while (currentRow < rowCount) {             // process rows (if any)
      for (int i=0;i<methods.length;i++) {      // process columns
        Object aReturn = null;
        if (array[i]) {                         // if aray
          Object[] aArray = (Object[]) methods[i].invoke(aObj, none);
          if (aArray != null) {                 // check length
            if ((currentRow == 0) && (rowCount < aArray.length))
              rowCount = aArray.length;
            if (currentRow < aArray.length)     // return current object
              aReturn = aArray[currentRow];
          }
        } else                                  // normally: get object
          if (currentRow == 0)
            aReturn = methods[i].invoke(aObj, none);
        if (aReturn == null) {                  // check
          if (values[i] == null)
            values[i] = "0";
        } else {                                // only unknown values are written ...
          values[i] = getValue(aReturn, new Integer(i));
          if ((types[i] == REKURSE) && (rekurse[i] != null))
            rekurse[i].updateKeys( aReturn, key, fieldnr[i]);
        }
        insStr += BeanBrokerCall.addListClause( values[i]);                // build string
      }
      args[1] = insStr;
      // MAY BE ITERATED FOR ARRAY-OBJECTS
      insertKey.exec(args, "BeanKeyCall.insertKeys#" + (String) args[0]);  // insert new
      currentRow++;
    }
    return true;
  }

  /**
   * Returns the value as String dependend on the Type of the property
   *@param value to convert
   *@param index the index of the Property
   *@return the value as String
   */
  public String getValue(Object value, Integer index) throws Exception {
    switch (types[index.intValue()]) {                   // transform values
      case BEAN:    RemoteBean sb = (RemoteBean) value;
                          return String.valueOf(sb.getKey());
      case BOOLEAN: Boolean b = (Boolean) value;
                          if (b.booleanValue())
                            return "1";
                          else
                            return "0";
      case BYTE: case SHORT: case INT: case LONG: case FLOAT: case DOUBLE:
                          return value.toString();
      case DATE:    java.util.Date d = (java.util.Date) value;
                          return String.valueOf(d.getTime());
      case REKURSE: return String.valueOf(value.hashCode());
      default:      return String.valueOf(value.toString().hashCode());
    }
  }

  /**
   * Analyses an Class and set up all nessecary Information
   *@param aClass the class which shall be analyzed
   *@throws Exception if the given Class cannot be instantiated
   */
  private void analyzeBean(Class aClass) throws Exception {
    setBeanclass(aClass);
    Method[] newMethods;
    Object aObj = beanclass.newInstance();
    if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#0", LogMsg.LOG_DEBUG, true, String.valueOf(beanclass)); } catch (Exception ex) { }
    setTableName(BeanBrokerCall.getTableName( p ));
    if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#1", LogMsg.LOG_DEBUG, true, String.valueOf(tableName)); } catch (Exception ex) { }
    if (aObj instanceof RemoteBean) {                   // check if Bean
      isBean = true;
      Class superClass = beanclass.getSuperclass();
      Object aParent = superClass.newInstance();
      if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#2", LogMsg.LOG_DEBUG, true, String.valueOf(aParent)); } catch (Exception ex) { }
      superCount = superClass.getMethods().length;
      RemoteBean aBean = (RemoteBean) aObj;
      newMethods = aBean.getMethods();
      if (aParent instanceof RemoteBean) {              // check if root
        Method[] superMethods = superClass.getMethods();
        if (! SkidBean.LAST_METHOD.equalsIgnoreCase(superMethods[superMethods.length-1].getName())) {
          isRoot = false;
          if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#3", LogMsg.LOG_DEBUG, true, String.valueOf(aParent.getClass().getName())); } catch (Exception ex){ }
          BeanBroker broker = (BeanBroker) BeanBroker.THIS;
          BeanKeyCall aCall = broker.getBeanKeyCall(superClass);
          if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#4", LogMsg.LOG_DEBUG, true, String.valueOf(aCall)); } catch (Exception ex){ }
          if (aCall.equals(this))
            throw new SkidException("Rekursive Keycalls!");
          parentClassCall = aCall;
        }
      } else {
        isRoot = true;
        while ((superCount < newMethods.length) &&      // methods to scratch
               (! SkidBean.LAST_METHOD.equalsIgnoreCase(newMethods[superCount].getName())))
          superCount++;
        if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#5", LogMsg.LOG_DEBUG, true, String.valueOf(aParent.getClass().getName())); } catch (Exception ex){ }
      }
    } else {                                            // check normal classes
      Class superClass = beanclass;
      while (! superClass.getSuperclass().getName().startsWith("java."))
        superClass = superClass.getSuperclass();
      superCount = superClass.getMethods().length;
      newMethods = beanclass.getMethods();
    }
    int aCount = 0;
    for (int i=superCount;i<newMethods.length;i++) {    // count the Methods
      if (newMethods[i].getName().startsWith( BeanBrokerCall.NAME_PREFIX ) )
        if (newMethods[i].getParameterTypes().length == 0)
          aCount++;
    }
    if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#5", LogMsg.LOG_DEBUG, true, String.valueOf(aCount) ); } catch (Exception ex) { }
    methods = new Method[aCount];                       // create Buffer
    fieldnr = new int[aCount];
    types = new byte[aCount];
    fieldNames = new String[aCount];
    array = new boolean[aCount];
    rekurse = new BeanKeyCall[aCount];
    aCount = 0;
    for (int i=superCount;i<newMethods.length;i++) {    // select the Methods
      if (newMethods[i].getName().startsWith( BeanBrokerCall.NAME_PREFIX ) )
        if (newMethods[i].getParameterTypes().length == 0) {
          if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#6", LogMsg.LOG_DEBUG, true, String.valueOf(methods[aCount].getName())); } catch (Exception ex) { }
          methods[aCount] = newMethods[i];
          fieldNames[aCount] = methods[aCount].getName().substring(BeanBrokerCall.NAME_PREFIX.length(), methods[aCount].getName().length());
          fieldnr[aCount] = i;                          // analyze Method
          try {
            try {
              array[aCount] = newMethods[i].getReturnType().isArray();
              Object aReturn = newMethods[i].getReturnType().newInstance();
              if ( aReturn instanceof Boolean)
                types[aCount] = BOOLEAN;
              else if ( aReturn instanceof Byte)
                types[aCount] = BYTE;
              else if ( aReturn instanceof Short)
                types[aCount] = SHORT;
              else if ( aReturn instanceof Integer)
                types[aCount] = INT;
              else if ( aReturn instanceof Long)
                types[aCount] = LONG;
              else if ( aReturn instanceof Float)
                types[aCount] = FLOAT;
              else if ( aReturn instanceof Double)
                types[aCount] = DOUBLE;
              else if ( aReturn instanceof java.util.Date)
                types[aCount] = DATE;
              else if ( aReturn instanceof RemoteBean)
                types[aCount] = BEAN;
              else if ( aReturn.getClass().getName().startsWith("java.") )
                types[aCount] = OTHER;
              else {
                types[aCount] = REKURSE;
                if (rekurse[aCount] == null) {
                  if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#7", LogMsg.LOG_DEBUG, true, String.valueOf(aReturn)); } catch (Exception ex) { }
                  rekurse[aCount] = new BeanKeyCall( aReturn.getClass(),
                                                     BeanBrokerCall.getTableName( BeanBroker.THIS.getClassKey( aReturn.getClass() )));
                }
              }
            } catch (InstantiationException e) {
              if (e.getMessage().equals("boolean"))
                types[aCount] = BOOLEAN;
              else if (e.getMessage().equals("byte"))
                types[aCount] = BYTE;
              else if (e.getMessage().equals("short"))
                types[aCount] = SHORT;
              else if (e.getMessage().equals("int"))
                types[aCount] = INT;
              else if (e.getMessage().equals("long"))
                types[aCount] = LONG;
              else if (e.getMessage().equals("float"))
                types[aCount] = FLOAT;
              else if (e.getMessage().equals("boolean"))
                types[aCount] = DOUBLE;
              else
                types[aCount] = OTHER;
            }
          } catch (Exception e) {
            if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#8", LogMsg.LOG_DEBUG, false, e.toString()); } catch (Exception ex) { }
            if (debug) {
              System.err.println(e.toString());
              e.printStackTrace();
            }
          }
          aCount++;
        }
    }
    if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#9", LogMsg.LOG_DEBUG, true, String.valueOf(tableExist)); } catch (Exception ex) { }
    if (!tableExist) {               // if it is new: create table and classes
      if (! checkTableExist())
        createKeys(beanclass);
    }
    if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "analyzeBean#A", LogMsg.LOG_DEBUG, tableExist, String.valueOf(tableExist)); } catch (Exception ex) { }

  }

  /**
   * Creates a new Table within the Database containing the (additional) keys
   * of the Class.
   *@param aClass the class to create a physical table
   *@return if there are any Keys whith this Bean
   */
  private boolean createKeys(Class aClass) throws Exception {
    if (methods.length < 1)
      return false;
    String create = BeanBrokerCall.getCreateKeyClause(aClass);
    for (int i=0;i<methods.length;i++) {    // create the keys
      create = BeanBrokerCall.getCreateKeyToken (fieldnr[i], types[i], create);
    }
    Object[] args = new Object[2];
    args[0] = tableName;
    args[1] = create;
    try {
      dropKey.exec(args, "BeanKeyCall.dropKeys#" + (String) args[0]);
    } catch (Exception e) {
      if (debug) try { BeanBroker.THIS.logMsg( this.getClass().getName(), "dropKeys#E", LogMsg.LOG_DEBUG, false, e.toString()); } catch (Exception ex) { }
    }
    createKey.exec(args, "BeanKeyCall.createKeys#" + (String) args[0]);
    tableExist = true;
    return true;
  }


}
