// This class is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 270499
//
//@created	270499  KH enhanced BeanLink to Filter Links
package asgaard.skid;

public class BeanLinkFilter extends BeanLink {
  /**
   * Link to the BeanBroker (one per VM).
   */
  private transient static RemoteBroker broker = BeanBroker.THIS;

  protected byte useFrom = BeanBroker.FILTER_EQ;
  protected byte useOrder = BeanBroker.FILTER_NONE;
  protected byte useTo = BeanBroker.FILTER_EQ;
  protected byte useBy = BeanBroker.FILTER_EQ;
  protected byte useCreated = BeanBroker.FILTER_NONE;
  protected byte useModified = BeanBroker.FILTER_NONE;
  protected byte useBegin = BeanBroker.FILTER_NONE;
  protected byte useEnd = BeanBroker.FILTER_NONE;
  protected byte useState = BeanBroker.FILTER_NONE;

  /**
   * Creates a new, empty Filter
   */
  public BeanLinkFilter() {
      useFrom = BeanBroker.FILTER_NONE;;
      useTo = BeanBroker.FILTER_NONE;;
      useBy = BeanBroker.FILTER_NONE;;
      Created = 0;
      Modified = 0;
      Age = 0;
  }

  /**
   * Creates a new Filter based on the given BeanLink.
   * By default only the Key of the Link (From, To, By) is used.
   *
   *@param link the source link
   */
  public BeanLinkFilter(BeanLink link) {
    From = link.From;
    Order = link.Order;
    To = link.To;
    By = link.By;
    Created = link.Created;
    Modified = link.Modified;
    Age = link.Age;
    Begin = link.Begin;
    End = link.End;
    State = link.State;
  }

    /**
     * Sets the value of the Linked-From key, especially used for creating an Filter.
     *@value  the value to set
     *@filter the Filter condition set (see <code>BeanBroker.FILTER_*</code>
     *@see asgaard.skid.BeanBroker
     */
    public void setFrom(int value, byte filter) {
      From = value;
      useFrom = filter;
    }
    /**
     * Sets the value of the order Field, especially used for creating an Filter.
     *@value  the value to set
     *@filter the Filter condition set (see <code>BeanBroker.FILTER_*</code>
     *@see asgaard.skid.BeanBroker
     */
    public void setOrder(int value, byte filter) {
      Order = value;
      useOrder = filter; }
    /**
     * Sets the value of the Linked-To key, especially used for creating an Filter.
     *@value  the value to set
     *@filter the Filter condition set (see <code>BeanBroker.FILTER_*</code>
     *@see asgaard.skid.BeanBroker
     */
    public void setTo(int value, byte filter) {
      To = value;
      useTo = filter;
    }
    /**
     * Sets the value of the Linked-By key, especially used for creating an Filter.
     *@value  the value to set
     *@filter the Filter condition set (see <code>BeanBroker.FILTER_*</code>
     *@see asgaard.skid.BeanBroker
     */
    public void setBy(int value, byte filter) {
      By = value;
      useBy = filter;
    }
    /**
     * Sets the value of for the Created-Timestamp, especially used for creating an Filter.
     *@value  the value to set
     *@filter the Filter condition set (see <code>BeanBroker.FILTER_*</code>
     *@see asgaard.skid.BeanBroker
     */
    public void setCreated(long value, byte filter) {
      Created = value;
      useCreated = filter;
    }
    /**
     * Sets the value of the Modified-Timestamp, especially used for creating an Filter.
     *@value  the value to set
     *@filter the Filter condition set (see <code>BeanBroker.FILTER_*</code>
     *@see asgaard.skid.BeanBroker
     */
    public void setModified(long value, byte filter) {
      Modified = value;
      useModified = filter;
    }
    /**
     * Sets the value of the Begin-Timestamp, especially used for creating an Filter.
     *@value  the value to set
     *@filter the Filter condition set (see <code>BeanBroker.FILTER_*</code>
     *@see asgaard.skid.BeanBroker
     */
    public void setBegin(long value, byte filter) {
      Begin = value;
      useBegin = filter;
    }
    /**
     * Sets the value of the End-Timestamp, especially used for creating an Filter.
     *@value  the value to set
     *@filter the Filter condition set (see <code>BeanBroker.FILTER_*</code>
     *@see asgaard.skid.BeanBroker
     */
    public void setEnd(long value, byte filter) {
      End = value;
      useEnd = filter;
    }
    /**
     * Sets the value of the Status-Field, especially used for creating an Filter.
     *@value  the value to set
     *@filter the Filter condition set (see <code>BeanBroker.FILTER_*</code>
     *@see asgaard.skid.BeanBroker
     */
    public void setState(int value, byte filter) {
      State = value;
      useState = filter;
    }

    /**
     * Returns the Links found with this filter
     *@return a list of Links
     *@throws Exception 
     */
    public BeanLink[] getLinks() throws Exception {
      return broker.getLink( this );
    }

}