// This class is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 270499
//
//@created	270499  KH
package asgaard.skid;
// Import the java classes used in applets
import java.rmi.*;
import java.sql.*;

/**
 * This is the remote interface for the BeanBroker
 */
public interface RemoteBroker extends Remote {

    /**
     * Class-Key of the Bean, differ between different classes.
     *@param aBean a Bean asked for the class-id
     */
    public int getClassKey(RemoteBean aBean) throws RemoteException;

    /**
     * Class-Key of the Bean, differ between different classes.
     *@param aClass the Bean-Class asked for the class-id
     */
    public int getClassKey(Class aClass) throws RemoteException;

    /**
     * Excanges an user with an authorised object (hardcoded)
     */
    public Object getUser(Object o, String name, String encr) throws SkidException, RemoteException, SQLException;

    /**
     * Excanges an user with an authorised object (hardcoded)
     */
    public Object getUser(String name, String encr) throws SkidException, RemoteException, SQLException;

    /**
     * Excanges an user with an authorised object (hardcoded)
     */
    public boolean putUser(Object o) throws SkidException, RemoteException, SQLException;
//    public boolean putUser(Object o, String name, String encr) throws SkidException, RemoteException, SQLException;

    /**
     * Checks if the given username exists or not
     *@param	userName name of the user
     *@param	encr the encrypted password (optional)
     *@returns the user's status if the username exists and else null
     *@exception    RemoteException	if RMI doesen't work
     *@exception    SQLException the db doesen't work
     */
    public String checkUser( String userName, String encr ) throws SkidException, RemoteException, SQLException;

    /**
     * Puts the Bean into the Chache and saves it <b>do not use manually</b>.
     * Checks the Timestamps according to a update-policy given by the Bean and
     * saves al modifications permanently.
     * <b><i>To enforce a Bean's saving use the Bean's saving method.</i> <code>flush</code></b>
     *<p>
     * <i>Remark:</i> A System-Object cannot be modified and carries the created-Timestamp 0
     *
     *@see asgaard.skid.RemoteBean#saveflush
     *@param aObj the calling Bean
     *@return wether an update is possible or not.
     *@throws Exception if an error occoured during the writeback.
     */
    public Object[] putBean(RemoteBean aObj) throws Exception, RemoteException;

    /**
     * Searches for a Bean by a given key.
     *@param aClass the searched class
     *@param paramName list of the parameter to search for
     *@param searchValues values to fit
     *@param search-operator
     *@return a set of Bean as result of the search
     *@throws Exception if anything fails
     */
    public int[] getBeanKey(String aClass, String[] paramName, Object[] searchValue, byte[] searchOp) throws SkidException , RemoteException;

    /**
     * Searches for a Bean by a given key.
     *@param key the primary key of the object
     *@return the searched Bean if the key exist
     *@throws Exception if anything fails
     */
    public RemoteBean getBean(int key) throws Exception, RemoteException;

    /**
     * Searches for Beans by a given set of keys. Simply calles the primitive getBean method.
     *@param key the primary key of the object
     *@return the searched Bean if the key exist
     *@throws Exception if anything fails
     */
    public RemoteBean[] getBean(int[] key) throws Exception, RemoteException;

    /**
     * Returns whether an Bean is available within the Broker or not.
     *@param key the unique key of the Bean
     *@return true for an existing Bean with the given key
     */
    public boolean beanExist(int key) throws RemoteException;

    /**
     * Returns the Key-Call-Adapter for a asked Class (represented by it's key).
     *@param classkey the id of the class
     *@throws Exception if the key's couldn't be mapped
     */
    public BeanKeyCall getBeanKeyCall(Class aClass) throws Exception;
    /**
     * Adds a new Relation between Beans.
     *@param link one relation
     *@return wether an update is possible or not.
     *@throws Exception if the key is not valid or anything else fails
     */
    public boolean putLink(BeanLink link) throws Exception, RemoteException;

    /**
     * Searches for a specific Link using it's primary key.
     *@param from key of the origin Bean
     *@param to key of the target Bean
     *@param  by key of the link-Bean (kind of Link)
     *@return the link if found
     *@throws Exception if anything goes wrong.
     */
    public BeanLink getLink(int from, int to, int by) throws Exception, RemoteException;

    /**
     * Searches for a set of Links using the filter - capabilities of
     * <code>BeanLink</code>.
     *@param filter the BeanLink used as Filter
     *@return the links if any found
     *@throws Exception if anything goes wrong.
     */
    public BeanLink[] getLink(BeanLinkFilter filter) throws Exception, RemoteException;

    /**
     * Returns if a specific Key exists, hiding all Exceptions.
     *@param from key of the origin Bean
     *@param to key of the target Bean
     *@param  by key of the link-Bean (kind of Link)
     *@return the link if found
     */
    public boolean linkExist(int from, int to, int by) throws RemoteException;

    /**
     * Adding an Event and replicating it to all suscribed Objects.
     *@param producer of the event (not <code>null</code>)
     *@param event the kond of the event (not <code>null</code>)
     *@throws Exception if it doesen't work
     */
    public void addEvent(RemoteBean producer, RemoteBean event) throws Exception, RemoteException;

    /**
     * Changing the State of an specific event.
     *@param producer of the event (not <code>null</code>)
     *@param event the kond of the event (not <code>null</code>)
     *@param consumer of the event (not <code>null</code>)
     *@param fromState ols State of the event
     *@param toState ols State of the event
     *@throws Exception if it doesen't work
     */
    public void setEvent(RemoteBean producer, RemoteBean event, RemoteBean consumer, byte fromState, byte toState) throws Exception, RemoteException;

    /**
     * Adding an Event-Pattern and suscribing this type of event.
     *@param producer of the event (not <code>null</code>)
     *@param event the kond of the event (not <code>null</code>)
     *@param consumer of the event (not <code>null</code>)
     *@throws Exception if it doesen't work
     */
    public void suscribeEvent(RemoteBean producer, RemoteBean event, RemoteBean consumer) throws Exception, RemoteException;

    /**
     * Returns the Events for a consumer object which are newer than the given time
     *@param consumer of the events
     *@param newer than the given time
     *@returns a set of events
     *@throws Exception
     */
    public SkidEvent[] getEvent(RemoteBean consumer, long newer) throws Exception, RemoteException;

  /**
   * Invokes the called method of the filterinterface. The BeanKeyCall-Objects are cached by

   * the BeanBroker and available via this one-method interface

   *
   *@param beanClass the class which the asked BeanKeyCall object is for
   *@param name the name of the Method
   *@param args the Arguments
   *@return a result returned (if any)
   *@see java.lang.reflect.Method#invoke
   *@throws IllegalAccessException if the underlying method is inaccessible.

   *@throws IllegalArgumentException if the number of actual and formal parameters differ, or if an unwrapping conversion fails.
   *@throws InvocationTargetException if the underlying method throws an exception.
   *@throws NullPointerException if the specified object is null.
   *@throws NoSuchMethodException if there is no Method with this name
   */
  public Object invokeCall(Class beanClass, String name, Object args[], String[] classnames) throws Exception;

  /**
   *  Switches the test - mode on / off. All changes at the Database between on and off
   * will be deleted.
   */
  public void testMode(boolean on) throws RemoteException;

  /**
   * Puts a Message to the LOG-Server if available and returns whether it was sent or not
   *@see asgaard.utils.log.LogMsg
   *@param server     The calling Java-Serverclass
   *@param point      The Method / place where the statement was created
   *@param typ        The Type of the LOG-Statement
   *@param ok         Was the ServicePoint passed successfuly
   *@param msg        Human redable Message of the Statement
   */
  public boolean logMsg(String server, String point, byte typ, boolean ok, String msg) throws RemoteException;

}

