//
// This file is part of Skid
// (P) 1998 by Klaus Hammermller, klaus@ifs.tuwien.ac.at
// http://www.ifs.tuwien.ac.at/~hammer
//
//@version  1.0
//@autor    Klaus Hammermller
//@created	090398 KH jdk 1.1.4
//@modified	260398 KH + add SQL-Server Connection
//@modified 300398
//@modified 200299 KH ~ adapted to Oracle
//@modified 260299 KH ~ jdk 1.2 (but works with jdk 1.1.6 as well)
//                    + ressources (ResourceBundle)
//@modified 060399 KH fixing trouble with BeanDecoder
//@modified 080399 KH added LogBroker
//@modified 140499 KH - Applet
package asgaard.skid;
// Import the JDBC classes
import java.sql.*;
import jdbc.SimpleText.*;

// Import the java classes used in applets
import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.io.*;
import java.net.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.*;
import java.util.*;
import java.text.*;

// Import SkidAPI
import asgaard.utils.*;
import asgaard.utils.log.*;
import asgaard.skid.*;
import asgaard.lang.*;
import asgaard.utils.test.*;

/**
  * Servertask (Objectbroker) of the Asgaard - Databaselayer - API "Skid".
  *<p>
  * Before starting <code>asgaard.skid.Skid</code> the <code><b>rmiregistry</b></code>
  * need to be started!
  *<p>
  * When <i>Skid</i> is started, aware that the uses <code>.property</code> files
  * are in the local directory.
  */
public class Skid extends UnicastRemoteObject implements RemoteBroker {

  /**
   * Name for the Binding-service.
   */
    public static String ServerName = "SkidServer";        // Binding Name
  /**
   * The Service Name
   */
    public final static String SERVER = "Skid";
  /**
   * The version number
   */
 	  public final static String VERSION = "2.0.8";
  /**
   * The Copyright message
   */
    public static final String COPYRIGHT = "(P) by the Asgaard Project group http://www.ifs.tuwien.ac.at/asgaard\n"+
                                           "    under GNU public license - absolute NO WARRANTY on this software.";

    public static byte LogLevel = LogMsg.LOG_DEBUG;        // VALUES DEFINING which messeges are loged
    public static byte MailLevel = LogMsg.LOG_ERROR;
    public final static boolean debug = true;
    private static boolean test = false;
    private static boolean remoteLog = false;

    private long cSerienNr = 2;	                // Serialnumber of the SkidServer 1... public number without support
    private static String default_Property =    // Filename for the defaults

                    "default";
    private static String property_Extension =  // Property Extension
                    ".properties";
    public static String driver_class =		      // The driver to load
                    "jdbc.SimpleText.SimpleTextDriver";
//                  "oracle.jdbc.driver.OracleDriver";
//                  "com.sybase.jdbc.SybDriver";// sqlAnywhere
//                  "org.gjt.mm.mysql.Driver";  // mySQL thin driver
    public final static String CONNECT_FLATFILE =
                    "jdbc:SimpleText";          // Flatfile-Driver
    public static String connect_string =	  	  // The connect string
                    CONNECT_FLATFILE;
//   		            "jdbc:oracle:thin:@nornen.ifs.tuwien.ac.at:1521:orcl";
//                  "jdbc:sybase:Tds:localhost:7373/sademo";
//                  "jdbc:mysql://nornen.ifs.tuwien.ac.at:3306/skid";
    private static String connect_user =	"";   // User-name
//                                  "scott";	  // Oracle
//                                  "DBA";	    // SQL-Anywhere
    private static String connect_pwd =	"";	    // Password
//                                  "tiger";	  // Oracle
//                                  "SQL";	    // SQL-Anywhere
    private static String db_name = "";         // Name of the db
    private static String db_ver = "";          // Version of the db

    public static String local_db_path = ".skiddb";  // default path for flatfile - db
    private static final String SQL_RES = "asgaard.skid.resources.SQLRes";

    /**
     * Index of all persistent Objects.
     */
    private BeanBroker broker = null;
    /**
     * LOGFILE of this Server
     */
    protected static LogClient log;

    private Connection	conn;			// The connection to the database
    private static String propertyFile = "";
    private static String LOGpath = ".skidlog";
    private static long LOGsize = 0;
    private static String LOGmailTO = null;
    private static String LOGmailCC = "";
    private static String LOGmailHOST = null;
    private static String LOGserver = null;
    private static Long LOGtimeout = new Long(10000);   // Timeout
	  public  static Boolean localDB = Boolean.TRUE;

    private static ResourceBundle sres = null;
    private static ResourceBundle SQLres = null;
	  private static Properties properties;

    private boolean invokedStandalone = false;

    /**
     * Reads a property - file
     */
    private static void readProperties(Properties properties) {
			db_name = properties.getProperty("dbName", db_name);
			db_ver = properties.getProperty("dbVersion", db_ver);
			driver_class = properties.getProperty("jdbcDriver", driver_class);
			connect_string = properties.getProperty("connectString", connect_string);
			connect_user = properties.getProperty("connectUser", connect_user);
			connect_pwd = properties.getProperty("connectPwd", connect_pwd);
      local_db_path = properties.getProperty("localDBPath", local_db_path);
      localDB = Boolean.valueOf(properties.getProperty("localDB", localDB.toString ()));
      LogLevel = Byte.valueOf(properties.getProperty("logLevel", String.valueOf(LogLevel))).byteValue();
      MailLevel = Byte.valueOf(properties.getProperty("MailLevel", String.valueOf(MailLevel))).byteValue();
      LOGsize = Long.valueOf(properties.getProperty("logSize", String.valueOf(LOGsize))).longValue();
      LOGmailHOST = properties.getProperty("mailHost", LOGmailHOST);
      LOGmailTO = properties.getProperty("mailTo", LOGmailTO);
      LOGmailCC = properties.getProperty("mailCc", LOGmailCC);
      propertyFile = properties.getProperty("propertyFile", propertyFile);
    }

    /**
     * Application - main, follows start of the Commandline.
     * Never called directly.
     *
     *@param argv   Commandlineparameters given within Start of the Servertask
     */
    public static void main( String args[] ) {
      sres = ResourceBundle.getBundle("asgaard.skid.resources.SkidRes", Locale.US);
		  try {
        Properties df = new Properties();
			  df.load(new FileInputStream(new File (default_Property + property_Extension)));
		    properties = new Properties(df);
        readProperties(properties);
			} catch (Exception e) {
          System.err.println(default_Property+property_Extension+": " + e.toString());
          if (debug) e.printStackTrace();
  			}
      try {                                               // --------------------------------
        out("greetings", SERVER + " V" + VERSION);        // ANALYZING THE COMMANDLINE_PARAMs
        System.out.println(COPYRIGHT);
		    for (int i = 0; i < args.length; i++) {
			    if (args[i].charAt(0) == '-') {
				    boolean skiparg = false;
    				int j = 1;
	    			while ((j < args[i].length()) && (!skiparg)) {
		    			switch(args[i].charAt(j)) {
			    		case 'h': case '?': usage(true); break;     // HELP
              case 't': test = true; break;
              case 'l':                                   // LOGLEVEL
                switch(args[i].charAt(j+1)) {
                  case '0': LogLevel = -1; j += 1; break;
                  case '1': LogLevel = 0; j += 1; break;
                  case '2': LogLevel = 1; j += 1; break;
                  case '3': LogLevel = 2; j += 1; break;
                  case '4': LogLevel = 3; j += 1; break;
                  case '5': LogLevel = 4; j += 1; break;
                  case '6': LogLevel = 5; j += 1; break;
					    		default: usage(false); break;
                } break;
              case 'm':                                   // MailLevel
                switch(args[i].charAt(j+1)) {
                  case '0': MailLevel = -1; j += 1; break;
                  case '1': MailLevel = 0; j += 1; break;
                  case '2': MailLevel = 1; j += 1; break;
                  case '3': MailLevel = 2; j += 1; break;
                  case '4': MailLevel = 3; j += 1; break;
                  case '5': MailLevel = 4; j += 1; break;
                  case '6': MailLevel = 5; j += 1; break;
					    		default: usage(false); break;
                } break;
  				  	case 'p':                                   // PATH
	  				  	if ((i+1 >= args.length) || (j+1 == args[i].length()))
		  				  	usage(false);
  			  			else {
	  			  			String tstr = null;
		  			  		if (j+2 >= args[i].length()) {
			  			  		tstr = args[i+1];
				  			  	skiparg = true;
  				  			} else
	  				  			tstr = args[i].substring(j+2, args[i].length());
  			  				switch (args[i].charAt(j+1)) {
		    				  	case 'C': connect_string = tstr; break;
    					  		case 'D': driver_class = tstr; break;
	  			  		  	case 'F': if (!tstr.endsWith(File.separator)) tstr += File.separator;
                              local_db_path = tstr; break;
	      						case 'H': LOGmailHOST = tstr; break;
                    case 'L': if (!tstr.endsWith(File.separator)) tstr += File.separator;
                              LOGpath = tstr; break;
	      						case 'M': LOGmailCC = tstr; break;
		  			  		  case 'P': if (!tstr.endsWith(property_Extension)) tstr += property_Extension;
                              propertyFile = tstr;
                              try {
  			                        properties.load(new FileInputStream(new File (propertyFile)));
                                readProperties(properties);
			                        } catch (Exception e) {
                                System.err.println(propertyFile+": " + e.toString());
                                if (debug) e.printStackTrace();
			                        }
                              break;
		      					case 'S': LOGserver = tstr;
                              remoteLog = true;
                      break;
	      						case 'T': LOGmailTO = tstr; break;
				    			  case 'U': connect_user = tstr; break;
				  		    	case 'W': connect_pwd = BeanBroker.encryptPwd(tstr); break;
	      						case 'X': LOGsize = Long.getLong(tstr).longValue() * 1024; break;
  					    		default: usage(false); break;
	  						  }
    	  					if (!skiparg) j = args[i].length()-1;
		  	  			}
			  	  		break;
				  	  default: usage(false); break;
  					  }
  	  				j++;
	  	  		}
		  	  	if (skiparg) i++;
          }
  			}
      } catch (Exception e) {
          if (debug) {
            System.err.println(e.toString());
            e.printStackTrace();
          } else
 				  	usage(false);                                 // ANALYZING THE COMMANDLINE_PARAMs
      }                                                   // --------------------------------
      if (db_name.length() == 0) {
        StringTokenizer dbn = new StringTokenizer(connect_string, ":");
        dbn.nextToken();
        db_name = dbn.nextToken();
      }
		  SQLres = ResourceBundle.getBundle(SQL_RES, new Locale(db_name, db_ver));
      try {                                               // setup the LOGFILE
        out("log");
        if (remoteLog) {                                  // create a LogClient
          log = new LogClient(SERVER + ": ", LOGserver, true);
          out("connecting", LOGserver);
        } else
          log = new LogClient(SERVER + ": ", LOGpath, MailLevel, LOGmailTO, SERVER, LOGmailCC, LOGmailHOST, LOGsize, false);
        remoteLog = log.isRemote();                       // what is it?
      } catch (Exception e) {
        out("sorry");
        System.err.println(SERVER + ": " + e.toString());
        if (debug) e.printStackTrace();
        out("bye");
      	System.exit(1);
      }
      try {                                               // INITIALIZING THE SERVER
                                                          // -----------------------
        if (debug) logMsg( "main#D1", LogMsg.LOG_DEBUG, true, "Set Security-Manager");
        if (System.getSecurityManager() == null) {
          System.setSecurityManager(new RMISecurityManager());
        }
        if (debug) logMsg( "main#D2", LogMsg.LOG_DEBUG, true, "Try to instanciate Skid application");
        Skid app = new Skid();                             // Instanciate the Skid-server
        if (debug) logMsg( "main#D3", LogMsg.LOG_DEBUG, true, "Try to open Connection");
        app.openConnection();                              // connect to DB
        if (test)
          app.broker.testIt(app);
        if (debug) logMsg( "main#D4", LogMsg.LOG_DEBUG, true, "Try to Rebind");
        Naming.rebind( ServerName, app );                  // binding the Name in the registry
        app.invokedStandalone = true;
        logMsg( "main#1", LogMsg.LOG_REMARK, true, SERVER + " V" + VERSION + " started successfuly.");
        java.util.Date now = new java.util.Date();
        System.out.println( SERVER + " V" + VERSION + " started at " + now.toString() +
                           " bound in the registry to the name " + ServerName);
      } catch (Exception e) {
        out("sorry");
        logMsg( "main#2", LogMsg.LOG_ERROR, false, ServerName + " V" + VERSION + " failed to start: " + e.toString());
        if (debug) System.err.println(e.toString());
        out("bye");
        System.exit(1);
      }
    }

    /**
     * Instanciates an Server-Object
     */
    public Skid() throws RemoteException {
      super();
    }

    /**
     * Opens Databaseconnection, if not open
     */
    private void openConnection() throws Exception {// CONNECTION PROPERTYs
      out("db");
	    if (broker == null) {		                // open the connection to the database
	        out("JDBCdriver", driver_class );
	        out("connecting", connect_string );
          localDB = new Boolean (connect_string.equalsIgnoreCase(CONNECT_FLATFILE));
	        broker = new BeanBroker(driver_class, connect_string, connect_user, connect_pwd, local_db_path, log, LogLevel, localDB.booleanValue(), SQLres, cSerienNr, BeanBroker.encryptPwd(connect_pwd));
	        out("connected");	            // Connect to the databse
	    }
    }

//------------------------------------------------------------------------
// Broker Interface Implementation ***************************************

    /**
     * Class-Key of the Bean, differ between different classes.
     *@param aBean a Bean asked for the class-id
     */
    public int getClassKey(RemoteBean aBean) throws RemoteException {
      return broker.getClassKey(aBean);
    }

    /**
     * Class-Key of the Bean, differ between different classes.
     *@param aClass the Bean-Class asked for the class-id
     */
    public int getClassKey(Class aClass) throws RemoteException {
      return broker.getClassKey(aClass);
    }

    /**
     * Excanges an user with an authorised object (hardcoded)
     */
    public final Object getUser(String name, String encr) throws SkidException, RemoteException, SQLException {
      return broker.getUser( name, encr );
    }

    /**
     * Excanges an user with an authorised object (hardcoded)
     */
    public final Object getUser(Object o, String name, String encr) throws SkidException, RemoteException, SQLException {
      return broker.getUser( o, name, encr );
    }

    /**
     * Excanges an user with an authorised object (hardcoded)
     */
    public final boolean putUser(Object o) throws SkidException, RemoteException, SQLException {
      return broker.putUser(o);
//    public final boolean putUser(Object o, String name, String encr) throws SkidException, RemoteException, SQLException {
//      return broker.putUser(o, name, encr );
    }

    /**
     * Checks if the given username exists or not
     *@param	userName name of the user
     *@param	encr the encrypted password (optional)
     *@returns the user's status if the username exists and else null
     *@exception    RemoteException	if RMI doesen't work
     *@exception    SQLException the db doesen't work
     */
    public final String checkUser( String userName, String encr ) throws SkidException, RemoteException, SQLException {
      return broker.checkUser( userName, encr );
    }

    /**
     * Puts the Bean into the Chache and saves it <b>do not use manually</b>.
     * Checks the Timestamps according to a update-policy given by the Bean and
     * saves al modifications permanently.
     * <b><i>To enforce a Bean's saving use the Bean's saving method.</i> <code>flush</code></b>
     *<p>
     * <i>Remark:</i> A System-Object cannot be modified and carries the created-Timestamp 0
     *
     *@see asgaard.skid.RemoteBean#saveflush
     *@param aObj the calling Bean
     *@return wether an update is possible or not.
     *@throws Exception if an error occoured during the writeback.
     */
    public Object[] putBean(RemoteBean aObj) throws Exception {
      return broker.putBean( aObj );
    }

    /**
     * Searches for a Bean by a given key.
     *@param aClass the searched class
     *@param paramName list of the parameter to search for
     *@param searchValues values to fit
     *@param search-operator
     *@return a set of Bean as result of the search
     *@throws Exception if anything fails
     */
    public int[] getBeanKey(String aClass, String[] paramName, Object[] searchValue, byte[] searchOp) throws SkidException {
      return broker.getBeanKey(aClass, paramName, searchValue, searchOp);
    }

    /**
     * Searches for a Bean by a given key.
     *@param key the primary key of the object
     *@return the searched Bean if the key exist
     *@throws Exception if anything fails
     */
    public RemoteBean getBean(int key) throws Exception {
      return broker.getBean( key );
    }
    /**
     * Searches for Beans by a given set of keys. Simply calles the primitive getBean method.
     *@param key the primary key of the object
     *@return the searched Bean if the key exist
     *@throws Exception if anything fails
     */
    public RemoteBean[] getBean(int[] key) throws Exception {
      return broker.getBean( key );
    }

    /**
     * Returns whether an Bean is available within the Broker or not.
     *@param key the unique key of the Bean
     *@return true for an existing Bean with the given key
     */
    public boolean beanExist(int key){
      return broker.beanExist( key );
    }

    /**
     * Returns the Key-Call-Adapter for a asked Class (represented by it's key).
     *@param classkey the id of the class
     *@throws Exception if the key's couldn't be mapped
     */
    public BeanKeyCall getBeanKeyCall(Class aClass) throws Exception {
      return broker.getBeanKeyCall( aClass );
    }

    /**
     * Adds a new Relation between Beans.
     *@param link one relation
     *@return wether an update is possible or not.
     *@throws Exception if the key is not valid or anything else fails
     */
    public boolean putLink(BeanLink link) throws Exception {
      return broker.putLink( link );
    }

    /**
     * Searches for a specific Link using it's primary key.
     *@param from key of the origin Bean
     *@param to key of the target Bean
     *@param  by key of the link-Bean (kind of Link)
     *@return the link if found
     *@throws Exception if anything goes wrong.
     */
    public BeanLink getLink(int from, int to, int by) throws Exception {
      return broker.getLink( from, to, by );
    }

    /**
     * Searches for a set of Links using the filter - capabilities of
     * <code>BeanLink</code>.
     *@param filter the BeanLink used as Filter
     *@return the links if any found
     *@throws Exception if anything goes wrong.
     */
    public BeanLink[] getLink(BeanLinkFilter filter) throws Exception {
      return broker.getLink( filter );
    }

    /**
     * Returns if a specific Key exists, hiding all Exceptions.
     *@param from key of the origin Bean
     *@param to key of the target Bean
     *@param  by key of the link-Bean (kind of Link)
     *@return the link if found
     */
    public boolean linkExist(int from, int to, int by) {
      return broker.linkExist( from, to, by );
    }

    /**
     * Adding an Event and replicating it to all suscribed Objects.
     *@param producer of the event (not <code>null</code>)
     *@param event the kond of the event (not <code>null</code>)
     *@throws Exception if it doesen't work
     */
    public void addEvent(RemoteBean producer, RemoteBean event) throws Exception {
      broker.addEvent(producer, event);
    }

    /**
     * Changing the State of an specific event.
     *@param producer of the event (not <code>null</code>)
     *@param event the kond of the event (not <code>null</code>)
     *@param consumer of the event (not <code>null</code>)
     *@param fromState ols State of the event
     *@param toState ols State of the event
     *@throws Exception if it doesen't work
     */
    public void setEvent(RemoteBean producer, RemoteBean event, RemoteBean consumer, byte fromState, byte toState) throws Exception {
      broker.setEvent(producer, event, consumer, fromState, toState);
    }

    /**
     * Adding an Event-Pattern and suscribing this type of event.
     *@param producer of the event (not <code>null</code>)
     *@param event the kond of the event (not <code>null</code>)
     *@param consumer of the event (not <code>null</code>)
     *@throws Exception if it doesen't work
     */
    public void suscribeEvent(RemoteBean producer, RemoteBean event, RemoteBean consumer) throws Exception {
      broker.suscribeEvent(producer, event, consumer);
    }

    /**
     * Returns the Events for a consumer object which are newer than the given time
     *@param consumer of the events
     *@param newer than the given time
     *@returns a set of events
     *@throws Exception
     */
    public SkidEvent[] getEvent(RemoteBean consumer, long newer) throws Exception {
      return broker.getEvent(consumer, newer);
    }

  /**
   * Invokes the called method of the filterinterface. The BeanKeyCall-Objects are cached by

   * the BeanBroker and available via this one-method interface

   *
   *@param beanClass the class which the asked BeanKeyCall object is for
   *@param name the name of the Method
   *@param args the Arguments
   *@return a result returned (if any)
   *@see java.lang.reflect.Method#invoke
   *@throws IllegalAccessException if the underlying method is inaccessible.

   *@throws IllegalArgumentException if the number of actual and formal parameters differ, or if an unwrapping conversion fails.
   *@throws InvocationTargetException if the underlying method throws an exception.
   *@throws: NullPointerException if the specified object is null.
   *@throws NoSuchMethodException if there is no Method with this name
   */
  public Object invokeCall(Class beanClass, String name, Object args[], String[] classnames) throws Exception {

    return broker.invokeCall(beanClass, name, args, classnames);

  }

  /**
   *  Switches the test - mode on / off. All changes at the Database between on and off
   * will be deleted.
   */
  public void testMode(boolean on) {
    broker.testMode(on);
  }

  /**
   * Puts a Message to the LOG-Server if available and returns whether it was sent or not
   *@see asgaard.utils.log.LogMsg
   *@param server     The calling Java-Serverclass
   *@param point      The Method / place where the statement was created
   *@param typ        The Type of the LOG-Statement
   *@param ok         Was the ServicePoint passed successfuly
   *@param msg        Human redable Message of the Statement
   */
  public boolean logMsg(String server, String point, byte typ, boolean ok, String msg) {
    return broker.logMsg(server, point, typ, ok, msg);
  }

// Broker Interface Implementation ***************************************
//------------------------------------------------------------------------

  /**
   * Puts a Message to the LOG-Server if available and returns whether it was sent or not
   *@see asgaard.utils.log.LogMsg
   *@param point      The Method / place where the statement was created
   *@param typ        The Type of the LOG-Statement
   *@param ok         Was the ServicePoint passed successfuly
   *@param msg        Human redable Message of the Statement
   */
  private static boolean logMsg(String point, byte typ, boolean ok, String msg) {
    if ((debug || (LogLevel >= typ)) && (log != null)) {
      log.put( SERVER, point, typ, ok,msg);
      return true;
    } else
      return false;
  }
	// This comment intentionally left blank

	public static void usage(boolean longusage) {
    String arg[] = new String[3];
    arg[0] = SERVER + " V" + VERSION;
    arg[1] = "asgaard.skid.Skid";
    arg[2] = property_Extension;
 		if (longusage) out("longusage", arg);
		else out("usage", arg);
		System.exit(1);
	}

  /**
   * Prints a Message referenced by <code>key</code> to a global standard out, enriched by given <code>args</code>.
   *
   *@param  key Search key
   *@param  key Set of arguments integrated into the message
   */
	public static void out(String key, Object args[]) {
			System.out.println(MessageFormat.format(sres.getString(key), args));
	}

  /**
   * Prints a Message referenced by <code>key</code> to a global standard out, enriched by a given <code>arg</code>.
   *
   *@param  key Search key
   *@param  key a argument integrated into the message
   */
	public static void out(String key, String arg) {
	  Object args[] = { arg };
		out(key, args);
	}

  /**
   * Prints a Message referenced by <code>key</code> to a global standard out.
   *
   *@param  key Search key
   */
	public static void out(String key) {
		System.out.println(sres.getString(key));
	}

	public static String format(String key, Object args[]) {
		return MessageFormat.format(sres.getString(key), args);
	}

	public static String format(String key, String arg) {
		Object args[] = { arg };
		return format(key, args);
	}

	public static String format(String key) {
		return sres.getString(key);
	}

  public static java.util.Date getNow() {
    return new java.util.Date(System.currentTimeMillis());
  }
}

