// This is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 110599
//
//@created	110599  KH
package asgaard.skid;
//import asgaard api
import asgaard.utils.log.*;
//Java classes
import java.util.*;
import java.rmi.*;

/**
 * This is a Filterclass to search for a Bean about it's keys
 *@see asgaard.skid.BeanKeyCall
 *@see asgaard.skid.SkidBean
 */
public class SkidBeanFilter {

  private static RemoteBroker broker = BeanBroker.THIS;

  private Vector entries = new Vector();
  /**
   * The search-operation
   *@see asgaard.skid.BeanBroker.FILTER_*
   */
  protected byte[] op;
  private SkidBeanFilter parentFilter = null;
  private Class beanClass;
  private final static boolean debug = Skid.debug;

  /**
   * Creates a new Filter for a given Class
   *@param aClass the Class to build the filter for
   *@throws Exception if the filter could not be generated from this class
   */
  public SkidBeanFilter(Class aClass) throws Exception {
    this.broker = BeanBroker.THIS;
    beanClass = aClass;
    Boolean root = (Boolean) broker.invokeCall(beanClass, "isRoot", null, null);
    if (! root.booleanValue())                                 // superclasses
      parentFilter = new SkidBeanFilter(aClass.getSuperclass());
  }

  /**
   * Resets the List with Filter-entries.
   */
  public void removeFilterList() {
    entries.removeAllElements();
    if (parentFilter != null)
      parentFilter.removeFilterList();
  }

  /**
   * Searches for the property and sets the corresponding clause
   * which all have to be true.
   *@param prop the name of the property
   *@param value the object representing the value
   *@param op the operation
   *@throws Exception if the given propertyname is not valid
   */
  public void setFilterEntry(String prop, Object value, byte op) throws Exception {
    Integer aIdx = (Integer) broker.invokeCall(beanClass, "getFieldIdx", new Object[] {prop}, null);
    if (aIdx.intValue() < 0)
      if (parentFilter != null)
        parentFilter.setFilterEntry(prop, value, op);
      else
        throw new SkidException("Property '"+prop+"' not found!");
    else {
      String aValue = (String) broker.invokeCall(beanClass, "getValue", new Object[] {value, aIdx}, new String[] {"java.lang.Object"});
      Integer fNr = (Integer) broker.invokeCall(beanClass, "getFieldNr", new Object[] {aIdx}, null);
      entries.addElement( BeanBrokerCall.getSelectKeyToken(fNr.intValue(), op, aValue));
    }
  }

  /**
   * Returns a Set of Keys korresponding to the set Filter-items
   *@returns the set with Bean-Keys
   *@throws Exception
   */
  public int[] getResultKeys() throws Exception {
    int[] pSet = null;                                  // check parent
    if (debug) broker.logMsg( this.getClass().getName(), "getResultKeys#0", LogMsg.LOG_DEBUG, true, String.valueOf( this));
    if (parentFilter != null)
      pSet = parentFilter.getResultKeys();
    else
      pSet = new int[0];
    if (entries.isEmpty())
      return pSet;
    String clause = "";                                 // create select clause
    for (int i=0;i<entries.size();i++) {
      String entry = (String) entries.elementAt(i);
      if (entry.length() > 0) {
        if (clause.length() > 0)
          clause += BeanBrokerCall.addAndClause(entry);
        else
          clause = entry;
      }
    }
    if (debug) broker.logMsg( this.getClass().getName(), "getResultKeys#1", LogMsg.LOG_DEBUG, clause.length()>0, String.valueOf(this) + ":" + String.valueOf(clause));
    if (clause.length() == 0)
      return pSet;                                      // ask db
    Vector res = (Vector) broker.invokeCall(beanClass, "getResultKeys", new Object[] {clause}, null);
    if (res.size() == 0)
      return pSet;
    if (pSet.length != 0) {                             // check to common keys
      int check = Math.min(pSet.length, res.size());
      Vector pRes = new Vector();
      for (int k=0;k<pSet.length;k++) {
        Integer K = new Integer( pSet[k] );
        if ((res.indexOf( K ) >= 0) && (check > 0)) {
          if (0 < pRes.indexOf( K )) {                  // no doubles
            pRes.addElement( K );
            check--;
          }
        }
      }
      res = pRes;
    }
    int[] aSet = new int[res.size()];                   // return the result as array
    for (int j=0;j<res.size();j++) {
      Integer I = (Integer) res.elementAt(j);
      aSet[j] = I.intValue();
    }
    if (debug) broker.logMsg( this.getClass().getName(), "getResultKeys#2", LogMsg.LOG_DEBUG, aSet.length>0, String.valueOf(this) + ":" + String.valueOf(aSet));
    return aSet;
  }

  /**
   * Returns a set with the beans found by the Filter Definition
   *@returns the found Beans
   *@throws Exception it it doesn't work
   */
   public RemoteBean[] getBeans() throws Exception {
     return broker.getBean(this.getResultKeys());
   }

}