// This class is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 270499
//
//@created	270499  KH
package asgaard.skid;
// Import Java classes
import java.util.*;

/**
 * An event can be produced to indicate an major modification and may be
 * suscribed by any object to recognize this modification. It has to be
 * explicitely set by an object and is also consumed explicitely by consumed
 * by an object. The aim is to avoid event-avalanches by automated event-
 * mechanism and adding explicite push-capabilities to the framework.
 */
public class SkidEvent {

  /**
   * The type/state of this event is an pattern for generating Events
   */
  public static final byte EVENT_PATTERN = 0;
  /**
   * The state of this event is UNREAD
   */
  public static final byte EVENT_UNREAD = 1;
  /**
   * The state of this event is processed from the consumer
   */
  public static final byte EVENT_READ = 2;
  /**
   * The state of this event is CHECKED from the consumer
   */
  public static final byte EVENT_CHECKED = 3;
  /**
   * The state of this event is DELETED
   */
  public static final byte EVENT_DELETED = 4;

  private static RemoteBroker broker = BeanBroker.THIS;

  private int Consumer;
  private int Producer;
  private int Event;
  private byte State;
  private long ts;

  /**
   * Creates a new Event
   *@param consumer key of the consumer
   *@param producer key of the producer
   *@param evnet key of the producer
   *@param state of the event
   *@param created timestamp
   */
  public SkidEvent(int consumer, int producer, int event, byte state, long created) {
    Consumer = consumer;
    Producer = producer;
    Event = event;
    State = state;
    ts = created;
  }

  /**
   * Chages the State of the event
   *@param newState the new state
   *@throws Exception 
   */
  public void setState(byte newState) throws Exception {
    broker.setEvent( getProducer(), getEvent(), getConsumer(), State, newState);
    State = newState;
  }

  /**
   * Returns the Time the Event was created
   *@returns a Date
   */
  public Date getTime() {
    return new Date( ts );
  }

  /**
   * Returns the Name of the Event was created
   *@returns a String
   *@throws Exception
   */
  public String getName() throws Exception {
    return broker.getBean( Event ).getName();
  }

  /**
   * Returns the state of the Event
   *@returns state
   */
  public byte getState() {
    return State;
  }

  /**
   * Returns the the Event
   *@returns Bean
   *@throws Exception
   */
  public RemoteBean getEvent() throws Exception {
    return broker.getBean( Event );
  }

  /**
   * Returns the the Producer
   *@returns Bean
   *@throws Exception
   */
  public RemoteBean getProducer() throws Exception {
    return broker.getBean( Producer );
  }

  /**
   * Returns the the Consumer
   *@returns Bean
   *@throws Exception
   */
  public RemoteBean getConsumer() throws Exception {
    return broker.getBean( Consumer );
  }

}