// This class is part of Skid
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	2.0
//
//@created	090399  KH
//@modified 190899  KH redesign
package asgaard.skid;
import java.beans.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.sql.*;

/**
 * SkidMaintain holds some Methods for maintaining the Database
 */
public class SkidMaintain  {

  private final static String DELIM = "s";
  private Connection conn;
  private ResourceBundle sRes;

  static final boolean debug = BeanBroker.debug;


  /**

   * Replace some Quotes with an Deliminator

   */

  public static String replQuote(String s, char q) {

    int j=0;

    while (j<s.length()) {

      if (s.charAt(j) == q) {
        s=s.substring(0, j)+DELIM+s.substring(j+1);
      }
      j++;
    }
    return s;
  }


  /**

   * Replaces the Deliminator with the Quotes

   */

  public static String replDelim(String s, char q) {

    int j = s.indexOf(DELIM);

    while (!(j<0)) {

      s = s.substring(0, j)+q+s.substring(j+DELIM.length());
      j = s.indexOf(DELIM);
    }
    return s;
  }


  /**

   * Does a safe java.text.FormatMessage by temporary replacing the quotes

   */

   public static String saveFormat(String s, Object[] args) {

      return replDelim(java.text.MessageFormat.format(replQuote(s, '\''), args), '\'');

   }


  /**
   * New Instance of a maintenance-object
   *@param	aConn	Connection to the database
   *@param  sRes  Resources with the SQL-Syntax
   *@see java.sql.Connection
   */
  public SkidMaintain( Connection aConn, ResourceBundle sRes ) {
	  this.conn = aConn;
    this.sRes = sRes;
  }

  /**
   * get's all od the content of a Table
   *@param bFile Backupfile
   *@param queryname Query for the table
   *@param stmtname  name of the insert-statement
   */
  private void dumpTable(FileWriter bFile, String queryname, String stmtname) throws SQLException, IOException {
    Vector rows = new Vector();
    String ins = replQuote(sRes.getString(stmtname), '\'');
    Statement stmt = conn.createStatement ( );
	  ResultSet rset = stmt.executeQuery(sRes.getString(queryname));
    int fieldcount = 0;
	  while (rset.next()) {
      if (fieldcount==0)
        try {
          while (true) {
            fieldcount++;
            rset.getString(fieldcount);
          }
        } catch (Exception e) {
          fieldcount--;
        }
      String[] fields = new  String[fieldcount];
      for (int i=1;i<=fieldcount;i++)
        fields[i-1] = rset.getString(i);
      rows.addElement( fields );
    }
    while (!rows.isEmpty()) {
      String[] fields = (String[]) rows.elementAt(0);
      bFile.write(replDelim(java.text.MessageFormat.format(ins, fields), '\'') + "\n\n");
      rows.removeElementAt(0);
    }
  }

  /**
   * Creates a Backup file containing the content of the Database.
   *
   *@param bFile	Backupfile
   *@exception java.io.IOException	wenn Fileoperation milingt
   *@see SkidMaintain#Restore
   *@returns success
   */
  public boolean dump(FileWriter bFile) throws IOException {
    boolean ok = true;
    try {
    	dumpTable(bFile, "SelectAllUser", "InsertUser");
	    dumpTable(bFile, "SelectAllBeans", "InsertBean");
  	  dumpTable(bFile, "SelectAllLinks", "InsertLink");
	    dumpTable(bFile, "SelectAllEvents", "InsertEvent2");
    } catch (Exception e) {
      ok = false;
      if (debug) System.err.println("Dump failed: " + e.toString());
    }
    bFile.close();
    return ok;
  }

  /**
   * Execute SQL-Script
   *
   *@param input	Backupfile: InputStream
   *@exception    java.io.IOException
   *@see SkidMaintain#SkidMaintain
   */
  public void ExecuteSQLScript(BufferedReader input) throws Exception {
    String aLine = "";				                    // Def.
   	String aQuery = "";
   	String aToken = "";
   	boolean rem = false;				                  // Comment
  	boolean remLine = false;
  	Statement stmt = conn.createStatement ();	    // Create a statement
	  Hashtable hashVar = new Hashtable();		      // Objektnames
   	while ( input.ready() ) {
	    aLine = input.readLine();			              // read line
	    remLine = false;
	    if (aLine.length() > 0) {			              // create query
    		StringTokenizer tokens = new StringTokenizer( aLine, " ", false );
		    while ((tokens.hasMoreTokens()) && (! remLine)) {
		      aToken = tokens.nextToken();
    	    if ( aToken.startsWith( "/*" ))	        // comment
    		  	rem = true;
		    if (! rem) {
    			if ( aToken.startsWith( "//" ))
		  	    remLine = true;
    			else {
		  	    if ( aToken.startsWith(""))          // chech Varaible
			      	aQuery += (String) hashVar.get( aToken ) + " ";
			      else
      		    aQuery += aToken + " ";	            // add token to Query
    			}
		    } else {
    			if ( aToken.startsWith( "" ))	        // mark variable
   	    	  hashVar.put( aToken, tokens.nextToken() );
    		  }
		      if ( aToken.endsWith( "*/" ))	          // end comment
    		  	rem = false;
	    	}
      }						                                // EMPTY LINE == END OF QUERY
      if (((aLine.length() < 1) || (! input.ready())) && (aQuery.length() > 0)) {
		    try {					                            // Execute the query
	        ResultSet rset = stmt.executeQuery (aQuery);
//	        if ( outputArea != null ) {
//  			    outputArea.append ("Executing query " + aQuery + "\n");
//	  		    while (rset.next ())		              // Dump the result
//			        outputArea.append (rset.getString (1) + "\n");
//	        }
	  	  } catch (Exception e) {			              // Oops
//  		    if ( outputArea != null ) {
//    	  	  outputArea.append ("Executing query " + aQuery + "\n");
//		    	  outputArea.append (e.getMessage () + "\n");
//	  	    }
  		  }
		    aQuery = "";
      }
    }
  }

  /**
   * Restore
   *
   *@param input	Backupfile
   *@exception    java.io.IOException	wenn Fileoperation milingt
   *@see SkidMaintain#Backup
   */
  public void Restore(DataInputStream input, Scrollbar scrollBar) throws Exception {
  }

  /**
   * Datenbank-Replication
   *
   *@param aSQL
   *@param rFile	Replicationfile
   *@see SkidMaintain#Consolidate
   */
  public void Replicate(String aSQL, FilterOutputStream rFile, Scrollbar scrollBar) {
  }

  /**
   * Datenbank-Consolidation
   *
   *@param rFile	Replikationsfile
   *@exception java.io.IOException
   *@see SkidMaintain#Replicate
   */
  public void Consolidate(FilterInputStream rFile, Scrollbar scrollBar) {
  }

}


