// This class is part of Banshee using Skid
// Banshee is an implementation from the asgaard-project in the sport-domain
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 080699
//
//@created	080699  KH
package asgaard.skid;

import java.util.*;
import java.io.*;
import java.sql.*;
import java.rmi.*;
import javax.servlet.http.*;

import asgaard.utils.log.*;

/**
 * This class holds all information nessecary to identify an valid user-session
 */
public final class UserSession extends Thread {

  /**
   * Minimum length of an valid Password
   */
  public final byte MIN_PWD_SIZE = 4;
  /**
   * The user is not in use any more
   */
  public final static char STATUS_DELETED = 'X';
  /**
   * The normal user with default access-rights
   */
  public final static char STATUS_DEFAULT = 'D';
  /**
   * Administrator
   */
  public final static char STATUS_ADMIN = 'A';
  /**
   * Kind of Expert for Form-Maintenance
   */
  public final static char STATUS_EXPERT_FORM = 'F';
  /**
   * Format of the autom. reply PUSHED
   */
  public final static char STATUS_REPLY_HTML = 'h';
  /**
   * Interval in which a reply is calculated
   */
  public final static char STATUS_REPLY_WEEKLY = 'w';

  /**
   * Survival time of the Session without user-interaction
   */
  public static long offlinetime =  7200000;  // 2h =  7200000ms without user-action
  public static long waitFor = 10000;         // Check it all 10 seconds

  private final static Random rnd = new Random(); // The Number-generator
  private final static Hashtable sessionList = new Hashtable();

  private final Hashtable pageList = new Hashtable();
  private RemoteBroker broker = BeanBroker.THIS;
  private boolean isNew = true;
  private boolean isOwn = true;

  private long tsCreated = System.currentTimeMillis();
  private long tsModified = tsCreated;
  private long tsRegistered = tsCreated;
  private long tsExpires = 0;
  private long tsEnded = 0;
  private long p = 0;                             // key of associated the Person-Bean
  private String status = String.valueOf(STATUS_DEFAULT)+String.valueOf(STATUS_REPLY_HTML)+String.valueOf(STATUS_REPLY_WEEKLY);
  private String id = String.valueOf( Math.abs( rnd.nextInt()));
  private String name;
  private String encr;
  private String lang;
  private String ip;
  private String eMail;
  private String url = "";

  /**
   * Checks whether a Session whith a given ID exists or not and sets the
   * modify-Timestamp.
   *@ throws IllegalAccessException if the ip-Adress doesen't match
   */
  public static boolean checkSession(String id, String ipAdr)  throws IllegalAccessException {
    boolean exist = sessionList.containsKey(id);
    if (exist) {
      UserSession s = (UserSession) sessionList.get(id);
      if (! s.getIpAdr().equals( ipAdr ))
        throw new IllegalAccessException("Illegal IP-Adress: "+s.getIpAdr());
    }
    return exist;
  }

  /**
   * Returns a Session whith a given ID exists or not and sets the
   * modify-Timestamp.
   *@ throws IllegalAccessException if the ip-Adress doesen't match
   */
  public static UserSession getSession(String id, String ipAdr)  throws IllegalAccessException {
    boolean exist = sessionList.containsKey(id);
    UserSession s = null;
    if (exist) {
      s = (UserSession) sessionList.get(id);
      s.modify();
      if (! s.getIpAdr().equals( ipAdr ))
        throw new IllegalAccessException("Illegal IP-Adress: "+s.getIpAdr());
    }
    return s;
  }

  /**
   * Returns a Session whith a given ID exists or not and sets the
   * modify-Timestamp.
   */
  public static UserSession getSession(String id) {
    boolean exist = sessionList.containsKey(id);
    UserSession s = null;
    if (exist) {
      s = (UserSession) sessionList.get(id);
      s.modify();
    }
    return s;
  }

  /**
   * Creates a new User, for each class which may create a UserSession has to be registrated explicitely (hardcoded).
   *@param creator the requesting Object (has to be a BeanBroker)
   *@param ipAdr IP-Adress of the user
   */
  public UserSession(HttpServletRequest request) {
    ip = request.getRemoteAddr();
  }

    /**
     * Checks if the given user has a valid password and throws an exception if not.
     *@param	userName name of the user
     *@param	pwd  the encrypted password
     *@param	ipAdr the Adress of the user
     *@returns the key of the person associated with the user
     *@exception    SkidException	if password doesen't match
     *@exception    RemoteException	if RMI doesen't work
     *@exception    SQLException the db doesen't work
     */
    public UserSession(String userName, String pwd, String ipAdr) throws SkidException, RemoteException, SQLException {
      ip = ipAdr;
      isNew = false;
      if ((userName == null) || (userName.length() < 1))
        throw new SkidException("An empty username is not valid!");
      if ((pwd == null) || (pwd.length() < 4))
        throw new SkidException("Invalid password!");
      String[] s = new String[11];
      s[0] = userName;
      s = (String[]) broker.getUser( s, userName, BeanBroker.encryptPwd(pwd) );
      if (s != null) {
        if (s[2].indexOf(UserSession.STATUS_DELETED) >= 0) {
          throw new SkidException("User no longer valid!");
        }
        try {
          name = s[0];
          encr = s[1];
          status = s[2];
          p = Integer.parseInt(s[3]);
          lang = s[4];
          eMail = s[5];
          url = s[6];
          tsRegistered = Long.parseLong(s[7]);
          tsExpires = Long.parseLong(s[8]);
          tsEnded = Long.parseLong(s[9]);
        } catch (Exception e) {
          throw new SkidException("User instanciation failed!");
        }
      } else {
        throw new SkidException("User not valid!");
      }
      sessionList.put(id, this);
      this.setPriority(Thread.MIN_PRIORITY);
      this.start();
    }

    /**
     * Checks the access-rights from the asking session and replys an instance in maintenance mode
     *@param	su the Administrator's session
     *@returns the search result if any
     *@exception    RemoteException	if RMI doesen't work
     *@exception    SQLException the db doesen't work
     */
    public UserSession(UserSession su, String searchName) throws SkidException, RemoteException, SQLException {
      if ((su==null) || (!su.checkStatus( UserSession.STATUS_ADMIN)))
        throw new SkidException("No valid UserSession");
      isNew = false;
      isOwn = false;
      String[] s = new String[11];
      s[0] = searchName;
      s = (String[]) broker.getUser( s, su.getUserName(), su.getEncrypt() );
      if (s != null) {
        name = s[0];
        encr = s[1];
        status = s[2];
        p = Integer.parseInt(s[3]);
        lang = s[4];
        eMail = s[5];
        url = s[6];
        tsRegistered = Long.parseLong(s[7]);
        tsExpires = Long.parseLong(s[8]);
        tsEnded = Long.parseLong(s[9]);
      } else {
        throw new SkidException("Username not found.");
      }
    }

  /**
   * Returns the ID of this user
   */
  public String getID() throws IllegalAccessException {
    if (isNew)
      throw new IllegalAccessException("Maintainance Mode.");
    modify();
    return id;
  }

  /**
   * Returns the IP-Adress of the current Connection
   */
  public String getIpAdr() throws IllegalAccessException {
    if (isNew)
      throw new IllegalAccessException("Maintainance Mode.");
    modify();
    return ip;
  }

  /**
   * Returns the Language of the current Connection
   */
  public String getLanguage() {
    modify();
    return lang;
  }

  /**
   * Returns the Name of this user
   */
  public String getUserName() {
    modify();
    return name;
  }

  /**
   * Returns the e-Mail adress
   */
  public String getEMail() {
    modify();
    return eMail;
  }

  /**
   * Returns the url of the homepage of the user
   */
  public String getURL() {
    modify();
    return url;
  }

  /**
   * Returns the encrypted password of the reply
   */
  public String getEncrypt() {
    modify();
    return encr;
  }

  /**
   * Returns when the session has been created
   */
  public long getCreated() {
    modify();
    return tsCreated;
  }

  /**
   * Returns when the session has been modified at last
   */
  public long getModified() {
    long m = tsModified;
    modify();
    return m;
  }

  /**
   * Sets when the account will expire
   */
  public void setExpiring(long when) {
    modify();
    tsExpires = when;
  }

  /**
   * Returns when the account will expire
   */
  public long getExpiring() {
    modify();
    return tsExpires;
  }

  /**
   * Returns when the session has been modified at last
   */
  public long getStarted() {
    modify();
    return tsRegistered;
  }

  /**
   * Returns when the session has been modified at last
   */
  public long getEnded() {
    modify();
    return tsEnded;
  }

  /**
   * Returns the key of the associated Person-Bean
   */
  public long getKey() {
    modify();
    return p;
  }

  /**
   * Sets the Person.Key of the current Connection
   *@param  languages set with valid languages
   */
  public boolean setKey(int key) {
    p = key;
    modify();
    return true;
  }

  /**
   * Sets the Language of the current Connection
   *@param  languages set with valid languages
   */
  public boolean setLanguage(String value, String[] languages) {
    boolean ok = false;
    if (value != null)
      if (languages == null)
        ok = true;
      else for (int i=0;i<languages.length;i++)
        if (languages[i].equalsIgnoreCase(value))
          ok = true;
    if (ok)
      lang = value.toUpperCase();
    modify();
    return ok;
  }

  /**
   * Sets the Name of this user only new names are accepted
   */
  public boolean setUserName(String value) {
    value = value.toUpperCase();
    boolean ok = false;
    try {
      if (value != null)
        ok = broker.checkUser(value, null) == null;
      if (ok)
        name = value;
    } catch (Exception e) { }
    modify();
    return ok;
  }

  /**
   * Sets the e-Mail adress
   */
  public boolean setEMail(String value) {
    modify();
    eMail = value;
    return true;
  }

  /**
   * Sets the url of the homepage of the user
   */
  public boolean setURL(String value) {
    modify();
    url = value;
    return true;
  }

  /**
   * Sets a new Password if long enough
   *@see MIN_PWD_SIZE
   */
  public boolean setPwd(String value, String verify) {
    modify();
    boolean ok = (value.length() >= MIN_PWD_SIZE) && (value.equals(verify));
    if (ok)
      encr = BeanBroker.encryptPwd( value );
    return ok;
  }

  /**
   * Sets the encrypted Password
   *@see MIN_PWD_SIZE
   */
  public boolean setEncrypt(String value) {
    modify();
    encr = value ;
    return true;
  }

  /**
   * Sets the status of this session
   */
  boolean setStatus(String value) {
    modify();
    status = value ;
    return true;
  }

  /**
   * Returns the status of this session
   */
  public String getStatus() {
    modify();
    return status;
  }

  /**
   * Checks if a given Atribute exists for this session
   *@see STATUS_*
   */
  public boolean checkStatus(char c) {
    modify();
    return status.indexOf(c) >= 0;
  }

  /**
   * Adds a specific Atribute to this this session
   *@see STATUS_*
   */
  public void addStatus(char c) {
    if (status.indexOf(c) < 0)
      status += String.valueOf( c );
    modify();
  }

  /**
   * Removes a specific Atribute from this this session
   *@see STATUS_*
   */
  public void removeStatus(char c) {
    int idx = status.indexOf(c);
    if (idx >= 0)
      status = status.substring(0, idx) + status.substring(idx+1, status.length());
    modify();
  }

  /**
   * Resets the Session
   */
  public void modify() {
    tsModified = System.currentTimeMillis();
  }

  /**
   * Removes the session from the list of valid sessions.
   */
  public void remove() {
    sessionList.remove(id);
  }

  /**
   * Cecks whether the Session shall be removed from the list of valid sessions or not.
   */
  public void run() {
    while ((tsModified + offlinetime) > System.currentTimeMillis())
      try {
        this.sleep( waitFor );
      } catch (InterruptedException e) { }
    remove();
  }

  /**
   * Adds the ID for a valid page in this session to the page-list
   *@returns the generates page-id
   */
  public String addPage(String pageName)  throws IllegalAccessException {
    if (isNew)
      throw new IllegalAccessException("Maintainance Mode.");
    String pid = String.valueOf(Math.abs(rnd.nextInt()));
    pageList.put(pageName, pid);
    modify();
    return pid;
  }

  /**
   * Adds the ID for a valid page in this session to the page-list
   *@returns the generates page-id
   */
  public boolean save()  throws SkidException, SQLException, RemoteException {
//  public boolean save(UserSession su)  throws SkidException, SQLException, RemoteException {
    String[] s = new String[11];
    s[0] = getUserName();
    s[1] = getEncrypt();
    s[2] = getStatus();
    s[3] = String.valueOf( getKey());
    s[4] = getLanguage();
    s[5] = getEMail();
    s[6] = getURL();
    s[7] = String.valueOf( tsRegistered );
    s[8] = String.valueOf( tsExpires );
    s[9] = String.valueOf( tsEnded );
    s[10] = "save";
//    return broker.putUser(s, su.getUserName(), su.getEncrypt() );
    return broker.putUser(s);
  }
  /**
   * Checks the page-ID if it is a valid page in this session
   */
  public boolean checkPage(String pageName, String pid) throws IllegalAccessException {
    if (isNew || !isOwn)
      throw new IllegalAccessException("Maintainance Mode.");
    modify();
    if (pageName == null)
      return pageList.contains(pid);
    else if (pid == null)
      return pageList.containsKey(pageName);
    else
      return pageList.get(pageName).equals(pid);
  }

}