/**
 *@autor    Klaus Hammermller
 *@created  020398
 */

package asgaard.utils.encode;
import java.io.IOException;

    /**
     * Converts ASCII-HEX-String in bytes.
     */
public class HexDecInputStream extends CodedInputStream {

    private static final String tab = "0123456789ABCDEF";
    private int pos = 0;          // position in buffer
    private int av = 0;           // available
    private int mark_pos = 0;
    private int mark_av = 0;
    private char c0;
    private char c1;

    /**
     * sets a HEX-ASCII-String to buf
     */
    public void writeBuffer( String s ) {
      buf = s;
      av = s.length() / 2;
      pos = 0;
      mark_pos = pos;
      mark_av = av;
    }

    /**
     * Dummy
     */
    public String getCoded() {
      return buf;
    }

    /**
     * Returns the number of bytes that can be read from this input stream without blocking.
     */
    public int available() throws IOException {
      if (in == null) {
        if (av < 0) av = 0;
        return av;
      } else {
        return (in.available() + 1) / 2;
      }
    }

    /**
     * Reads the next byte of data from this input stream. The value byte is returned
     * as an int in the range 0 to 255. If no byte is available because the end of the
     * stream has been reached, the value -1 is returned.
     */
    public int read() throws IOException {
      if (in == null) {
        if (av < 1)
          return -1;
        c0 = buf.charAt(pos);
        c1 = buf.charAt(pos+1);
        av -= 1;
        pos += 2;
      } else {
        if (in.available() < 2)
          return -1;
        c0 = (char) in.read();
        c1 = (char) in.read();
      }
      return (int) (tab.indexOf(c0)*16 + tab.indexOf(c1));
    }

    /**
     * Skips over and discards n bytes of data from this input stream. The skip method may,
     * for a variety of reasons, end up skipping over some smaller number of
     * bytes, possibly 0. The actual number of bytes skipped is returned.
     *
     *@param n the number of bytes to be skipped.
     *@return the actual number of bytes skipped.
     *@throw IOException if an I/O error occurs.
     */
    public long skip(long n) throws IOException {
      if (in == null) {
        if (n > av) {
          pos += av * 2;
          long s = av;
          av = 0;
          return s;
        } else {
          pos += n * 2;
          av -= n;
          return n;
        }
      } else {
        int l = 0;
        while ((l<n) && !(read()<0)) ;
        return l;
      }
    }

    /**
     * Marks the current position in this input stream. A subsequent call to the reset
     * method repositions this stream at the last marked position so that subsequent
     * reads re-read the same bytes.
     */
    public synchronized void mark(int readlimit) {
      readlimit = av;
      mark_pos = pos;
      mark_av = av;
    }

    /**
     * Repositions this stream to the position at the time the mark method was last
     * called on this input stream.
     */
    public synchronized void reset() throws IOException {
      pos = mark_pos;
      av = mark_av;
    }

    /**
     * Tests if this input stream supports the mark and reset methods (TRUE).
     */
    public boolean markSupported() {
      return (in == null);
    }
}
