/**
 *@autor    Klaus Hammermller
 *@created  030399
 */
package asgaard.utils.encode;

//used jdk packages
import java.lang.*;
import java.io.*;
import java.util.zip.*;

/**
 * Is a Encoder, using <code>HexEncOutputStream</code> to encode an object by
 * serialising and streaming it.
 * <p>
 * To prevent creating a new ObjectOutputStream (we need the full header) ObjectOutputStream
 * has to be overwritten to add a reset - method.
 *
 *@see asgaard.utils.encode.HexEncOutputStream
 */
public class ObjectEncoder {

  private CodedOutputStream enc;        // The Encoder Stream
  private ObjectOutputStream oo;        // The output-Stream
  private boolean reset = false;        // after reset skip the first byte

  /**
   * Creates a new Decoder.
   */
  public ObjectEncoder( ) {
    try {
      enc = new HexEncOutputStream( );
      oo = new ObjectOutputStream( enc );
    } catch (Exception e) {
      System.err.println(e.toString() + " " + e.getMessage());
    }
  }

  /**
   * Set the Object.
   */
  public void setObject( Object o ) throws IOException {
    oo.writeObject( o );
  }

  /**
   * Get the encoded String.
   */
  public String getEncoded( ) throws IOException {
    String s = enc.readBuffer();
    if (reset)
      s = s.substring( 2, s.length() );
    oo.reset();
    reset = true;
    return s;
  }
}