/**
 * This file is part of Skid and Log
 * (P) 1998 by Klaus Hammermller, klaus@ifs.tuwien.ac.at
 * http://www.ifs.tuwien.ac.at/~hammer
 *
 *@version  1.0 270299
 *@autor    Klaus Hammerller
 *@created	270299	KH
 *@modified 060399  KH fixing trouble with BeanDecoder
 *@modified 080399  KH adding FixedMsgSend (Mailservice)
 */
package asgaard.utils.log;

// Import the java classes used
import java.io.*;
import java.util.*;
import java.net.*;
// Import the JDBC classes
import java.sql.*;
import jdbc.SimpleText.*;
// Import SkidAPI
import asgaard.utils.*;

import asgaard.utils.encode.*;

import asgaard.utils.mail.*;


/**

 * This is the Collector (and distributor) for all Log-statements
 * based on LogMsg-Beans.
 *
 *@see asgaard.utils.log.LogMsg
 */
public class LogBroker implements RemoteLog {

  public static final String VERSION = "1.0.3";
  /**
   * Used flatfile-driver
   */
  public static final String CONNECT_STRING =	"jdbc:SimpleText";
  /**
   * Used flatfile-filename
   */
  public static final String LOG_FILE_NAME =	"log";
  /**
   * Used flatfile-file-extension
   */
  public static final String LOG_FILE_EXTENSION =	".sdf";
  /**
   * Used prefix of the mail-subject (used if mailclient active)
   */
  public static final String MAIL_SUBJECT =	"LOGBROKER ";
  private String LocalHost = LogMsg.LOCAL_HOST;
  private boolean debug = false;

  // For Bean-String conversion
  private static final char ESC = '^';
  private static final int OFFS = 32;

  /**
   * Path to the Logfile (may be modified before instantiation).
   */
  public static String local_path = ".log";                        // default path for flatfile - db
  private Connection	conn;			                                      // The connection to the database
  private long msgCounter = 0;                                        // Zhlt die Eintrge im Logfile

  private ObjectDecoder dec = new ObjectDecoder( );                   // set UUDecoder
  private ObjectEncoder enc = new ObjectEncoder( );                   // set UUEncoder
  private byte MailLevel = LogMsg.LOG_NONE;                           // Which Messages are sent via eMail too?
  private FixedMsgSend mailer;                                        // The Mailer for sending mails
  private long LOGsize = 0;                                           // Limit to send a warning if Logsize exceeds Size
  private boolean log_warning = false;                                // Send a warning if Logsize exceeds Size?
  private File testfile;
  private boolean reconnect = true;                                   // Try to reconnect to LOGFILE (e.g. if moved)
  private boolean reject = false;                                     // reject if there is no Logfile available

  /**
   * Creating a LogBroker Object with the default settings,
   * LOGFILE is chreated automatically if nessecary.
   *@exception SQLException if the JDBC connection to the flatfile (logfile) failes
   *@exception IOException if the access to the flatfile (logfile) failes
   */
  public LogBroker() throws SQLException, IOException {
    connectLogfile( 1 );
  }

  /**
   * Creating a LogBroker, LOGFILE is chreated automatically if nessecary.
   *@exception SQLException if the JDBC connection to the flatfile (logfile) failes
   *@exception IOException if the access to the flatfile (logfile) failes
   */
  public LogBroker(String path) throws SQLException, IOException {
    local_path = path;
    connectLogfile( 1 );
  }

  /**
   * Doing the connection to the LOGFILE, if it does not exist a new LOGFILE will be created.
   *@exception SQLException if the JDBC connection to the flatfile (logfile) failes
   *@exception IOException if the access to the flatfile (logfile) failes
   */
  private void connectLogfile( long startNr) throws SQLException, IOException {
	  SimpleTextDriver D = new SimpleTextDriver();                      // Load the JDBC driver
    Properties dir = new Properties();
    if (!local_path.endsWith(File.separator)) local_path += File.separator;
      File testdir = new File(local_path);                         // Check if directory exists
    if (! testdir.exists())
      testdir.mkdir();
    dir.put("Directory", local_path);                              // set LOG-file
    conn = DriverManager.getConnection (CONNECT_STRING, dir);         // establishing the connection
    Statement stmt = conn.createStatement ();	                        // Create a statement
	  try {						                                                  // Hallo LOG-file?
		  ResultSet rset = stmt.executeQuery (" SELECT LogNr FROM " + LOG_FILE_NAME );
	    while (rset.next()) msgCounter = rset.getInt( 1 );                            // get last Message
      msgCounter += 1;                                                // Set Number
      reject = false;
	  } catch (Exception e) {
      try {                                                           // CREATE NEW LOGFILE
        stmt.executeQuery ( " DROP TABLE " + LOG_FILE_NAME );         // remove old (if exists)
      } catch (Exception f) { }
      java.util.Date now = new java.util.Date(System.currentTimeMillis());
      try {
        try {
          InetAddress local = InetAddress.getLocalHost(); // name of the Host the requesting Server is located
          LocalHost = local.getHostName() + " ["+local.getHostAddress()+"]";
        } catch (UnknownHostException g) { }
		    stmt.executeQuery ( " CREATE TABLE " + LOG_FILE_NAME +         // create table
                                               " ( LogNr INTEGER, " +
                                               " HostName VARCHAR, " +
                                               " ServerClass VARCHAR, " +
                                               " ServicePoint VARCHAR, " +
                                               " Type VARCHAR, " +
                                               " Success VARCHAR, " +
                                               " LogMessage VARCHAR, " +
                                               " Created VARCHAR, " +
                                               " TsDiff LONG, " +
                                               " Received LONG, " +
                                               " LogType INTEGER, " +
                                               " BeanType INTEGER, " +
                                               " LogBean VARCHAR ) ");
      } catch (SQLException g) {                                       // failed: create a statement
        reject = true;
        System.err.println("Cannot Create new Logfile: " + local_path+LOG_FILE_NAME+LOG_FILE_EXTENSION);
        if (mailer != null) {
          mailer.newMsg(MAIL_SUBJECT + LogMsg.getLogType( LogMsg.LOG_ERROR ) + ": Cannot Create new Logfile",
                          "Host:          " + LocalHost +
                        "\nServer Class:  " + this.getClass().getName() +
                        "\nService Point: createStatement#1 ["+ LogMsg.getOk(false) + "]" +
                        "\nCreated:       " + now.toString() + "\n\n" +
                        "Cannot Create new Logfile [" +local_path+LOG_FILE_NAME+LOG_FILE_EXTENSION+"]\n\n"+
                         g.toString() + " " + g.getMessage());
        }
        throw g;
      }
      msgCounter = startNr;                                           // Write first line: Creation message
      insertStmt( LocalHost,
                  this.getClass().getName(),
                  "CreateLog",
                  LogMsg.getLogType( LogMsg.LOG_REMARK ),
                  now.toString(),                                              // Time as String
                  String.valueOf(now.getTime()%999),                           // Tausendstel
                  String.valueOf(now.getTime()), // Time as Seconds
                  LogMsg.getOk(true),
                  "LOGFILE ("+ local_path+LOG_FILE_NAME+LOG_FILE_EXTENSION +") V"+ VERSION +" created.",
                  Byte.toString( LogMsg.LOG_REMARK ),
                  Byte.toString( LogMsg.BEAN_NONE ),
                  "");                                                // Create a Statement
      System.out.println("New Logfile created: " + local_path+LOG_FILE_NAME+LOG_FILE_EXTENSION);
      if ((mailer != null) && (MailLevel > LogMsg.LOG_ERROR)) {
        mailer.newMsg(MAIL_SUBJECT + LogMsg.getLogType( LogMsg.LOG_WARNING ) + ": New Logfile created",
                        "Host:          " + LocalHost +
                      "\nServer Class:  " + this.getClass().getName() +
                      "\nService Point: createStatement#2 ["+ LogMsg.getOk(true) + "]" +
                      "\nCreated:       " + now.toString() + "\n\n" +
                      "New Logfile created [" +local_path+LOG_FILE_NAME+LOG_FILE_EXTENSION+"]");
      }
      reject = false;
    }
    testfile = new File (local_path + LOG_FILE_NAME + LOG_FILE_EXTENSION);// For size-check
  }

  /**
   * Defines a Mailserver for automatically forward LOGMSG also via eMail.
   *
   *@param level Which Error-Level should be forwarded
   *@param to mailto - eMailadress
	 *@param from eMailadress
	 *@param cc carbon copy (optional)
	 *@param mailhost assumption: mail.smtp.host
   *@param logsize send warning if logfile exceeds size (0 for no warning)
   */
  public void setMailServer(byte level, String to, String from, String cc, String mailhost, long size) {
    mailer = new FixedMsgSend(to, from, cc, mailhost);
    MailLevel = level;
    LOGsize = size;
    log_warning = (LOGsize > 0);
  }

  /**
   * Get the current MailLevel
   *
   *@see asgaard.utils.log.LogMsg
   */
  public byte getMailLevel() {
    return MailLevel;
  }

  /**
   * Puts a Statement into the LOGFILE returning the (modified) Statement-bean.
   * If the Statement-bean exists already, it is replaced.
   *
   *@see asgaard.utils.log.LogMsg
   *@param stmt Statement as a LogMsg-Bean
   *@exception SQLException if the JDBC connection to the flatfile (logfile) failes
   *@exception IOException if the access to the flatfile (logfile) failes
   */
  public void put(LogMsg stmt) throws SQLException, IOException {
    if (reject) throw new IOException("No logfile available, creation failed");
    if (stmt == null) throw new IOException("No Log-statement available");
    if (stmt.getLogNr() == 0) {
      stmt.setLogNr( msgCounter );
      stmt.setReceived( new java.util.Date(System.currentTimeMillis()) );
      try {
        insertStmt( stmt.getHostName(),
                    stmt.getServerClass(),
                    stmt.getServicePoint(),
                    LogMsg.getLogType( stmt.getLogType() ),
                    stmt.getCreated().toString(),
                    String.valueOf( stmt.getReceived().getTime()-stmt.getCreated().getTime()),
                    String.valueOf( stmt.getReceived().getTime()),
                    stmt.getOk(),
                    bean2string( stmt.getLogMessage()),
                    String.valueOf( stmt.getLogType() ),
                    String.valueOf( stmt.getBeanType() ),
                    bean2string( stmt.getLogBean() ));
      } catch (SQLException e) {
        if (reconnect) {                              // Try to reconnect (e.g. if moved)
          connectLogfile(msgCounter);
          reconnect = false;
          stmt.setLogNr( 0 );
          put( stmt );
        } else                                        // Failed: Error Message
          System.err.println(MAIL_SUBJECT + LogMsg.getLogType(LogMsg.LOG_ERROR) + ": Can not store LOG message");
          mailer.newMsg(MAIL_SUBJECT + LogMsg.getLogType(LogMsg.LOG_ERROR) + ": Can not store LOG message",
                          "Host:          " + LocalHost +
                        "\nServer Class:  " + this.getClass().getName() +
                        "\nService Point: put#1 ["+ LogMsg.getOk(false) + "]" +
                        "\nCreated:       " + stmt.getReceived().toString() + "\n" +
                        "\nCant't store LOG message:\n"+
                           e.toString() + " " + e.getMessage()+"\n\n" +
                        "\n" + LogMsg.getLogType(stmt.getLogType()) +
                        "\nHost:          " + stmt.getHostName() +
                        "\nServer Class:  " + stmt.getServerClass() +
                        "\nService Point: " + stmt.getServicePoint() + " ["+ stmt.getOk() + "]" +
                        "\nCreated:       " + stmt.getCreated().toString() + "\n" +
                        "\n" +stmt.getLogMessage());
          throw e;
      }
      if (MailLevel >= stmt.getLogType()) {
        mailer.newMsg(MAIL_SUBJECT + LogMsg.getLogType(stmt.getLogType()) + ": " + stmt.getLogMessage(),
                        "Host:          " + stmt.getHostName() +
                      "\nServer Class:  " + stmt.getServerClass() +
                      "\nService Point: " + stmt.getServicePoint() + " ["+ stmt.getOk() + "]" +
                      "\nCreated:       " + stmt.getCreated().toString() + "\n" +
                      "\n" +stmt.getLogMessage());
      }
    } else {
      stmt.setReceived( new java.util.Date(System.currentTimeMillis()) );
      updateStmt( String.valueOf( stmt.getLogNr() ),
                  stmt.getHostName(),
                  stmt.getServerClass(),
                  stmt.getServicePoint(),
                  LogMsg.getLogType( stmt.getLogType() ),
                  stmt.getCreated().toString(),
                  String.valueOf( stmt.getReceived().getTime()-stmt.getCreated().getTime()),
                  String.valueOf( stmt.getReceived().getTime()),
                  stmt.getOk(),
                  bean2string( stmt.getLogMessage()),
                  String.valueOf( stmt.getLogType() ),
                  String.valueOf( stmt.getBeanType() ),
                  bean2string( stmt.getLogBean() ));
    }
    if (log_warning) {
      if (testfile.length() > LOGsize) {         // Check ob File too big
        java.util.Date now = new java.util.Date();
        mailer.newMsg(MAIL_SUBJECT + LogMsg.getLogType( LogMsg.LOG_WARNING ) + ": Logfile Size Extended (" +String.valueOf(testfile.length()/1024)+" kb)",
                        "Host:          " + LocalHost +
                      "\nServer Class:  " + this.getClass().getName() +
                      "\nService Point: put#2 ["+ LogMsg.getOk(true) + "]" +
                      "\nCreated:       " + now.toString() + "\n\n" +
                      "Logfile ["+local_path+LOG_FILE_NAME+LOG_FILE_EXTENSION+"]Size Extended (" +String.valueOf(testfile.length())+" bytes)");
        log_warning = false;
      }
    } else {
      if (LOGsize > 0)                          // Bei neuem Logfile Warnung wieder einschalten
        if (testfile.length() < LOGsize)
          log_warning = true;
    }
    reconnect = true;
  }

  /**
   * Gets a Statement from the LOGFILE returning the Statement-bean.
   *
   *@see asgaard.utils.log.LogMsg
   *@param nr Number of the statement in the current Logfile
   *@param withBean if the Bean carried by the Statement is required
   *@return a List with the found Statement-Beans
   */
  public LogMsg[] get(long nr, boolean withBean) {
    return selectStmt("LogNr = "+String.valueOf(nr), withBean);
  }

  /**
   * Gets a Statement from the LOGFILE returning the Statement-bean. (<b>NOT TESTED YET!</b>)
   *
   *@see asgaard.utils.log.LogMsg
   *@param date minimum date of the statement in the current Logfile
   *@param type minimum type of the statement in the current Logfile
   *@param withBean if the Bean carried by the Statement is required
   *@return a List with the found Statement-Beans
   */
  public LogMsg[] get(java.util.Date date, byte type, boolean withBean) {
    return selectStmt("(Received >= "+String.valueOf(date.getTime())+") and (LogType >= "+String.valueOf(type)+")", withBean);
  }

  /**
   * Appends the Statement to the Logfile
   *
   *@param host       name of the Host the requesting Server is located
   *@param server     The calling Java-Serverclass
   *@param point      The Method / place where the statement was created
   *@param typ        The Type of the LOG-Statement (String)
   *@param ts_c       Timestamp when the statement was created (Date)
   *@param ts_d       Difference betwenn ts_c and ts_r (Long)
   *@param ts_r       Timestamp when the statement was received (Long)
   *@param ok         the Servicepoint was passed successfully (ok, fail)
   *@param msg        Human redable Message of the Statement
   *@param lt         The Type of the LOG-Statement (byte)
   *@param bt         Type of the streamable Object
   *@param bean       Some streamable Object
   */
  private void insertStmt(String host, String server, String point, String typ, String ts_c, String ts_d, String ts_r, String ok, String msg, String lt, String bt, String bean)  throws SQLException {
    Statement stmt = conn.createStatement ();	                        // Create a statement
	  stmt.executeQuery ( " INSERT INTO " + LOG_FILE_NAME +
                                     "  VALUES ( " + Long.toString( msgCounter ) + ", " +
                                                 "'" + host + "', " +
                                                 "'" + server + "', " +
                                                 "'" + point + "', " +
                                                 "'" + typ + "', " +
                                                 "'" + ok + "', " +
                                                 "'" + msg + "', " +
                                                 "'" + ts_c + "', " +
                                                       ts_d + ", " +
                                                       ts_r + ", " +
                                                       lt + ", " +
                                                       bt + ", " +
                                                 "'" + bean + "' ) " );
    msgCounter += 1;
  }
  /**
   * Updates an existing Statement to the Logfile
   *
   *@param nr         Message-Number
   *@param host       name of the Host the requesting Server is located
   *@param server     The calling Java-Serverclass
   *@param point      The Method / place where the statement was created
   *@param typ        The Type of the LOG-Statement (String)
   *@param ts_c       Timestamp when the statement was created (Date)
   *@param ts_d       Difference betwenn ts_c and ts_r (Long)
   *@param ts_r       Timestamp when the statement was received (Long)
   *@param ok         the Servicepoint was passed successfully (ok, fail)
   *@param msg        Human redable Message of the Statement
   *@param lt         The Type of the LOG-Statement (byte)
   *@param bt         Type of the streamable Object
   *@param bean       Some streamable Object
   *@exception SQLException if the JDBC connection to the flatfile (logfile) failes
   */
  private void updateStmt(String nr, String host, String server, String point, String typ, String ts_c, String ts_d, String ts_r, String ok, String msg, String lt, String bt, String bean)  throws SQLException {
    Statement stmt = conn.createStatement ();	                        // Create a statement
	  stmt.executeQuery ( " UPDATE " + LOG_FILE_NAME +
                                     " SET LogNr = " + nr + ", " +
                                          "HostName = '" + host + "', " +
                                          "ServerClass = '" + server + "', " +
                                          "ServicePoint = '" + point + "', " +
                                          "Type = " + typ + ", " +
                                          "Success = '" + ok + "', " +
                                          "LogMessage = '" + msg + "', " +
                                          "Created ='" + ts_c + "', " +
                                          "TsDiff =" + ts_d + ", " +
                                          "Received = " + ts_r + ", " +
                                          "LogType = " + lt + ", " +
                                          "BeanType = " + bt + ", " +
                                          "LogBean = '" + bean + "' " +
                                            "WHERE LogNr = " + nr );
  }

  /**
   * Selects a set of Statements out of the Logfile
   *
   *@param where  a SQL-WHERE without parensis
   *@param withBean if the Bean carried by the Statement is required
   *@return a List with the found Statement-Beans
   */
  private LogMsg[] selectStmt(String where, boolean withBean) {
    try {
      Statement stmt = conn.createStatement ();	                        // Create a statement
	    ResultSet rst = stmt.executeQuery ( " SELECT * FROM " + LOG_FILE_NAME + " WHERE " + where);
      Vector s = new Vector();
      if (withBean) {
        while (rst.next()) {
          s.addElement( new LogMsg( rst.getLong( 1 ),           // lognr
                                          rst.getString( 2 ),   // hostname
                                          rst.getString( 3 ),   // serverclass
                                          rst.getString( 4 ),   // servicepoint
                                          new java.util.Date( rst.getLong( 10 )-rst.getLong( 9 ) ),  // created
                                          new java.util.Date( rst.getLong( 10 )),                    // received
                                          (byte) rst.getInt( 11 ),          // logtype
                                          LogMsg.getOk(rst.getString( 6 )), //ok
                                          rst.getString( 7 ),               // message
                                          string2bean( rst.getString( 13 ), (byte) rst.getInt( 11 )),// bean
                                          true ));                          // beanexist
        }
      } else {
          s.addElement( new LogMsg( rst.getLong( 1 ),
                                          rst.getString( 2 ),
                                          rst.getString( 3 ),
                                          rst.getString( 4 ),
                                          new java.util.Date( rst.getLong( 10 )-rst.getLong( 9 ) ),
                                          new java.util.Date( rst.getLong( 10 )),
                                          (byte) rst.getInt( 11 ),
                                          LogMsg.getOk( rst.getString( 6 )),
                                          rst.getString( 7 ),
                                          "",
                                          rst.getInt( 12 ) > LogMsg.BEAN_NOT_INCLUDED ));
      }
      LogMsg[] res = new LogMsg[s.size()];
      for (int i = 0; i < s.size(); i++)
        res[i] = (LogMsg) s.elementAt(i);
      return res;
    } catch (Exception e) {
      System.err.print( e.getMessage() );
      if (debug) e.printStackTrace();
      return null;
    }                                                                  // ignore it
  }
  /**
   * Converts a Bean to a String
   */
  private String bean2string(Object bean) {
    if (bean == null) return "";
    if (bean instanceof java.lang.String) {                            // CHECK Strings for forbidden CHARS
      String s = (String) bean;
      int i=0;
      while (i<s.length()) {
        char c = s.charAt(i);                                          // check quotes & unprintable chars
        if ((c<(char) OFFS) || (c==(char)(39)) || (c=='"') || (c=='\'') || (c==ESC) || (c=='\t') || (c=='\n') || (c=='\r'))
          s = s.substring(0, i) + ESC + (char) (c + OFFS) + s.substring(i+1, s.length());
        i++;
      }
      return s;
    }
    try {                                                              // DO NON-STRINGS
      enc.setObject( bean );		                                       //   ... und ab!
      return enc.getEncoded();                                         // ... it's magic
    } catch (Exception e) {
      System.err.print( e.getMessage() );
      if (debug) e.printStackTrace();
      return "";
    }
  }

  /**
   * Converts a String to a Bean
   */
  private Object string2bean(String bean, byte type) {
    if (bean == null) return "";
    if (type == LogMsg.BEAN_STRING) {                                  // DECODE STRINGS
      for (int i=0;i<bean.length();i++) {
        if (bean.charAt(i) == ESC) {
          bean = bean.substring(0, i) + (char) (bean.charAt(i+1)-OFFS) + bean.substring(i+2, bean.length());
        }
      }
      return bean;
    }
    try {                                                              // DO NON-STRINGs
      dec.setEncoded( bean );		                                       //   ... und ab!
      return dec.getObject();                                          // ... it's magic
    } catch (Exception e) {
      System.err.print( e.toString() + " " + e.getMessage() );
      if (debug) e.printStackTrace();
      return "";
    }
  }
}

