/**
 * This file is part of Skid and Log
 * (P) 1998 by Klaus Hammermller, klaus@ifs.tuwien.ac.at
 * http://www.ifs.tuwien.ac.at/~hammer
 *
 *@version  1.0 170399
 *@autor    Klaus Hammerller
 *@created	170399	KH
 */


package asgaard.utils.log;

// Import the java classes used in applets
import java.rmi.*;
import java.io.IOException;
import java.sql.SQLException;

/**
 * This is the Remote interface for the LOG-server
 */
public interface RemoteLog extends Remote {

  /**
   * Defines a Mailserver for automatically forward LOGMSG also via eMail.
   *
   *@param level Which Error-Level should be forwarded
   *@param to mailto - eMailadress
	 *@param from eMailadress
	 *@param cc carbon copy (optional)
	 *@param mailhost assumption: mail.smtp.host
   *@param logsize send warning if logfile exceeds size (0 for no warning)
   */
  public void setMailServer(byte level, String to, String from, String cc, String mailhost, long size) throws RemoteException;

  /**
   * Puts a Statement into the LOGFILE.
   * If the Statement exists already, it is replaced.
   * If any Exception rises it will be ignored
   *
   *@see asgaard.utils.log.LogMsg
   *@param stmt Statement as a LogMsg-Bean
   *@exception SQLException from LogBroker.put forwarded exception
   *@exception IOException from LogBroker.put forwarded exception
   *@exception RemoteException if the remote (only if remote!) Object cannot be accessed
   */
  public void put(LogMsg stmt) throws RemoteException, IOException, SQLException;

  /**
   * Gets a Statement from the LOGFILE returning the Statement-bean.
   *
   *@see asgaard.utils.log.LogMsg
   *@param nr Number of the statement in the current Logfile
   *@param withBean if the Bean carried by the Statement is required
   *@return a List with the found Statement-Beans
   *@exception RemoteException if the remote (only if remote!) Object cannot be accessed
   */
  public LogMsg[] get(long nr, boolean withBean) throws RemoteException;

}
