/*
 *@template msgsend.java	1.12 98/11/23 from mail-api 1.1
 *@author KH
 *@created 060399
 *
 * REQUIRED : mail.jar, activation.jar
 *
 * @(#)msgsend.java	1.12 98/11/23
 *
 * Copyright (c) 1996-1998 by Sun Microsystems, Inc.
 * All Rights Reserved.
 */

package asgaard.utils.mail;

import java.io.*;
import java.net.InetAddress;
import java.util.*;

import javax.mail.*;
import javax.mail.internet.*;

/**
 * Modified demo-app that shows how to construct and send an RFC822
 * (singlepart) message.
 *
 * @author Max Spivak
 * @author Bill Shannon
 */

public class FixedMsgSend extends Thread {

	protected String to;
	protected final String  from;
	protected final String  cc;
	protected final String mailhost;

  protected Vector queue = new Vector();

  /**
   * Turns the mailer into debug - mode, displays msg on System.out
   */
  protected boolean debug = false;
  protected String charset = MimeUtility.mimeCharset(MimeUtility.getDefaultJavaCharset());
	protected final String mailer = this.toString();
//	protected final String mailer = "FixedMsgSend";

  /**
   * Creates a new fixed mailer (only subject and message may varie).
   *
   *@param to mailto - eMailadress
	 *@param from eMailadress
	 *@param cc carbon copy (optional)
	 *@param mailhost assumption: mail.smtp.host
   */
  public FixedMsgSend (String to, String from, String cc, String mailhost) {
	  this.to = to;
	  this.from = from;
	  this.cc = cc;
	  this.mailhost = mailhost;
    this.setPriority(Math.max(this.getPriority()-1, Thread.MIN_PRIORITY));
  }

  /**
   * Creates a new fixed mailer (only subject and message may varie).
   *
   *@param to mailto - eMailadress
	 *@param from eMailadress
	 *@param cc carbon copy (optional)
	 *@param mailhost assumption: mail.smtp.host
   *@param start starts the thread
   */
  public FixedMsgSend (String to, String from, String cc, String mailhost, boolean start) {
    this(to, from, cc, mailhost);
    if (start)
      start();
  }

  public void setDebugMode(boolean on) {
    debug = on;
  }

  public void setMimeCharset(String mime) {
    charset = mime;
  }

  /**
   * Sends a Message to the specified target, using the spooler (if the thread is running).
   */
  public void spoolMsg(String to, String subject, String messagebody) {
    String[] args = new String[] { to, subject, messagebody };
    queue.addElement( args );
    this.resume();
  }

  /**
   * Sends a Message to the specified target.
   */
  public void newMsg(String subject, String messagebody) {
	  try {

	    Properties props = System.getProperties();
	    // XXX - could use Session.getTransport() and Transport.connect()
	    // XXX - assume we're using SMTP
	    if (mailhost != null)
		    props.put("mail.smtp.host", mailhost);

	    // Get a Session object
	      Session session = Session.getDefaultInstance(props, null);
	    if (debug)
		    session.setDebug(true);

	    // construct the message
	    MimeMessage msg = new MimeMessage(session);
	    if (from != null)
		    msg.setFrom(new InternetAddress(from));
	    else
		    msg.setFrom();

	    msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(to, false));
	    if (cc != null)
		    msg.setRecipients(Message.RecipientType.CC, InternetAddress.parse(cc, false));

	    msg.setSubject(subject);
	    msg.setText(messagebody, charset);
	    msg.setHeader("X-Mailer", mailer);
	    msg.setSentDate(new Date());

	    // send the thing off
	    Transport.send(msg);

      if (debug) System.out.println("\nMail was sent successfully.");

	  } catch (Exception e) {
      if (debug) System.err.println(e.toString() + " " + e.getMessage());
	    e.printStackTrace();
	  }
  }

  /**
   * Does the spooling when the mailer has been started
   */
  public void run() {
    while (true) {
      if (!queue.isEmpty()) {           // Get the first Element from the Vector
        String[] msg = (String[]) queue.firstElement();
        queue.removeElementAt( 0 );
        to = msg[0];
        newMsg(msg[1], msg[2]);
      } else
        this.suspend();
    }
  }
}
