/**
 * This file is part of Asgaard implementing an Mechanism for Testing Software
 * (P) 1998 by Klaus Hammermller, klaus@ifs.tuwien.ac.at
 * http://www.ifs.tuwien.ac.at/~hammer
 *
 *@version  1.0 030899
 *@autor    Klaus Hammerller
 *@created	030899	KH
 */

package asgaard.utils.test;

import java.util.*;
import asgaard.utils.log.*;

/**
 * This Class implements an Mechanism for testing Software.
 * All <b>you</b> need to do is to implement the <code>TestIt</code> interface
 * (for each class) and, of course, write some <b>Testscripts</b>
 *@see asgaard.utils.test.Test.TESTSCRIPT
 *@see asgaard.utils.test.TestIt
 */
public class Test implements TestIt {

  public final static String SERVER = "TEST";
  public static String mailTo = "klaus@ifs.tuwien.ac.at";
  public static String mailHost = "garbo.ifs.tuwien.ac.at";
  public static String logDir = ".testlog";

  protected static LogClient log = null;
  protected static String tName = "(none)";
  protected static String sName = "(none)";
  protected static int testCount = 0;
  protected static int testFail = 0;
  protected static boolean started = false;
  protected static Hashtable result = new Hashtable();
  protected static Object local_object = null;

  private final static byte test_method = 0;
  private final static byte test_method_local = 1;
  private final static byte test_compare_equal = 2;
  private final static byte test_get_array_item = 3;
  public final static Byte TEST_METHOD = new Byte( test_method );
  public final static Byte TEST_METHOD_LOCAL = new Byte( test_method_local );
  public final static Byte TEST_COMPARE_EQUAL = new Byte( test_compare_equal );
  public final static Byte TEST_GET_ARRAY_ITEM = new Byte( test_get_array_item );

  /**
   * Prefix that indicates that not the given String but the Result from the
   * preliminary test has to be invoked.
   */
  public final static String RES_OBJ = "robj$";
  /**
   * Describes the call of an Object/Method.
   *<p>
   * TESTSCRIPT for Method- and Constructorcalls:
   *<ul>
   *<li> TEST_METHOD
   *<li> NAME of the Teststatement
   *<li> INSTANCE or CLASSNAME of the tested Object
   *<li> METHODNAME (null for a constructor)
   *<li> SET of PARAMETERS
   *<li> SET of PARAMETERS-CLASSNAMES (optional NoSuchMethodException rises)
   *</ul>
   *<p>
   * TESTSCRIPT for Compares to EQUAL:
   *<ul>
   *<li> TEST_COMPARE_EQUAL
   *<li> NAME of the Teststatement
   *<li> INSTANCE 1 <code>Object</code>
   *<li> INSTANCE 2 <code>Object</code>
   *</ul>
   *<p>
   * TESTSCRIPT to get an Item out of an array:
   *<ul>
   *<li> TEST_GET_ARRAY_ITEM
   *<li> NAME of the Teststatement
   *<li> Array-object <code>Object[]</code>
   *<li> index <code>Integer</code>
   *</ul>
   *<p>
   * Instead of an Instance for the Script the RES_OBJ + NAME of the Teststatement
   * will be replaced by the result that line has produced.
   *<p>
   *e.g. <br>
   *<code>
  public final static Object[][] TESTSCRIPT = {<br><dir>
    { TEST_METHOD, "newString#1", "java.lang.String", null, new Object[] {"EQUAL"} },<br>
    { TEST_METHOD, "newString#2", "java.lang.String", null, new Object[] {"EQUAL"} },<br>
    { TEST_COMPARE_EQUAL, "compareString 1/2", RES_OBJ+"newString#1", RES_OBJ+"newString#2" },<br>
    { TEST_METHOD, "compStr#1", RES_OBJ+"newString#1", "compareTo", new Object[] {RES_OBJ+"newString#2"} },<br>
    { TEST_METHOD, "newVector#1", "java.util.Vector", null, null },<br>
    { TEST_METHOD, "addVector#1", RES_OBJ+"newVector#1", "addElement", new Object[] {RES_OBJ+"newString#1"}, new String[] {"java.lang.Object"} },<br>
    { TEST_METHOD, "addVector#2", RES_OBJ+"newVector#1", "addElement", new Object[] {new Object()} },<br>
    { TEST_METHOD, "getVector#1", RES_OBJ+"newVector#1", "firstElement", null },<br>
    { TEST_COMPARE_EQUAL, "compareString 1/1", RES_OBJ+"newString#1", RES_OBJ+"getVector#1" },<br></dir>
  };<br></code>
   */
  public final static Object[][] TESTSCRIPT1 = {
    { TEST_METHOD, "newString#1", "java.lang.String", null, new Object[] {"EQUAL"} },
    { TEST_METHOD, "newString#2", "java.lang.String", null, new Object[] {"EQUAL"} },
    { TEST_COMPARE_EQUAL, "compareString 1/2", RES_OBJ+"newString#1", RES_OBJ+"newString#2" },
    { TEST_METHOD, "compStr#1", RES_OBJ+"newString#1", "compareTo", new Object[] {RES_OBJ+"newString#2"} },
    { TEST_METHOD, "newVector#1", "java.util.Vector", null, null },
    { TEST_METHOD, "addVector#1", RES_OBJ+"newVector#1", "addElement", new Object[] {RES_OBJ+"newString#1"}, new String[] {"java.lang.Object"} },
    { TEST_METHOD, "addVector#2", RES_OBJ+"newVector#1", "addElement", new Object[] {new Object()} },
  };

  public final static Object[][] TESTSCRIPT2 = {
    { TEST_METHOD, "getVector#1", RES_OBJ+"newVector#1", "firstElement", null },
    { TEST_COMPARE_EQUAL, "compareString 1/1", RES_OBJ+"newString#1", RES_OBJ+"getVector#1" },
    { TEST_GET_ARRAY_ITEM, "getArray#1", new char[] {'A', 'B', 'C'}, new Integer(1) },
    { TEST_COMPARE_EQUAL, "compareItem", RES_OBJ+"getArray#1", new Character('B') },
  };

  /**
   * Creates a new Test-Instance
   */
  public Test() {
    local_object = this;
  }

  /**
   * Return the result-object of an performed Test
   */
  public static Object getResult(String testName) {
    return (Object) result.get(tName+"."+testName);
  }

  /**
   * Adds a list of testscripts to one
   */
  public static Object[][] concatScripts(Object[] scriptlist) {
    int total = 0;
    for (int i=0;i<scriptlist.length;i++) // calc length
      total += java.lang.reflect.Array.getLength(scriptlist[i]);
    Object[][] newlist = new Object[total][0];
    int k=0;
    for (int j=0;j<scriptlist.length;j++) {
      Object[][] script = (Object[][]) scriptlist[j];
      for (int l=0;l<script.length;l++) {
        newlist[k] = script[l];
        k++;
      }
    }
    return newlist;
  }

  /**
   * Starts a new Test, set's the LOG-Server (if not already defined)
   */
  public static boolean startTest(String test) {
    System.out.println(SERVER + " starting at " + new java.util.Date().toString());
    try {
      tName = test;
      if (tName.indexOf('.') > 0)
        sName = tName.substring(tName.lastIndexOf('.')+1, tName.length());
      else
        sName = tName;
      if (log == null) {
        log = new LogClient(SERVER + ": ", logDir, LogMsg.LOG_ERROR, mailTo, SERVER.toLowerCase(), "", mailHost, Long.MAX_VALUE, true);
      }
      log.put( SERVER, "StartTest#0", LogMsg.LOG_REMARK, true, tName);
      started = true;
      System.out.println(SERVER + " LOG: " + logDir + " MAILTO: " + mailTo);
    } catch (Exception e) {
      System.err.println(SERVER + " FAILED TO START: " + e.toString());
      return false;
    }
    return true;
  }

  /**
   * Checks if the Object is a String and starts with the RES_OBJ,
   * if so looks up in the result-table and replaces it.
   */
  private static Object replace(Object aObj) {
    if (aObj instanceof String) {
      String s = (String) aObj;

        if (s.startsWith(RES_OBJ)) {

          return getResult(s.substring(RES_OBJ.length(), s.length()));

        }

    }

    return aObj;

  }

  /**
   * Returns a Class-object related to it's name (The name has to include the package-path)
   */
  public static Class forName(String name, boolean prefix) throws ClassNotFoundException {
    if (prefix) {
      if (name.startsWith("boolean"))
        return boolean.class;
      else if (name.startsWith("char"))
        return char.class;
      else if (name.startsWith("byte"))
        return byte.class;
      else if (name.startsWith("short"))
        return short.class;
      else if (name.startsWith("int"))
        return int.class;
      else if (name.startsWith("long"))
        return long.class;
      else if (name.startsWith("float"))
        return float.class;
      else if (name.startsWith("double"))
        return double.class;
      else if (name.startsWith("void"))
        return void.class;
      else
        return Class.forName(name);
    } else {
      if (name.equals("boolean"))
        return boolean.class;
      else if (name.equals("char"))
        return char.class;
      else if (name.equals("byte"))
        return byte.class;
      else if (name.equals("short"))
        return short.class;
      else if (name.equals("int"))
        return int.class;
      else if (name.equals("long"))
        return long.class;
      else if (name.equals("float"))
        return float.class;
      else if (name.equals("double"))
        return double.class;
      else if (name.equals("void"))
        return void.class;
      else
        return Class.forName(name);
    }
  }

  /**
   * Compares two results to test them to equal. Using th Object.equal method.
   */
  protected static Object arrayItem(String testName, Object arr, Integer index) {
    Object ret = null;
    if (!started) {
      System.out.println(SERVER + " NOT STARTED YET!");
    } else {
      testCount++;
      testFail++;
      try {
        if (arr == null) {
          throw new Exception(SERVER + ": Compareing Objects are null!");
        }
        if (arr instanceof String)
          arr= replace( arr );
        ret = java.lang.reflect.Array.get(arr, index.intValue());
        testFail--;
        if (ret != null)
          result.put(tName+"."+testName, ret);
        try {
          log.put( SERVER, "arrayItem#0", LogMsg.LOG_REMARK, true, sName+"#"+String.valueOf(testCount)+":"+testName+" SUCCEED");
        } catch (Exception e) {
          System.err.println(SERVER + " LOG ERROR ARR: statement not recorded.");
        }
        System.out.println(SERVER +"#"+String.valueOf(testCount)+ " ARR " + sName+":"+testName+" SUCCEED");
      } catch (Exception e) {
        try {
          log.put( SERVER, "arrayItem#1", LogMsg.LOG_REMARK, false, sName+"#"+String.valueOf(testCount)+":"+testName+" FAILED: " + e.toString());
        } catch (Exception f) {
          System.err.println(SERVER + " LOG ERROR ARR: statement not recorded.");
        }
        System.err.println(SERVER +"#"+String.valueOf(testCount)+ " ARR " +sName+ ":"+testName+" FAILED: " + e.toString());
      }
    }
    return ret;
  }

  /**
   * Compares two results to test them to equal. Using th Object.equal method.
   */
  private static boolean doCompare(String testName, Object obj1, Object obj2, boolean equal) {
    boolean ret = false;
    String pair = "";
    if (!started) {
      System.err.println(SERVER + " NOT STARTED YET!");
    } else {
      testCount++;
      testFail++;
      try {
        if ((obj1 == null) || (obj2 == null)) {
          if ((obj1 == null) && (obj2 == null))
            ret = equal;
          else
            ret = !equal;
        } else {
          obj1 = replace( obj1 );
          obj2 = replace( obj2 );
          if ((obj1 == null) || (obj2 == null)) {
            if ((obj1 == null) && (obj2 == null))
              ret = equal;
            else
              ret = !equal;
          } else {
            pair = "("+obj1.toString()+"=="+obj2.toString()+")";
            ret = equal == obj1.equals(obj2);
          }
        }
        if (pair.length()==0)
          pair = "("+ String.valueOf(obj1) + "==" + String.valueOf(obj2) + ")";
        if (!ret)
          throw new Exception("Comparison failed");
        testFail--;
        try {
          log.put( SERVER, "doCompare#0", LogMsg.LOG_REMARK, true, sName+"#"+String.valueOf(testCount)+":"+testName+" SUCCEED "+pair);
        } catch (Exception e) {
          System.err.println(SERVER + " LOG ERROR CMP: statement not recorded.");
        }
        System.out.println(SERVER +"#"+String.valueOf(testCount)+ " CMP " + sName+":"+testName+" SUCCEED "+pair);
      } catch (Exception e) {
        try {
          log.put( SERVER, "doCompare#1", LogMsg.LOG_REMARK, false, sName+"#"+String.valueOf(testCount)+":"+testName+" FAILED: " + pair + e.toString());
        } catch (Exception f) {
          System.err.println(SERVER + " LOG ERROR CMP: statement not recorded.");
        }
        System.err.println(SERVER +"#"+String.valueOf(testCount)+ " CMP " + sName+":"+testName+" FAILED: " + pair + e.toString());
      }
    }
    return ret;
  }

  /**
   * Compares two results to test them to equal. Using th Object.equal method.
   */
  protected static boolean isEqual(String testName, Object obj1, Object obj2) {
    return doCompare(testName, obj1, obj2, true);
  }

  /**
   * Invokes a method from a called Object.
   */
  protected static Object doTest(String testName, Object aObj, String methodName, Object[] args, String[] classnames) {
    if (!started) {
      System.err.println(SERVER + " NOT STARTED YET!");
      return null;
    } else {
      testCount++;
      testFail++;
      String classname = "";
      Object ret = null;
      try {
        if (methodName == null) {
          methodName = "";
        }
        if (aObj == null) {
          aObj = new Object();
          throw new Exception(SERVER + ": Invalid test-parameter!");
        }
        if (args == null) {
          args = new Object[0];

        }

        Class[] params = new Class[args.length];

        for (int i=0;i<args.length;i++) {

          args[i] = replace( args[i] );

          if ((classnames == null) || (classnames.length <= i))

            params[i] = args[i].getClass();

          else

            params[i] = forName(classnames[i], false);

        }

        long start = 0;

        long stop = 0;

        if (methodName.length() == 0) {

          java.lang.reflect.Constructor c = Class.forName((String) aObj).getConstructor(params);

          start = System.currentTimeMillis();

          ret = c.newInstance(args);

          stop = System.currentTimeMillis();
          classname = ret.getClass().getName();
        } else {

          java.lang.reflect.Method m = aObj.getClass().getMethod(methodName, params);

          start = System.currentTimeMillis();

          ret = m.invoke(aObj, args);

          stop = System.currentTimeMillis();
          classname = aObj.getClass().getName();
        }
        if (ret != null) {
          result.put(tName+"."+testName, ret);
        }
        testFail--;
        try {
          log.put( SERVER, "doTest#0", LogMsg.LOG_REMARK, true, sName+"#"+String.valueOf(testCount)+":"+testName+" SUCCEED ["+String.valueOf(stop-start)+"ms] ("+classname+"."+methodName+")");
        } catch (Exception e) {
          System.err.println(SERVER + " LOG ERROR TST: statement not recorded.");
        }
        System.out.println(SERVER +"#"+String.valueOf(testCount)+ " TST " + sName+":"+testName+" SUCCEED ["+String.valueOf(stop-start)+"ms] ("+classname+"."+methodName+")");
      } catch (Exception e) {
        try {
          log.put( SERVER, "doTest#1", LogMsg.LOG_REMARK, false, sName+"#"+String.valueOf(testCount)+":"+testName+" FAILED ("+classname+"."+methodName+"): " + e.toString());
        } catch (Exception f) {
          System.err.println(SERVER + " LOG ERROR TST: statement not recorded.");
        }
        System.err.println(SERVER +"#"+String.valueOf(testCount)+ " TST " + ":"+testName+" FAILED ("+classname+"."+methodName+"): " + e.toString());
      }
      return ret;
    }
  }

  /**
   * Invokes a method from a called class. The Class is instantiated with a
   * paramless constructor.
   */
  protected static Object doTest(String testName, String className, String methodName, Object[] args, String[] classnames) {
    if (!started) {
      System.err.println(SERVER + " NOT STARTED YET!");
      return null;
    } else try {
      Object aObj = replace(className);
      if ((aObj instanceof String) && (methodName != null) && (methodName.length() > 0))
        try {
          aObj = Class.forName(className).newInstance();
        } catch (Exception f) {
          System.err.println(SERVER + " LOG ERROR: statement not recorded.");
        }
      return doTest(testName, aObj, methodName, args, classnames);
    } catch (Exception e) {
      testCount++;
      testFail++;
      try {
        log.put( SERVER, "doTest#2", LogMsg.LOG_REMARK, false, sName+"#"+String.valueOf(testCount)+":"+testName+" FAILED ("+className+"."+methodName+"): " + e.toString());
      } catch (Exception f) {
          System.err.println(SERVER + " LOG ERROR TST: statement not recorded.");
      }
      System.err.println(SERVER +"#"+String.valueOf(testCount)+ " TST " + sName+":"+testName+" FAILED ("+className+"."+methodName+"): " + e.toString());
      return null;
    }
  }

  /**
   * Ends the Test
   */
  public static void stopTest() {
    if (!started) {
      System.err.println(SERVER + " NOT STARTED YET!");
    } else {
      started = false;
      try {
        log.put( SERVER, "StopTest#0", LogMsg.LOG_REMARK, testFail == 0, tName+": " + String.valueOf(testFail) + "/" + String.valueOf(testCount) + " tests failed");
      } catch (Exception f) {
        System.err.println(SERVER + " LOG ERROR: statement not recorded.");
      }
      if (testFail == 0)
        System.out.println(SERVER + " SUCCEED: " + tName + " " + String.valueOf(testFail) + "/" + String.valueOf(testCount) + " tests failed");
      else
        System.err.println(SERVER + " FAILED: " + tName + " " + String.valueOf(testFail) + "/" + String.valueOf(testCount) + " tests failed");
    }
  }

  /**
   * Returns the Test-Script for this Class
   */
  public Object[][] getTestScript() {
    return  concatScripts(new Object[] { TESTSCRIPT1, TESTSCRIPT2 });
  }

  /**
   *  Performs some Tests on the living Object
   */
  public boolean testIt(Object local_handle) {
    local_object = local_handle;
    return doTheTest(this.getClass().getName(), getTestScript());
  }

  /**
   *  Performs some Tests on the living Object
   */
  public static boolean testIt(String test, Object[][] testscript, Object local_handle) {
    local_object = local_handle;
    return doTheTest(SERVER, concatScripts(new Object[] { TESTSCRIPT1, TESTSCRIPT2 })) && doTheTest(test, testscript);
  }

  /**
   *  Performs some Tests on the living Object
   */
  private static boolean doTheTest(String test, Object[][] testscript) {
    Test.startTest(test);
    for (int i=0;i<testscript.length;i++) {
      Object[] testline = testscript[i];
      try {
        if (testline[1] == null)
         testline[1] = "(empty)";
        byte type = ((Byte) testline[0]).byteValue();
        switch (type) {
          case test_method: {
            if (testline[2] == null)
              testline[2] = "";
            if (testline[3] == null)
              testline[3] = "";
            if (testline[4] == null)
              testline[4] = new Object[0];
            String[] classnames = null;
            if ((testline.length > 5) && (testline[5] != null))
              classnames = (String[]) testline[5];
            if (testline[2] instanceof String)
              Test.doTest((String) testline[1], (String) testline[2], (String) testline[3], (Object[]) testline[4], classnames);
            else
              Test.doTest((String) testline[1], testline[2], (String) testline[3], (Object[]) testline[4], classnames);
            break;
          }
          case test_method_local: {
            if (testline[2] == null)
              testline[2] = "";
            if (testline[3] == null)
              testline[3] = new Object[0];
            String[] classnames = null;
            if ((testline.length > 4) && (testline[4] != null))
              classnames = (String[]) testline[4];
            Test.doTest((String) testline[1], local_object, (String) testline[2], (Object[]) testline[3], classnames);
            break;
          }
          case test_compare_equal: {
            Test.isEqual((String) testline[1], testline[2], testline[3]);
            break;
          }
          case test_get_array_item: {
            Test.arrayItem((String) testline[1], testline[2], (Integer) testline[3]);
            break;
          }
          default: {
            testCount++;
            testFail++;
            try {
              log.put( SERVER, "testIt#1", LogMsg.LOG_REMARK, false, sName+"#"+String.valueOf(testCount)+": FAILED: invalid test-script parameter count");
            } catch (Exception f) {
              System.err.println(SERVER + " LOG ERROR: statement not recorded.");
            }
            System.err.println(SERVER +"#"+String.valueOf(testCount)+ " " + sName+": FAILED: invalid test-script parameter count");
            break;
          }
        }
      } catch (Exception e) {
        testCount++;
        testFail++;
        try {
          log.put( SERVER, "testIt#2", LogMsg.LOG_REMARK, false, sName+"#"+String.valueOf(testCount)+": FAILED: invalid Test-Script: " + e.toString());
        } catch (Exception f) {
          System.err.println(SERVER + " LOG ERROR: statement not recorded.");
        }
        System.err.println(SERVER +"#"+String.valueOf(testCount)+ " " + sName+": FAILED: invalid Test-Script: " + e.toString());
      }
    }
    Test.stopTest();
    return testFail == 0;
  }

  /**
   * Main-Methode
   */
  public static void main(String[] args) {
    System.out.println("Hello, this is asgaard.utils.test.Test.");
    Test app = new Test();
    app.testIt(app);
  }

}

























