// This class is part of Banshee using Skid
// Banshee is an implementation from the asgaard-project in the sport-domain
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 010699
//
//@created	010699  KH
package banshee;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.rmi.*;
import java.util.*;
import java.text.*;

import asgaard.skid.*;
import asgaard.utils.log.*;
import asgaard.utils.mail.*;

/**
 * Holds some common functionality and the Session-handling for all Servlets
 */
public abstract class Servlet extends HttpServlet {

  public final static String[] LANGUAGES = {"ENG", "GER"};
  public final static String RESOURCE_PATH = "/usr/local/jserv/asgaard/banshee/resources";
//  private static String LOGserver = "128.130.167.56";  // hildegard
//  private static String SKIDserver = "128.130.167.56";  // hildegard
  private static String LOGserver = "128.130.167.57";  // nornen
  private static String SKIDserver = "128.130.167.57";  // nornen
  private static String MAILserver = "128.130.167.26";  // garbo
  private static String SUPPORTurl = "klaus@ifs.tuwien.ac.at";
  public final static String VERSION = "1.0.0";
  public final static String SERVER = "Servlet";
  public final static String DELIM ="$";

  protected final static boolean debug = true;

  private static boolean readFiles = true;

  private static Hashtable binRes = new Hashtable();

  protected static boolean doLog = true;
  protected final static boolean useBrokerLog = false;
  protected static LogClient log;

	protected boolean skidOffline = true;
	protected UserSession currentUser = null;
	protected String lang = "ENG";
  protected String tmp = "";

  private final static FixedMsgSend mailer = new FixedMsgSend(null, SUPPORTurl, null, MAILserver, true);

  /**
   * Serviceroutine: Read Content File (support of different languages)
   *@param filename including starting "/" but no path
   *@see RESOURCE_PATH
   */
  public static void readBinary(String filename) throws IOException {
    if (! binRes.containsKey(filename)) {
      FileInputStream bin = new FileInputStream( RESOURCE_PATH + filename );
      byte[] buf = new byte[bin.available()];
      bin.read(buf);
      bin.close();
      binRes.put(filename, buf);
    }
  }

  /**
   * Serviceroutine: Writes some binary data to the OutputStream
   *@param response the http-response
   *@param type the response-type
   *@param filename the name of the binary resource
   *@see readBinary
   */
  public void out(HttpServletResponse response, String type, String filename) throws IOException {
      response.setContentType(type);
      OutputStream o = response.getOutputStream();
      o.write((byte[]) binRes.get(filename));
      o.close();
  }


  /**
   * Serviceroutine: Opens a Pinwriter to response to the Request
   *@param response the http-response
   *@param type the response-type
   */
  public static PrintWriter open(HttpServletResponse response, String type) throws IOException {
    response.setContentType(type);
    return new PrintWriter (response.getOutputStream());
  }

  /**
   * Serviceroutine: Opens a Pinwriter to response to the Request
   *@param response the http-response
   *@param type the response-type
   */
  public static void close(PrintWriter o) throws IOException {
    o.close();
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param response the http-response
   *@param type the response-type
   */
  public void out(HttpServletResponse response, String type, String[][] html, String lang) throws IOException {
    PrintWriter o = open(response, type);
    out(o, html, lang);
    close(o);
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param response the http-response
   *@param type the response-type
   */
  public void out(PrintWriter o, String[][] html, String lang) throws IOException {
    int l = html[0].length-1;              // search Language
    while ((l>0) && (! html[0][l].equalsIgnoreCase(lang)))
      l--;
    if (l<1) l=1;                         // set default if not found
    o.print(html[0][0]);                  // first statement
    for (int i=1;i<html.length;i++) {     // write rest of the document static
      o.print(html[i][l]);
      o.print(html[i][0]);
    }
    if (html == MsgServlet.db_error)
      skidOffline = true;
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to the mail-spooler
   *@param to the receiver
   *@param msg the message simmilar to the html-array, but line 1 holds the subject
   *@param lang the used language
   */
  public void mail(String to, String[][] msg, String lang) throws IOException {
    int l = msg[0].length-1;              // search Language
    while ((l>0) && (! msg[0][l].equalsIgnoreCase(lang)))
      l--;
    if (l<1) l=1;                         // set default if not found
    String s = msg[1][0];
    for (int i=2;i<msg.length;i++) {     // write rest of the document static
      s += msg[i][l];
      s += msg[i][0];
    }
    mailer.spoolMsg(to, msg[1][l], s);
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param response the http-response
   *@param search searches for the given String
   *@param replace if found it is replaced by
   *@param type the response-type
   */
  public void out(HttpServletResponse response, String type, String[][] html, String lang, String search, String replace) throws IOException {
    PrintWriter o = open(response, type);
    out(o, html, lang, search, replace);
    close(o);
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param search searches for the given String
   *@param replace if found it is replaced by
   *@param response the http-response
   *@param type the response-type
   */
  public void out(PrintWriter o, String[][] html, String lang, String search, String replace) throws IOException {
    int l = html[0].length-1;              // search Language
    while ((l>0) && (! html[0][l].equalsIgnoreCase(lang)))
      l--;
    if (l<1) l=1;                         // set default if not found
    int idx = html[0][0].indexOf(search);  // first statement (STRUCT)
    if (idx >= 0)
      o.print(html[0][0].substring(0, idx) + replace + html[0][0].substring(idx+search.length(), html[0][0].length()));
    else
      o.print(html[0][0]);
    for (int i=1;i<html.length;i++) {     // write rest of the document static
      idx = html[i][l].indexOf(search);  // first statement (STRUCT)
      if (idx >= 0)
        o.print(html[i][l].substring(0, idx) + replace + html[i][l].substring(idx+search.length(), html[i][l].length()));
      else
        o.print(html[i][l]);
      idx = html[i][0].indexOf(search);  // first statement (STRUCT)
      if (idx >= 0)
        o.print(html[i][0].substring(0, idx) + replace + html[i][0].substring(idx+search.length(), html[i][0].length()));
      else
        o.print(html[i][0]);
    }
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param response the http-response
   *@param replace if found it is replaced by
   *@param type the response-type
   */
  public void out(HttpServletResponse response, String type, String[][] html, String lang, String[][] replace) throws IOException {
    PrintWriter o = open(response, type);
    out(o, html, lang, replace);
    close(o);
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param replace if found it is replaced by
   *@param response the http-response
   *@param type the response-type
   */
  public void out(PrintWriter o, String[][] html, String lang, String[][] replace) throws IOException {
    int l = html[0].length-1;              // search Language
    while ((l>0) && (! html[0][l].equalsIgnoreCase(lang)))
      l--;
    if (l<1) l=1;                         // set default if not found
    String s = new String(html[0][0]);
    for (int j=0;j<replace.length;j++) {
      int idx = s.indexOf(replace[j][0]);  // first statement (STRUCT)
      if (idx >= 0)
        s = s.substring(0, idx) + replace[j][1] + s.substring(idx+replace[j][0].length(), s.length());
    }
    o.print(s);
    for (int i=1;i<html.length;i++) {     // write rest of the document static
      s = new String(html[i][l]);
      for (int j=0;j<replace.length;j++) {
        int idx = s.indexOf(replace[j][0]);
        if (idx >= 0)
          s = s.substring(0, idx) + replace[j][1] + s.substring(idx+replace[j][0].length(), s.length());
      }
      o.print(s);
      s = new String(html[i][0]);
      for (int j=0;j<replace.length;j++) {
        int idx = s.indexOf(replace[j][0]);
        if (idx >= 0)
          s = s.substring(0, idx) + replace[j][1] + s.substring(idx+replace[j][0].length(), s.length());
      }
      o.print(s);
    }
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param response the http-response
   *@param type the response-type
   */
  public void out(HttpServletResponse response, String type, String[][] html, String lang, Hashtable dyn) throws IOException {
    PrintWriter o = open(response, type);
    out(o, html, lang, dyn);
    close(o);
  }
  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param response the http-response
   *@param type the response-type
   */
  public void out(PrintWriter o, String[][] html, String lang, Hashtable dyn) throws IOException {
    if (dyn == null)
      out(o, html, lang);
    else {
      int l = html[0].length-1;              // search Language
      while ((l>0) && (! html[0][l].equalsIgnoreCase(lang)))
        l--;
      if (l<1) l=1;                         // set default if not found
      int idx = html[0][0].indexOf(DELIM);  // first statement (STRUCT)
      if (idx >= 0) {
        Object[] args = (Object[]) dyn.get(html[0][0].substring(0, idx));
        if (args != null)
          o.print(MessageFormat.format(html[0][0].substring(idx+1, html[0][0].length()), args));
        else
          o.print(html[0][0]);
      } else
        o.print(html[0][0]);
      for (int i=1;i<html.length;i++) {     // write rest of the document with dynamic information
        idx = html[i][l].indexOf(DELIM);// (CONTENT)
        if (idx >= 0) {
          Object[] args = (Object[]) dyn.get(html[i][l].substring(0, idx));
          if (args != null)
            o.print(MessageFormat.format(html[i][l].substring(idx+1, html[i][l].length()), args));
          else
            o.print(html[i][l]);
        } else
          o.print(html[i][l]);
        idx = html[i][0].indexOf(DELIM);   // (STRUCT PART 2)
        if (idx >= 0) {
          Object[] args = (Object[]) dyn.get(html[i][0].substring(0, idx));
          if (args != null)
            o.print(MessageFormat.format(html[i][0].substring(idx+1, html[i][0].length()), args));
          else
            o.print(html[i][0]);
        } else
          o.print(html[i][0]);
      }
    }
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param response the http-response
   *@param type the response-type
   */
  public void out(HttpServletResponse response, String type, String[][] html, String lang, Object[] dyn) throws IOException {
    PrintWriter o = open(response, type);
    out(o, html, lang, dyn);
    close(o);
  }
  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to an Writer.
   *@param response the http-response
   *@param type the response-type
   */
  public void out(PrintWriter o, String[][] html, String lang, Object[] dyn) throws IOException {
    if (dyn == null)
      out(o, html, lang);
    else {
      int l = html[0].length-1;              // search Language
      while ((l>0) && (! html[0][l].equalsIgnoreCase(lang)))
        l--;
      if (l<1) l=1;                         // set default if not found
      o.print(MessageFormat.format(html[0][0], dyn));   // first statement
      for (int i=1;i<html.length;i++) {     // write rest of the document with dynamic information
        o.print(MessageFormat.format(html[i][l], dyn));
        o.print(MessageFormat.format(html[i][0], dyn));
      }
    }
  }

  /**
   * Serviceroutine: Merges a Structure-String-array and a Content-String-array
   * and writes it to the mail-spooler
   *@param to the receiver
   *@param msg the message simmilar to the html-array, but line 1 holds the subject
   *@param lang the used language
   */
  public void mail(String to, String[][] msg, String lang, Object[] dyn) throws IOException {
    int l = msg[0].length-1;              // search Language
    while ((l>0) && (! msg[0][l].equalsIgnoreCase(lang)))
      l--;
    if (l<1) l=1;                         // set default if not found
    String s = msg[1][0];
    for (int i=2;i<msg.length;i++) {     // write rest of the document static
      s += msg[i][l];
      s += msg[i][0];
    }
    mailer.spoolMsg(to, MessageFormat.format(msg[1][l], dyn), MessageFormat.format(s, dyn));
  }

  /**
   * Writes the main part of the Serlet's outcome.
   */
  protected void writePage(HttpServletRequest request, PrintWriter o) throws ServletException, IOException {  }

  /**
   * Gets the UserSession if not currently existing and sets the user-Language
   */
  protected boolean userValid(HttpServletRequest request) throws ServletException, IllegalAccessException  {
			if (currentUser == null)
				currentUser = UserSession.getSession(getParameter( request, "sid" ), request.getRemoteAddr());
			lang = currentUser.getLanguage();
			return currentUser.checkPage( null, getParameter( request, "pid" ));
  }

  /**
   * Gets the Hashtable with the Properties
   */
  protected Hashtable getParams()  {
			Hashtable params = new Hashtable();
      try {
  			params.put("id", new Object[] { currentUser.getID(), currentUser.addPage(this.getClass().getName()) });
      } catch (Exception e) {
        logMsg(this.getClass().getName() , "getParams#1", LogMsg.LOG_ERROR, currentUser != null, e.toString());
      }
 			return params;
  }

  /**
   * Serviceroutine: Asks strongly for a requested parameter with only one
   * value and throws an Exception if it doesen't work.
   */
  public String getParameter(HttpServletRequest request, String paramName) {
    String[] val = request.getParameterValues( paramName );
    if ((val == null) || (val.length < 1))
      return null;
    return val[0];
  }

  /**
   * Serviceroutine: For multiple Select checks whether a value is selected or not
   */
  public boolean checkParameter(HttpServletRequest request, String paramName, String value) {
    String[] val = request.getParameterValues( paramName );
    if ((val == null) || (val.length < 1))
      return false;
    for (int i=0;i<val.length;i++)
      if (value.equalsIgnoreCase(val[i]))
        return true;
    return false;
  }

  /**
   * Serviceroutine: Asks if an requested parameter exists and wether there is an value or not.
   */
  public boolean paramExist(HttpServletRequest request, String paramName) {
    String res = "";
    try {
      res = getParameter(request, paramName);
    } catch (Exception e) {
      return false;
    }
    return (res != null) && (res.length() > 0);
  }

  /**
   * Does the initialisation
   */
  public void init(ServletConfig config) throws ServletException {
    super.init(config);
    if (readFiles) {                                            // CLASS INIT
      readFiles = false;
      if (doLog)                                                // LOG if REQUIRED
        connectLog(this.getClass().getName());
      connectSkid();                                            // Connect to DB
    }
  }

  /**
   * Tries to connect to the Log-Server
   */
  protected static boolean connectLog(String server) {
    try {
      log = new LogClient(SERVER + ": ", LOGserver, true);  // create a LogClient
      log.put( server, "init#0", LogMsg.LOG_REMARK, true, SERVER + " startet.");
    } catch (Exception e) {
      log = null;
      doLog = false;
    }
    return doLog;
  }

  /**
   * Tries to connect to the Skid
   */
  protected boolean connectSkid() {
		try {                                                       // INSTANCE INIT
      BeanBroker.HOST = SKIDserver;
			BeanBroker.THIS = (RemoteBroker) Naming.lookup("//"+SKIDserver+"/" + Skid.ServerName );
			skidOffline = false;
		} catch (Exception e) {
			skidOffline = true;
      logMsg(this.getClass().getName() , "init#1", LogMsg.LOG_ERROR, false, Skid.ServerName + ": " + e.toString());
    }
    return ! skidOffline;
  }

  /**
   * Puts a Message to the LOG-Server if available and returns whether it was sent or not
   *@see asgaard.utils.log.LogMsg
   *@param server     The calling Java-Serverclass
   *@param point      The Method / place where the statement was created
   *@param typ        The Type of the LOG-Statement
   *@param ok         Was the ServicePoint passed successfuly
   *@param msg        Human redable Message of the Statement
   */
  public static boolean logMsg(String server, String point, byte typ, boolean ok, String msg) {
    try {
      if (useBrokerLog && (BeanBroker.THIS != null))
        BeanBroker.THIS.logMsg(server, point, typ, ok, msg);
      else if (log == null) {
        if (connectLog("Servlet.retry#1"))
          log.put( server, point, typ, ok, msg);
      } else {
        log.put( server, point, typ, ok, msg);
      }
    } catch (Exception e) {
      try {
        if (connectLog("Servlet.retry#2"))
          log.put( server, point, typ, ok, msg);
      } catch (Exception ex) { }
    }
    return doLog;
  }

  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    currentUser = null;
  }
  public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    currentUser = null;
  }
  public void doPut(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    currentUser = null;
  }
  public void doDelete(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    currentUser = null;
  }

}


