// This class is part of Banshee using Skid
// Banshee is an implementation from the asgaard-project in the sport-domain
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 260599
//
//@created	260599  KH
package banshee.lang;

// Import the skid-API
import asgaard.skid.*;
import asgaard.lang.*;

/**
 * This class holds a generic workout-parameter based aproac of an elementar
 * action in sports. This is e.g. "running 1 hour with heart-frequence of 140".
 * Therefor a Template is used, which holds all default-settings for a typical
 * action (e.g. running) in a individual configuration.
 *
 *@see asgaard.banshee.lang.Template
 */
public class Action extends Entity {

  /**
   * List with defaults got from the template (as well as the items)
   */
  transient protected Object[] defs;

  transient private Template templ;
  private int kTemplate = 0;

  /**
   * Instanciates an Entry, setting the defaults and item-List from the template.
   * Do not use it to create a new Entry - you need to distinguish an template.
   *@throws Exception if there is no known template.
   */
  public Action() throws Exception {
    if (kTemplate != 0) {
      templ = (Template) broker.getBean(kTemplate);
      items = templ.getItems();
      defs = templ.getDefaults();
    } else
      throw new SkidException("Template not found!");
  }

  /**
   * Instanciates a new Entry
   *@param template the used template for this entry
   *@throws Exception
   */
  public Action(Template template) throws Exception {
      templ = template;
      kTemplate = templ.getKey();
      items = templ.getItems();
      defs = templ.getDefaults();
  }

  public String toString() {
    String s = "";
    return s;
  }

  /**
   * Returns the Template used by this Action
   *@returns an SkidBean
   *@see asgaard.banshee.lang.Template
   */
  public Entity getTemplate() {
    return templ;
  }

/* DUMMY --------------------------------------------------------------------
   VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV

  /**
   * Returns the "Trainingsmittel" - the kind of workout. E.g.: "Running"
   *
  public Def getTM() {
    if (super.getTM() != null)
      return super.getTM();
    else
      return template.getTM();
  }
  /**
   * Returns the Exercise - which way you do the workout. E.g.: "on one leg"
   *
  public Def getExercise() {
    if (super.getExercise() != null)
      return super.getExercise();
    else
      return template.getExercise();
  }
  /**
   * Returns the "Wiederholungen" - the count of repetitions. E.g.: "25 times"
   *
  public int getWH() {
    if (super.getWH() != 0)
      return super.getWH();
    else
      return template.getWH();
  }
  /**
   * Returns the Duration of the exercise in [ms]. E.g.: "25000 ms"
   *
  public long getDuration() {
    if (super.getDuration() != 0)
      return super.getDuration();
    else
      return template.getDuration();
  }
  /**
   * Returns the Rest betwee the exercises in [ms]. E.g.: "250 ms"
   *
  public long getRest() {
    if (super.getRest() != 0)
      return super.getRest();
    else
      return template.getRest();
  }
  /**
   * Returns the Length of the exercise in [m]. E.g.: "100 m"
   *
  public float getLength() {
    if (super.getLength() != 0)
      return super.getLength();
    else
      return template.getLength();
  }
  /**
   * Returns the Power of the exercise in [W]. E.g.: "100 W"
   *
  public float getPower() {
    if (super.getPower() != 0)
      return super.getPower();
    else
      return template.getPower();
  }
  /**
   * Returns the Load of the exercise in [g]. E.g.: "1000 g"
   *
  public float getLoad() {
    if (super.getLoad() != 0)
      return super.getLoad();
    else
      return template.getLoad();
  }
  /**
   * Returns the Frequency of the exercise in [1/sec]. E.g.: "0,5 / sec"
   *
  public float getFrequency() {
    if (super.getFrequency() != 0)
      return super.getFrequency();
    else
      return template.getFrequency();
  }
  /**
   * Returns the Heart Frequency of the exercise in [1/min]. E.g.: "140 / min"
   *
  public int getHeart() {
    if (super.getHeart() != 0)
      return super.getHeart();
    else
      return template.getHeart();
  }
  /**
   * Returns the Lactat of the exercise in [mmol]. E.g.: "4,5 mmol"
   *
  public float getLactat() {
    if (super.getLactat() != 0)
      return super.getLactat();
    else
      return template.getLactat();
  }
  /**
   * Returns the Quality of measurement
   *
  public Def getQuality() {
    if (super.getQuality() != null)
      return super.getQuality();
    else
      return template.getQuality();
  }

   ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
   DUMMY -------------------------------------------------------------------- */


}