// This class is part of Banshee using Skid
// Banshee is an implementation from the asgaard-project in the sport-domain
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 260599
//
//@created	260599  KH
package banshee.lang;

// Import the skid-API
import asgaard.skid.*;
import asgaard.lang.*;

/**
 * This class gives the general layout for a generic action with an atomic action
 * in workout. The outline is a quiert simple parameter-based characteristic of
 * an action executed by an sports(wo)men.
 */
public abstract class Entity extends SkidBean {

  /**
   * List whith Items, which descript each attribute of the list
   */
  transient protected Item[] items;
  private String name = "E" + String.valueOf(System.currentTimeMillis());
  /**
   * Values carried by the attribute
   */
  protected Object[] values;


  /**
   * Returns the name of the Bean which shold be human readable and may be
   * unique in some cases, but this is not strictly required.
   */
  public String getName() {
    return name;
  }

  /**
   * Sets the name of the Entity.
   *@param name the value
   */
  public void setName(String name) {
    this.name = name;
  }


  /**
   * Returns Object at the given index if any
   */
  public Object item(int idx) {
    return values[idx];
  }

  /**
   * Returns the number of attributes which are carried by this Action
   */
  public int length() {
    if (values == null)
      return 0;
    else
      return values.length;
  }

  /**
   * Returns the Template used by this Action
   *@returns an SkidBean
   *@see asgaard.banshee.lang.Template
   */
  public abstract Entity getTemplate();

/* DUMMY --------------------------------------------------------------------
   VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV

  transient private Def TM;
  private int kTM = 0;
  transient private Def Exercise;
  private int kExercise = 0;
  private int WH = 0;
  private long Duration = 0;
  private long Rest = 0;
  private float Length = 0;
  private float Power = 0;
  private float Load = 0;
  private float Frequency = 0;
  private int Heart = 0;
  private float Lactat = 0;
  transient private Def Quality;
  private int kQuality = 0;

  /**
   * Instanciates a Action <b>Do not use manually - only for serialisation</b>!
   *@throws Exception
   *
  public Action() throws Exception {
    if (kTM != 0)
      TM = (Def) broker.getBean(kTM);
    if (kExercise != 0)
      Exercise = (Def) broker.getBean(kExercise);
    if (kQuality != 0)
      Quality = (Def) broker.getBean(kQuality);
  }

  /**
   * Returns the "Trainingsmittel" - the kind of workout. E.g.: "Running"
   *
  public Def getTM() {
    return TM;
  }
  /**
   * Returns the Exercise - which way you do the workout. E.g.: "on one leg"
   *
  public Def getExercise() {
    return Exercise;
  }
  /**
   * Returns the "Wiederholungen" - the count of repetitions. E.g.: "25 times"
   *
  public int getWH() {
    return WH;
  }
  /**
   * Returns the Duration of the exercise in [ms]. E.g.: "25000 ms"
   *
  public long getDuration() {
    return Duration;
  }
  /**
   * Returns the Rest betwee the exercises in [ms]. E.g.: "250 ms"
   *
  public long getRest() {
    return Rest;
  }
  /**
   * Returns the Length of the exercise in [m]. E.g.: "100 m"
   *
  public float getLength() {
    return Length;
  }
  /**
   * Returns the Power of the exercise in [W]. E.g.: "100 W"
   *
  public float getPower() {
    return Power;
  }
  /**
   * Returns the Load of the exercise in [g]. E.g.: "1000 g"
   *
  public float getLoad() {
    return Load;
  }
  /**
   * Returns the Frequency of the exercise in [1/sec]. E.g.: "0,5 / sec"
   *
  public float getFrequency() {
    return Frequency;
  }
  /**
   * Returns the Heart Frequency of the exercise in [1/min]. E.g.: "140 / min"
   *
  public int getHeart() {
    return Heart;
  }
  /**
   * Returns the Lactat of the exercise in [mmol]. E.g.: "4,5 mmol"
   *
  public float getLactat() {
    return Lactat;
  }
  /**
   * Returns the Quality of measurement
   *
  public Def getQuality() {
    return Quality;
  }

  /**
   * Sets the "Trainingsmittel" - the kind of workout. E.g.: "Running"
   *@throws Exception
   *
  public void setTM(Def tm) throws Exception {
    kTM = tm.getKey();
    TM = tm;
  }
  /**
   * Sets the Exercise - which way you do the workout. E.g.: "on one leg"
   *@throws Exception
   *
  public void setExercise(Def exercise) throws Exception {
    kExercise = exercise.getKey();
    Exercise = exercise;
  }
  /**
   * Sets the "Wiederholungen" - the count of repetitions. E.g.: "25 times"
   *
  public void setWH(int wh) {
    WH = wh;
  }
  /**
   * Sets the Duration of the exercise in [ms]. E.g.: "25000 ms"
   *
  public void setDuration(long dur) {
    Duration = dur;
  }
  /**
   * Sets the Rest betwee the exercises in [ms]. E.g.: "250 ms"
   *
  public void setRest(long r) {
    Rest = r;
  }
  /**
   * Sets the Length of the exercise in [m]. E.g.: "100 m"
   *
  public void setLength(float len) {
    Length = len;
  }
  /**
   * Sets the Power of the exercise in [W]. E.g.: "100 W"
   *
  public void setPower(float pow) {
    Power = pow;
  }
  /**
   * Sets the Load of the exercise in [g]. E.g.: "1000 g"
   *
  public void setLoad(float load) {
    Load = load;
  }
  /**
   * Sets the Frequency of the exercise in [1/sec]. E.g.: "0,5 / sec"
   *
  public void setFrequency(float freq) {
    Frequency = freq;
  }
  /**
   * Sets the Heart Frequency of the exercise in [1/min]. E.g.: "140 / min"
   *
  public void setHeart(int hr) {
    Heart = hr;
  }
  /**
   * Sets the Lactat of the exercise in [mmol]. E.g.: "4,5 mmol"
   *
  public void setLactat(float lac) {
    Lactat = lac;
  }
  /**
   * Sets the Quality of measurement
   *@throws Exception
   *
  public void setQuality(Def quality) throws Exception {
    kQuality = quality.getKey();
    Quality = quality;
  }

   ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
   DUMMY -------------------------------------------------------------------- */


}
