// This class is part of Banshee using an XML parser and the DOM API
// Banshee is an implementation from the asgaard-project in the sport-domain
//@autor  (P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at
//@version	1.0 050699
//
//@created	050699  KH
//
// Tip: for checking the xml-File:
// cd c:/tmp/xml4j_2_0_9
// jre -cp swing.jar;xml4j.jar;xml4jSamples.jar ui.TreeViewer p:/asgaard/src/banshee/resources/logon.xml
//
package banshee.utils;

import com.ibm.xml.parser.*;
import org.w3c.dom.*;
import java.io.*;
import java.util.*;

/**
 * This parser creates an Servlet and all of it's static information out of
 * an xml-file.
 */
public class ServletParser {

  private static ResourceBundle pres = ResourceBundle.getBundle("banshee.utils.resources.ServletParserRes");
  private FileWriter classWriter = null;
  private FileWriter tmpWriter = null;
  private NamedNodeMap entities;

  public final static String PARSER_NAME = "ServletParser";
  public final static String VERSION = "1.0.0";
  public final static String COPYRIGHT = "(P) by Klaus Hammermller, mailto:klaus@ifs.tuwien.ac.at\n"+
                                         "    under GNU public license - absolute NO WARRANTY on this software.";
  public static String TMP_EXT = ".~pc";

  /**
   * starts the parser by a given xml-File
   */
  public static void main (String args[]) throws Exception {
    System.out.println(s("greetings") + PARSER_NAME + " " + VERSION);
    System.out.println(COPYRIGHT);
    System.out.println(s("usage"));
    System.out.println(s("tip"));
    if (args.length < 1) {
      System.out.println(s("bye"));
      System.exit( 0 );
    }
    System.out.println(s("start-1"));
    Parser ps = new Parser(PARSER_NAME);
    ServletParser app = new ServletParser();
    app.doit(ps.readStream(new FileInputStream(args[0])));
    System.out.println(s("bye"));
  }

  private void doit(Document doc) throws Exception {
    tmpWriter = new FileWriter(PARSER_NAME + TMP_EXT);
    NodeList root = doc.getElementsByTagName(s("root-tag"));         // get the ROOT
    Element r = (Element) root.item( 0 );
    String className = r.getAttribute(s("root-name"));
    String packageName = r.getAttribute(s("root-package"));
    String modifierName = r.getAttribute(s("root-modifier"));
    String extendsName = r.getAttribute(s("root-extends"));
    entities = doc.getDoctype().getEntities();
    // PAGEs ------------------
    NodeList pageList = doc.getElementsByTagName(s("page-tag"));     // get all PAGE-NODES
    int pageCount = pageList.getLength();

    classWriter = new FileWriter(className+s("out-ext"));            // open output-file
    Date d = new Date();                                             // ----------------
    classWriter.write( s("head-intro") + d.toString());              // write header
    classWriter.write( s("head-package") + packageName);
    classWriter.write( s("head-create") + modifierName + " " + className);
    classWriter.write( s("class-def-1")+ extendsName +s("class-def-2"));

    NodeList defList = doc.getElementsByTagName(s("def-tag"));
    if (defList.getLength() > 0)
      classWriter.write( getString( defList.item( 0 ), false ));

    for (int p=0;p<pageCount;p++)                                    // write all static page-infos
      if (pageList.item( p ).getNodeType() == Node.ELEMENT_NODE) {
        Element page = (Element) pageList.item( p );
        String pageName = page.getAttribute(s("page-name"));
        classWriter.write( s("static-def") + pageName + s("static-def-open"));
        // PAGE-SEGMENTS ------------------
        Element html = (Element) page.getElementsByTagName(s("html-tag")).item(0);
        String segmReuse = html.getAttribute(s("html-reuse"));
        boolean doReuse = segmReuse.length() > 0;
        NodeList segmList = page.getElementsByTagName(s("segment-tag"));      // get all SEGMENT-NODES
        int lineCount = 0;                                                    // starts first line
        if (doReuse) {
          classWriter.write( "    {" + segmReuse + "[0][0]");
        } else {
          classWriter.write( "    {\"");
          Element firstSegm = (Element) segmList.item( 0 );
          String dyn = firstSegm.getAttribute(s("statement-dyn"));
          if (dyn.length() > 0)
            classWriter.write(dyn + banshee.Servlet.DELIM);
          writeJava( classWriter, getString(firstSegm, true), true);
          classWriter.write( "\"");
          lineCount = segmList.getLength();
        }
        // CONTENTs -> LANGUAGES ------------------
        NodeList contList = page.getElementsByTagName(s("content-tag"));      // get all CONTENT-NODES
        int langCount = contList.getLength();
        NodeList[] stmtList = new NodeList[langCount];
        for (int c=0;c<langCount;c++) {                                       // writing languages to first line
          Element cont = (Element) contList.item( c );
          stmtList[c] = cont.getElementsByTagName( s("statement-tag"));
          String langName = cont.getAttribute(s("content-lang"));
          classWriter.write(", \"" + langName + "\"");
          if ((lineCount>0) && (stmtList[c].getLength() != (lineCount-1)))    // Check linecount
            System.err.println(s("err-lines-1") + pageName + " " + String.valueOf(lineCount) + s("err-lines-2") + langName + " " + String.valueOf(stmtList[c].getLength()));
        }
        if (doReuse)
          lineCount = stmtList[0].getLength() + 1;
        classWriter.write("},\n");
        // write STRING[][] ------------------
        for (int l=1;l<lineCount;l++) {                                     // starts next line(s)
          if (doReuse) {
            classWriter.write( "    {" + segmReuse + "["+String.valueOf(l)+"][0]");
          } else {
            classWriter.write( "    {\"");
            Element segm = (Element) segmList.item( l );
            String dyn = segm.getAttribute(s("statement-dyn"));
            if (dyn.length() > 0)
              classWriter.write(dyn + banshee.Servlet.DELIM);
            writeJava( classWriter, getString(segm, true), true);           // HTML
            classWriter.write( "\"" );
          }
          for (int k=0;k<langCount;k++) {
            classWriter.write( ",\n\t\t\t" );
            Element stmt = (Element) stmtList[k].item( l-1 );
            String reuse = stmt.getAttribute(s("html-reuse"));
            if (reuse.equalsIgnoreCase(s("reuse-yes")))
              classWriter.write(segmReuse + "["+String.valueOf(l)+"]["+String.valueOf(k+1)+"]");
            else {
              classWriter.write("\"");
              if (stmtList[k].getLength() >= l) {
                String dyn = stmt.getAttribute(s("statement-dyn"));
                if (dyn.length() > 0)
                  classWriter.write(dyn + banshee.Servlet.DELIM);
                writeJava( classWriter, getHTML(stmt), true);
              }
              classWriter.write("\"");
            }
          }
          classWriter.write("},\n");
        }
        classWriter.write( s("static-def-close"));
        tmpWriter.close();
        System.out.println(s("start-2") + pageName + s("start-3"));
        Parser pc = new Parser(PARSER_NAME + TMP_EXT); // check HTML
        pc.readStream(new FileInputStream(PARSER_NAME + TMP_EXT));
        tmpWriter = new FileWriter(PARSER_NAME + TMP_EXT);
      }
    // INIT ------------------
    NodeList initList = doc.getElementsByTagName(s("init-tag"));          // creates the SERVLET-INIT
    if (initList.getLength() > 0) {
      classWriter.write( s("class-init-open"));
      Element init = (Element) initList.item( 0 );
      NodeList resList = init.getElementsByTagName(s("resource-tag"));
      int resCount = resList.getLength();
      for (int u=0;u<resCount;u++)                                        // get all RESSOURCE-NAMES
        if (resList.item( u ).getNodeType() == Node.ELEMENT_NODE) {
          Element res = (Element) resList.item( u );
          String resName = res.getAttribute(s("resource-name"));
          classWriter.write( s("init-res-1") + resName + s("init-res-2"));
        }
      classWriter.write( s("init-res-3"));
      NodeList stmtList = init.getElementsByTagName(s("java-tag"));
      int stmtCount = stmtList.getLength();
      for (int w=0;w<stmtCount;w++)
        classWriter.write(getString(stmtList.item( w ), false));
      classWriter.write( s("class-init-close"));
    } else
      classWriter.write( s("class-init"));
    // SERVICES ------------------
    NodeList servList = doc.getElementsByTagName(s("service-tag"));  // get all SERVICE-ROUTINES
    int servCount = servList.getLength();
    for (int v=0;v<servCount;v++)                                    // write all static page-infos
      if (servList.item( v ).getNodeType() == Node.ELEMENT_NODE) {
        Element serv = (Element) servList.item( v );
        String servName = serv.getAttribute(s("service-name"));
        NodeList commList = serv.getElementsByTagName(s("comment-tag"));  // get all COMMENTS
        classWriter.write(s("comment-open"));
        int commCount = commList.getLength();
        for (int c=0;c<commCount;c++)
          classWriter.write(s("comment-lf") + getString(commList.item(c), true));
        classWriter.write(s("comment-close"));
        classWriter.write(s("service-open-1") + servName + s("service-open-2"));
        NodeList requList = serv.getElementsByTagName(s("request-tag"));  // get all REQUESTS
        int requCount = requList.getLength();
        for (int q=0;q<requCount;q++) {
          Element requ = (Element) requList.item( q );
          String requName = requ.getAttribute(s("request-name"));
          classWriter.write(s("request-1") + requName + s("request-2"));
          classWriter.write(getString(requ, false) + s("request-3"));
        }
        classWriter.write(s("service-close-1")+servName+s("service-close-2"));
      }
    // methods ------------------
    NodeList methList = doc.getElementsByTagName(s("method-tag"));        // get all METHODS
    int methCount = methList.getLength();
    for (int m=0;m<methCount;m++)                                         // write all static page-infos
      if (methList.item( m ).getNodeType() == Node.ELEMENT_NODE) {
        Element meth = (Element) methList.item( m );
        String methHead = meth.getAttribute(s("method-modifier")) + " " +
                          meth.getAttribute(s("method-name")) + "(" +
                          meth.getAttribute(s("method-param")) + ")";
        if (meth.getAttribute(s("method-throws")).length() > 0)
                          methHead += " throws " + meth.getAttribute(s("method-throws"));
        NodeList commList = meth.getElementsByTagName(s("comment-tag"));  // get all COMMENTS
        classWriter.write(s("comment-open"));
        int commCount = commList.getLength();
        for (int c=0;c<commCount;c++)
          classWriter.write(s("comment-lf") + getString(commList.item(c), true));
        classWriter.write(s("comment-close"));
        classWriter.write(s("method-open-1") + methHead + s("method-open-2"));
        NodeList requList = meth.getElementsByTagName(s("java-tag"));  // get all REQUESTS
        int requCount = requList.getLength();
        for (int q=0;q<requCount;q++) {
          Element requ = (Element) requList.item( q );
          String requName = requ.getAttribute(s("request-name"));
          classWriter.write(getString(requ, false));
        }
        classWriter.write(s("method-close"));
      }

    classWriter.write( s("class-end-1")+packageName+"."+className+s("class-end-2"));
    classWriter.close();
    tmpWriter.close();
  }

  /**
   * Returns the content of an element by processing the String to Java-conform notation
   */
  private String getString(Node node, boolean doTrimm) {
    String s = "";
    NodeList codeList = node.getChildNodes();
    try{
      for (int l=0;l<codeList.getLength();l++) {
        String cl = codeList.item( l ).getNodeValue();
        if (codeList.item( l ).getNodeType() == Node.CDATA_SECTION_NODE )
          for (int i=0;i<entities.getLength();i++) {
            if (entities.item(i).getNodeType() == Node.ENTITY_NODE) {
              EntityDecl e = (EntityDecl) entities.item(i);
              String eName = s("&") + e.getNodeName() + s(";");
              while (cl.indexOf( eName )>0)
                cl = cl.substring(0, cl.indexOf( eName )) + e.getValue() +
                        cl.substring(cl.indexOf( eName ) + eName.length(), cl.length());
            }
          } else if (codeList.item( l ).getNodeType() == Node.ENTITY_REFERENCE_NODE )
            cl = getString(codeList.item( l ), doTrimm);
        if (cl != null)
          if (doTrimm)
            s += cl.trim();
          else
            s += cl;
      }
    } catch (Exception e) {
      System.err.println(e.toString());
    }
    return s;
  }

  /**
   * Writes a String in Java-notation to a writer
   */
  private void writeJava(Writer w, String s, boolean writeTmp) throws IOException {
    int l = s.length();
    for (int i=0;i<l;i++)
      switch (s.charAt(i)) {
        case '\n': w.write('\\'); w.write('n');
          break;
        case '\t': w.write('\\'); w.write('t');
          break;
        case '"': w.write('\\'); w.write('"');
          break;
        default: w.write(s.charAt(i));
          break;
      }
    if (writeTmp)
      tmpWriter.write(s + "\n");
  }

  /**
   * Returns the content of an element by processing the String to HTML-conform notation
   */
  private String getHTML(Element node) {
    String s = getString(node, true);
    s = replaceChar(s, '', "&uuml;");
    s = replaceChar(s, '', "&ouml;");
    s = replaceChar(s, '', "&auml;");
    s = replaceChar(s, '', "&Uuml;");
    s = replaceChar(s, '', "&Ouml;");
    s = replaceChar(s, '', "&Auml;");
    s = replaceChar(s, '', "&szlig;");
    return s;
  }

  private static String replaceChar(String s, char c, String r) {
    String x = "";
    try {
      int i = s.indexOf(c);
      while (i >= 0) {
        x += s.substring(0, i) + r;
        s =  s.substring(i+1, s.length());
        i = s.charAt(c);
      }
    } catch (Exception e) {}
    return x + s;
  }

  /**
   * Returns the asked string
   */
  private static String s (String key) {
    return pres.getString( key );
  }

}
