// This file is part of Bonfire
// (P) 1998 by Robert "I'll do it tomorrow!" Kosara, rkosara@ifs.tuwien.ac.at
// http://www.ifs.tuwien.ac.at/~rkosara
//
// 160299 KH add *DTD methods and *DTD attributes after scema *HTML

package bonfire;

import java.io.*;
import com.objectspace.jgl.*;

public class BNFParserNode extends SimpleNode {

	public static final int TYPE = 0;

	public static final int OPTIONAL_F = 1;
	public static final int ONEORMORE_F = 2;
	public static final int ZEROORMORE_F = 4;
	public static final int OR_F = 8;

	public Token first_token, last_token;

	protected boolean visited;
	protected ClassAttribute attr;

	protected BNFParserNode parent = null;

	protected SList noqs = new SList();

	public BNFParserNode(int i) {
		super(i);
		visited = false;
		attr = null;
	}
	
	public BNFParserNode(BNFParser p, int i) {
		super(p, i);
		visited = false;
		attr = null;
	}

	public int getTYPE() {
		return TYPE;
	}

	public void setVisited(boolean val, boolean recurse) {
		visited = val;
	}
	
	public void setVisited(boolean val) {
		setVisited(val, false);
	}

	public boolean isVisited() {
		return visited;
	}

	public BNFParserNode getParent() {
		return parent;
	}
	
	public void dumpASCII(PrintWriter w) {
		w.println("BNFParserNode");
	}
	
	public void resolve(Definition user, BNFParserNode paren) {
		System.err.println("BNFParserNode.resolve()");
	}

	public void dumpHTML(PrintWriter w) {
		System.err.println("BNFParserNode.dumpHTML()");
	}

	public void dumpDTD(PrintWriter w) {
		System.err.println("BNFParserNode.dumpDTD()");
	}

	public void dumpHTMLComments(PrintWriter w) {
		Token tok = first_token;
		do {
			if (tok.specialToken != null)
				dumpSpecialTokens(w, tok.specialToken, true);
			if (tok != last_token) tok = tok.next;
			else tok = null;
		} while (tok != null);
	}

	public void dumpHTMLNewLines(PrintWriter w) {
		Token tok = first_token;
		if (tok.specialToken != null)
			dumpSpecialTokens(w, tok.specialToken, false);
	}

	/** Dump the special tokens: comments and linefeeds.
	 * @param w where to print the HTML
	 * @param tok start at this token
	 * @param comment if true, dump the comments, otherwise the linefeeds
	*/
	public void dumpSpecialTokens(PrintWriter w, Token tok, boolean comment) {
		if ((tok.specialToken != null) && (comment))
			dumpSpecialTokens(w, tok.specialToken, comment);
		if ((comment) && (tok.image.charAt(0) != '\n') &&
			(tok.image.charAt(0) != '\r')) {
			String s = HTMLify(translateComment(tok.image), false);
			int i = 0;
			while ((i < s.length()) && (Character.isWhitespace(s.charAt(i))))
				i++;
			if (i == s.length()) return;
			w.println("<blockquote>"+s);
			if (noqs.size() > 0) {
				w.print("<br><b>See also</b> ");
				SListIterator it = noqs.begin();
				if (it.get() instanceof String) w.print((String)it.get());
				else ((NoteorQuestion)it.get()).dumpHTMLSee(w);
				while (!it.atEnd()) {
					it.advance();
					if (!it.atEnd()) {
						w.print(", ");
						if (it.get() instanceof String) w.print((String)it.get());
						else ((NoteorQuestion)it.get()).dumpHTMLSee(w);
					}
				}
			}
			w.println("</blockquote>");
		} else if ((!comment) && ((tok.image.charAt(0) == '\n') ||
								(tok.image.charAt(0) == '\r'))) {
			w.println("<br>");
			tok.image = " "; // very dirty ... but prevents output of the same comment more than once
		}
	}

	public DList generateAttributes(Definition def, int env, boolean toplevel) {
		System.err.println("BNFParserNode.generateAttributes()");
		return null;
	}

	public void appendDList(DList list1, DList list2) {
		for (DListIterator it=list2.begin(); !it.atEnd(); it.advance())
			list1.add(it.get());
	}
    
	public void dumpParser(PrintWriter w) {
		System.err.println("BNFParserNode.dumpParser()");
	}  

	public void maketoString(PrintWriter w, int env) {

	}
	
	public void setAttribute(ClassAttribute a) {
		attr = a;
	}

	public ClassAttribute getAttribute() {
		return attr;
	}

	public String HTMLify(String s, boolean term) {
		StringBuffer sb = new StringBuffer(100);
		for (int i = 0; i < s.length(); i++)
			switch(s.charAt(i)) {
				/*case '\n':
				  case '\r':
				  sb.append("<br>"); break;*/
			case '':
				sb.append("&auml;"); break;
			case '':
				sb.append("&ouml;"); break;
			case '':
				sb.append("&uuml;"); break;
			case '':
				sb.append("&Auml;"); break;
			case '':
				sb.append("&Ouml;"); break;
			case '':
				sb.append("&Uuml;"); break;
			case '':
				sb.append("&szlig;"); break;
			case '<':
				if (term) sb.append("&lt;");
				else sb.append(s.charAt(i));
				break;
			case '>':
				if (term) sb.append("&gt;");
				else sb.append(s.charAt(i));
				break;
			default: sb.append(s.charAt(i));
			}
		return sb.toString();
	}

	static String translateNonTerm(String s) {
		StringBuffer sb = new StringBuffer(100);
		for (int i = 0; i < s.length(); i++)
			switch(s.charAt(i)) {
			case '-':
			case '*':
			case '=':
			case ' ':
			case '/':
			case '.':
				sb.append("_"); break;
			case '>':
				sb.append("GREATER"); break;
			case '<':
				sb.append("LESS"); break;
			case '[':
				sb.append("LBRACK"); break;
			case ']':
				sb.append("RBRACK"); break;
			case '{':
				sb.append("LBRACE"); break;
			case '}':
				sb.append("RBRACE"); break;
			case '(':
				sb.append("LPAREN"); break;
			case ')':
				sb.append("RPAREN"); break;
			case '&':
				sb.append("AMPERSAND"); break;
			case '%':
				sb.append("PERCENT"); break;
			case ',':
				sb.append("COMMA"); break;				
			case ':':
				sb.append("COLON"); break;								
			case ';':
				sb.append("SEMICOLON"); break;
			case '#':
				sb.append("HASH"); break;
			case '\\':
				sb.append("BACKSLASH"); break;
			default: sb.append(s.charAt(i));
			}
		return sb.toString();
	}
	
	static String translateTerm(String s) {
		StringBuffer sb = new StringBuffer(100);
		for (int i = 0; i < s.length(); i++)
			switch(s.charAt(i)) {
			case '_':
				if (s.length() == 1) sb.append("UNDERSCORE");
				else sb.append("_");
				break;
			case ' ':
				if (s.length() == 1) sb.append("SPACE");
				else sb.append("_");
				break;
			case '-':
				if (s.length() == 1) sb.append("MINUS");
				else sb.append("_");
				break;
			case '+':
				sb.append("PLUS"); break;
			case '*':
				sb.append("ASTERISK"); break;
			case ',':
				sb.append("COMMA"); break;
			case ':':
				sb.append("COLON"); break;
			case ';':
				sb.append("SEMICOLON"); break;
			case '.':
				sb.append("POINT"); break;
			case '>':
				sb.append("GREATER"); break;
			case '<':
				sb.append("LESS"); break;
			case '=':
				sb.append("EQUAL"); break;
			case '/':
				sb.append("SLASH"); break;
			case '\\':
				sb.append("BACKSLASH"); break;
			case '[':
				sb.append("LBRACK"); break;
			case ']':
				sb.append("RBRACK"); break;
			case '{':
				sb.append("LBRACE"); break;
			case '}':
				sb.append("RBRACE"); break;
			case '(':
				sb.append("LPAREN"); break;
			case ')':
				sb.append("RPAREN"); break;
			case '#':
				sb.append("HASH"); break;
			case '&':
				sb.append("AMPERSAND"); break;
			case '%':
				sb.append("PERCENT"); break;
			default: sb.append(s.charAt(i));
			}
		return sb.toString();
	}

	public String translateComment(String comment) {
		String lc = new String(comment);
		if (lc.startsWith("//")) lc = lc.substring(2, lc.length());
		else if (lc.startsWith("/*")) {
			int ofs = 2;
			if (lc.charAt(2) == '*') ofs = 3;
			lc = lc.substring(ofs, lc.length());
		}
		if (lc.endsWith("*/")) lc = lc.substring(0, lc.length()-2);
		char c[] = lc.toCharArray();
		int ofs = 0;
		while (ofs < lc.length()) {
			while ((ofs < lc.length()) && (c[ofs] != '\n')) ofs++;
			ofs++;
			while ((ofs < lc.length()) && ((c[ofs] == ' ') || (c[ofs] == '\t'))) ofs++;
			if ((ofs < lc.length()) && (c[ofs] == '*'))
				c[ofs] = ' ';								
			ofs++;
		}
		lc = new String(c);
		int i = 0;
		while (i < lc.length()) {
			if (lc.charAt(i) == '<') {
				int j = i+1;
				while ((j < lc.length()) && (lc.charAt(j) != '>')) j++;
				if (j < lc.length()) {
					String ntname = lc.substring(i+1, j);
					Definition def =  BNFParser.getDefinition(ntname);
					if (def != null)
						lc = lc.substring(0, i)+"<a href=\"#"+ntname+"\">&lt;"+ntname+"&gt;</a>"+lc.substring(j+1, lc.length());
					i = j+1;
				} else i++;
			} else i++;
		}
		c = lc.toCharArray();
		i = 0;
		while (i < lc.length()) {
			if (c[i] == '@') {
				int starti = i++;
				boolean blankout = false;
				String cmd = new String();
				while ((i < lc.length()) && (Character.isLetter(c[i]))) 
					cmd += c[i++];
				if (cmd.equals("note") || (cmd.equals("question"))) {
					while ((i < lc.length() && (c[i] != '\n'))) i++;
					int type = cmd.equals("note")?NoteorQuestion.T_NOTE:NoteorQuestion.T_QUESTION;
					noqs.add(new NoteorQuestion(type, new String(c, starti+cmd.length()+1, i-starti-cmd.length()-1), Definition.current));
					blankout = true;
				} else if (cmd.equals("remark")) {
					while ((i < lc.length() && (c[i] != '\n'))) i++;
					blankout = true;
				} else if (cmd.equals("see")) {
					i++;
					String link = new String();
					while ((i < lc.length()) && (c[i] != '\n')) 
						link += c[i++];
					noqs.add(link);
					blankout = true;
				}
				if (blankout)
					for (int j = starti; j < i; j++)
						c[j] = ' ';
			} else i++;
		}
		return new String(c);
	}


}
