// This file is part of Bonfire
// (P) 1998 by Robert "I'll do it tomorrow!" Kosara, rkosara@ifs.tuwien.ac.at
// http://www.ifs.tuwien.ac.at/~rkosara
//
// 160299 KH add *DTD methods and *DTD attributes after scema *HTML
//           add -pD switch for DTD dir
//           add -D  switch for create DTD only 

package bonfire;

import java.io.*;
import java.util.*;
import java.text.*;
import com.objectspace.jgl.*;

/** Bonfire is a tool that creates a parser and a class hierarchy from the BNF
 *	definition of a context-free language.
 *
 *  <p>Usage: Bonfire <i>language</i>
 *
 *  <p>The BNF is then read from the file <i>language.bnf</i>.
 *
 *  @author <a href="http://www.ifs.tuwien.ac.at/~rkosara">Robert Kosara</a>
 */
public class Bonfire {

	/** The version number of Bonfire. */
	public final static String VERSION = "2.1";

	/** The starting non-terminal for the parser. There isn't necessarily
	 *  a connection to the roots of the class hierarchy.
	 */
	private static Definition root = null;
	private static String pkg = null;
	private static String lastpathpart = null;
	private static StructComment currsection = null;
	private static StructComment firstsection = null;
	private static String langname = null;
	private static String langfilename = null;
	private static String pathname = "";

	private static ResourceBundle bres = null;
	private static ResourceBundle lres = null;

	private static boolean makeHTML = true;
	private static boolean makeDTD = true;
	private static boolean makeclasses = true;
	private static boolean silent = false;
	private static boolean usejikes = false;

	private static String classdir = null;
	private static String HTMLdir = null;
	private static String DTDdir = null;
	private static String HTMLjavadocdir = null;
	private static String javadocdir = null;

	/** Main procedure, calls all the other functions.
	 */
	public static void main(String args[]) {
		doit(args, Locale.US);
	}

	public static void doit(String args[], Locale loc) {
		bres = ResourceBundle.getBundle("bonfire.resources.BonfireRes", loc);
		lres = ResourceBundle.getBundle("bonfire.resources.JavaRes");
		langname = null;
		for (int i = 0; i < args.length; i++) {
			if (args[i].charAt(0) == '-') {
				boolean skiparg = false;
				int j = 1;
				while ((j < args[i].length()) && (!skiparg)) {
					switch(args[i].charAt(j)) {
					case 'c': makeHTML = false;
                    makeDTD = false;
						break;
					case 'p':
						if ((i+1 >= args.length) || (j+1 == args[i].length()))
							usage(false);
						else {
							String tstr = null;
							if (j+2 >= args[i].length()) {
								tstr = args[i+1];
								skiparg = true;
							} else
								tstr = args[i].substring(j+2, args[i].length());
							if (!tstr.endsWith("/")) tstr += "/";
							switch (args[i].charAt(j+1)) {
							case 'H': HTMLdir = tstr; break;
              case 'D': DTDdir = tstr; break;
							case 'c': classdir = tstr; break;
							case 'J': HTMLjavadocdir = tstr; break;
							case 'j': javadocdir = tstr; break;
							default: usage(false);
							}
							if (!skiparg) j = args[i].length()-1;

						}
						break;
					case 'H': makeclasses = false;
                    makeDTD = false;
						break;
					case 'D': makeclasses = false;
                    makeHTML = false;
						break;
					case 'h': usage(true);
						break;
					case 's': silent = true;
						break;
					case 'k': usejikes = true;
						break;
					default: 
						usage(false);
					}				
					j++;
				}
				if (skiparg) i++;
			}
			else if (langname == null) langname = args[i];
			else usage(false);
		}
		Object ver[] = { VERSION };
		out("greetings", ver);
		if  (langname == null) usage(false);
		if (langname.endsWith(".bnf"))
			langname = langname.substring(0, langname.length()-4);
		int lastindex = langname.lastIndexOf("/");
		if (lastindex >= 0) {
			pathname = langname.substring(0, lastindex+1);
			if (HTMLdir == null) HTMLdir = new String(pathname);
			if (DTDdir == null) DTDdir = new String(pathname);
			if (classdir == null) classdir = new String(pathname);
			langname = langname.substring(lastindex+1);
		} else {
			if (HTMLdir == null) HTMLdir = "";
			if (DTDdir == null) DTDdir = "";
			if (classdir == null) classdir = "";
		}			
		langfilename = new String(langname);
		if (HTMLjavadocdir == null) HTMLjavadocdir = format("javadocdefaultdir");
		if (javadocdir == null) HTMLjavadocdir = format("javadocdefaultdir");
		Object fname[] = { pathname+langfilename+".bnf" };
		out("opening", fname);
		//		System.out.println("Opening file \""+args[0]+".bnf\".");
		try {
			BNFParser.ReInit(new FileInputStream(pathname+langfilename+".bnf"));
		} catch (FileNotFoundException e) {
			error("filenotfound", fname);
		}
		out("ParsingEBNF", fname);
		EBNFDefinitions tree = null;
		try {
			tree = BNFParser.parse();
		} catch (ParseException e) {
			System.out.println(e.toString());
			error("parseerror", "");
		}
		Object termnonterms[] = { new Integer(BNFParser.getNumDefs()),
								  new Integer(BNFParser.getNumTerminals()) };
		out("termnonterms", termnonterms);
		out("looking");
		tree.resolve();
		if (tree.countRoots() > 1) {
			out("moreroots", new Integer(tree.countRoots()).toString());
			for (SListIterator it=tree.getFirstRootIt(); !it.atEnd(); ) {
				outs("<"+((Definition)it.get()).getName()+">");
				it.advance();
				if (!it.atEnd()) outs(", ");
			}
			outs("\n");
			root = tree.findRoot();
			out("using", root.name());
		} else if (tree.countRoots() == 1) {
			root = tree.findRoot();
			out("rootis", root.name());
		} else error("noroots", "");
		if (makeHTML)
			tree.dumpHTML(langfilename);
		if (makeDTD)
			tree.dumpDTD(langfilename);
		if (makeclasses) {
			out("deriving");
			tree.deriveClasses();
			out("writeclasses");
			tree.dumpClasses(langfilename);
			out("writeparser", classdir+langfilename+"Parser.jjt");
			tree.dumpParser(langfilename, classdir+langfilename, BNFParserNode.translateNonTerm(root.getName()));
			out("writemakefile", classdir+"Makefile");		
			PrintWriter m = openFile(classdir+"Makefile");
			Object margs[] = { (new Date()).toString(), langfilename, pkg, lastpathpart, javadocdir, (usejikes)?"jikes":"javac" };
			// This is so braindead! But the makefile must be split up,
			// otherwise MessageFormat.format() throws an exception!
			m.print(format("makefile", margs));
			m.println(format("makefile2", margs));
			m.println(format("makefile3", margs));
			m.close();
		}
		out("bye");
	}
		
	// This comment intentionally left blank
	
	public static void usage(boolean longusage) {
		silent = false;
		if (longusage) out("longusage");
		else out("usage");
		out("sorry");
		System.exit(1);
	}

	/** Print out an error message to stderr and exit with an error status.
	 *
	 *  @param msg Error-message to print
	 */
	public static void error(String key, Object[] args) {
		System.err.println();
		System.err.print("*** "+bres.getString("error")+" ");
		System.err.println(MessageFormat.format(bres.getString(key), args)+" ***");    
		System.err.println(bres.getString("sorry"));
		System.exit(1);
	}

	public static void error(String key, String arg) {
		Object args[] = { arg };
		error(key, args);
	}

	/** Open a file for writing, and if anything goes wrong, exit.
	 *
	 *  @param  fname Name of the file to open
	 *  @return PrintWriter object that can be written to
	 */
	public static PrintWriter openFile(String fname) {
		PrintWriter w=null;
		File ofile=new File(fname);
		try {
			w = new PrintWriter(new FileOutputStream(ofile));
		} catch (IOException e) {
			Object arg[] = { e.toString() };
			error("filenotfound", arg);
		}
		return w;
	}


	/** Return the starting non-term for the parser.
	 */
	public static Definition getRoot() {
		return root;
	}

	/** Set the name of the package the parser and all the classes should
	 *  be put in.
	 *
	 *  @param p Name of the Package
	 */
	public static void setPackage(String p) {
		pkg = p;
	}

	public static void setLastPathPart(String l) {
		lastpathpart = l;
	}

	/** Return the name of the package the parser and all the classes should
	 *  be put in.
	 *
	 *  @return Name of the package
	 */
	public static String getPackage() {
		return pkg;
	}

	/** Set the current section for documentation purposes.
	 *
	 *  @param c Section object that contains all the following definitions
	 *  @see StructComment
	 */
	public static void setCurrSection(StructComment c) {
		if (currsection != null) currsection.setNext(c);
		if (firstsection == null) firstsection = currsection;
		currsection = c;
	}

	/** Get the current section
	 *
	 *  @return The currently set section object
	 *  @see StructComment
	 */
	public static StructComment getCurrSection() {
		return currsection;
	}

	/** Return the first section object set.
	 *
	 *  @return The first section object
	 *  @see StructComment
	 */
	public static StructComment getFirstSection() {
		return firstsection;
	}


	public static void setLangName(String n) {
		langname = n;
	}

	/** Return the name of the language currently being processed.
	 *
	 *  @return Name of the language currently being processed
	 */
	public static String getLangName() {
		return langname;
	}

	public static String getLangFileName() {
		return langfilename;
	}

	public static void outs(String str) {
		if (!silent)
			System.out.print(str);
	}

	public static void out(String key, Object args[]) {
		if (!silent)
			System.out.println(MessageFormat.format(bres.getString(key), args));
	}

	public static void out(String key, String arg) {
		if (!silent) {
			Object args[] = { arg };
			out(key, args);
		}
	}

	public static void out(String key) {
		if (!silent)
			System.out.println(bres.getString(key));
	}

	public static String format(String key, Object args[]) {
		return MessageFormat.format(lres.getString(key), args);
	}

	public static String format(String key, String arg) {
		Object args[] = { arg };
		return format(key, args);
	}

	public static String format(String key) {
		return lres.getString(key);
	}

	
	public static String bformat(String key, Object args[]) {
		return MessageFormat.format(bres.getString(key), args);
	}

	public static String bformat(String key, String arg) {
		Object args[] = { arg };
		return bformat(key, args);
	}

	public static String getHTMLDir() {
		return HTMLdir;
	}

	public static String getDTDDir() {
		return DTDdir;
	}

	public static String getClassDir() {
		return classdir;
	}

	public static String getHTMLJavaDocDir() {
		return HTMLjavadocdir;
	}

	public static boolean getMakeClasses() {
		return makeclasses;
	}

}
