// ClassAttribute
// This file is part of Bonfire
// (P) 1998 by Robert "I'll do it tomorrow!" Kosara, rkosara@ifs.tuwien.ac.at
// http://www.ifs.tuwien.ac.at/~rkosara

package bonfire;

import java.lang.*;
import java.io.*;

public class ClassAttribute {

	public static final int TYPE = -1;

	String name;
	String typ;
	BNFParserNode owner;
	boolean intersection = false;

	ClassAttribute(String n, BNFParserNode o) {
		name = n;
		typ = "xyz";
		owner = o;
	}

	public int getTYPE() {
		return TYPE;
	}

	public String getName() {
		return name;
	}

	public void setName(String n) {
		name = n;
	}

	public String getType() {
		return typ;
	}

	public void setIntersection(boolean v) {
		intersection = v;
	}

	public boolean getIntersection() {
		return intersection;
	}

	public BNFParserNode getOwner() {
		return owner;
	}

	/** Return true if the argument is equal to this attribute.
	 *  Two attributes are considered equal if and only if their
	 *  attribute type, name and type name (Java type) are the same.
	 *
	 *  @param other Attribute to compare this one to
	 *  @return True, if they are the same, false otherwise
	 */	
	public boolean equals(ClassAttribute other) {
		return ((other.getName().equals(name)) &&
				(other.getType().equals(typ)) &&
				(other.getTYPE() == this.getTYPE()));
	}
	
	public void dumpDeclaration(PrintWriter w) {
		
	}
	
	void dumpDeclaration(PrintWriter w, String type, String name, String defname) {
		Object arg[] = { name, type };
		w.print(Bonfire.format("classattrdecl", arg));
		if (defname != null) w.print(" = "+defname);
		w.println(";");
		dumpListenerDeclaration(w, name, defname);
	}

	void dumpListenerDeclaration(PrintWriter w, String name, String defname) {
		w.println(Bonfire.format("classattrlistenerdecl", name));
	}

	public void dumpMethods(PrintWriter w, boolean intf) {
		
	}
	
	void dumpMethods(PrintWriter w, String type, String name, boolean intf) {
		Object nametype[] = { name, type };
		w.print(Bonfire.format("classattrget", nametype));
		if (intf) w.println(";");
		else w.println(Bonfire.format("classattrreturn", name));
		w.println();
		w.print(Bonfire.format("classattrset", nametype));
		if (intf) w.println(";");
		else {
			String casttype = null;
			if (type.equals("int")) casttype = "Integer";
			else if (type.equals("boolean")) casttype = "Boolean";
			else if (type.equals("double")) casttype = "Double";
			if (casttype == null)
				w.println(Bonfire.format("classattrnocast", name));
			else {
				Object arg[] = { name, casttype };
				w.println(Bonfire.format("classattrcast", arg));
			}
			w.println(Bonfire.format("classattrsetbody", name));
		}
		dumpListenerMethods(w, name, intf);
	}


	void dumpListenerMethods(PrintWriter w, String name, boolean intf) {
		w.println();
		w.print(Bonfire.format("classattraddlistener", name));
		if (intf) w.println(";");
		else w.println(Bonfire.format("classattralbody", name));
		w.println();
		w.print(Bonfire.format("classattrremlistener", name));
		if (intf) w.println(";"); 
		else w.println(Bonfire.format("classattrrlbody", name));
	}
	
	public void dumpParseCode(PrintWriter w) {
		
	}

	/** only call for PointerAttribute und ListAttribute! */
	public void findSuperClass() {
		Definition d = ((NonTerminal)owner).getDefinition();
		while (d.getSuperclass() != null) {
			//System.err.println(d.getName());
			d = d.getSuperclass();
		}
		//System.err.println();
		typ = BNFParserNode.translateNonTerm(d.getName());
	}

}
