/* Generated By:JJTree: Do not edit this line. NonTerminal.java */
// This file is part of Bonfire
// (P) 1998 by Robert "I'll do it tomorrow!" Kosara, rkosara@ifs.tuwien.ac.at
// http://www.ifs.tuwien.ac.at/~rkosara
//
// 170299 KH add *DTD methods after scema *HTML

package bonfire;

import java.io.*;
import com.objectspace.jgl.*;

/** Non-Terminal. An object of this type is created for every non-terminal
 *  every time it is used in the definition of the language. This is necessary
 *  not only because a non-terminal can be used before it is defined, but also
 *  because different uses of the same non-terminal can lead to different 
 *  attributes being associated with it.
 */
public class NonTerminal extends BNFParserNode {

	public static final int TYPE = 5;

	private String name;
	private Definition def;
	private boolean defined;

	/** Special Type: %STRING */
	static final int NT_STRING = 1;
	/** Special Type: %INTEGER */
	static final int NT_INTEGER = 2;
	/** Special Type: %FLOAT */
	static final int NT_FLOAT = 3;
	/** Special Type: %DATE */
	static final int NT_DATE = 4;

	/** If != 0, this is a special type, not an ordinary non-terminal.
	 *
	 *  @see Definition#Definition(int, int, String)
	 */
	private int specialtype = 0;

	private String myiter = null;

	public NonTerminal(int id) {
		super(id);
	}
	
	public NonTerminal(BNFParser p, int id) {
		super(p, id);
	}

	public int getTYPE() {
		return TYPE;
	}
	
	/** Get name from token, create predefined type definition if necessary.
	 *	Called by parser, don't use!
	 */
	public void jjtClose() {
		name = first_token.image.substring(1, first_token.image.length()-1);
		if (name.charAt(0) != '%') return;
		name = name.substring(1, name.length());
		if (name.equals("STRING")) {
			specialtype = NT_STRING;
		} else if (name.equals("INTEGER")) {
			specialtype = NT_INTEGER;
		} else if (name.equals("FLOAT")) {
			specialtype = NT_FLOAT;
		} else if (name.equals("DATE")) {
			specialtype = NT_DATE;
		} else return;
		def = BNFParser.getDefinition(name);
		if (def == null) {
			def = new Definition(0, specialtype, name);
		}
	}
	
	/** Return useless debugging message.
	 */
	public String toString() {
		return "NonTerminal: <"+name+">";
	}
	
	/** Set definition object of this non-terminal.
	 *
	 *  @param d Definition object to be set
	 *  @see NonTerminal#def
	 */
	public void setDefinition(Definition d) {
		def = d;
		defined = true;
	}

	/** Return definition object of this non-terminal.
	 *
	 *  @return Definition object of this non-terminal
	 */
	public Definition getDefinition() {
		return def;
	}
	
	/** Find definition object of this non-terminal and tell it about
	 *  using definition.
	 *
	 *  @param user Definition this non-terminal appears in
	 */
	public void resolve(Definition user, BNFParserNode paren) {
		parent = paren;
		def = BNFParser.getDefinition(name);
		defined = (def != null);
		if (defined) def.addReferrer(user);
	}

	/** Generate Attribute for this Non-Terminal. The created attribute is
	 *	either a ListAttribute (for ZERO_OR_MORE or ONE_OR_MORE) or
	 *  a PointerAttribute (otherwise).
	 *
	 *  @param def Definition this NonTerminal belongs to
	 *  @param env Description of the "history" of this call in terms of the constants defined in BNFParserNode
	 *  @param toplevel True, if this fragment is on the first level of a definition's tree.
	 *  @return List of generated attributes
	 *  @see ListAttribute
	 *  @see PointerAttribute
	 *  @see BNFParserNode#OPTIONAL_F
	 *  @see BNFParserNode#ONEORMORE_F
	 *  @see BNFParserNode#ZEROORMORE_F
	 *  @see BNFParserNode#OR_F
	 */
	public DList generateAttributes(Definition def, int env, boolean toplevel) {
		DList attrlist = new DList();
		if (!defined) {
			Object arg[] = { name };
			Bonfire.error("notdefined", arg);
		}
		if ((env & (ZEROORMORE_F | ONEORMORE_F)) != 0) {
			attr = new ListAttribute(translateNonTerm(name), this);
			//def.addAttribute(attr);
			attrlist.add(attr);
			def.setNeedNode(true);
		} else {
			attr = new PointerAttribute(translateNonTerm(name), this);
			//def.addAttribute(attr);
			attrlist.add(attr);
			def.setNeedNode(true);
		}
		return attrlist;
	}

	/** Useless debugging function. Please ignore.
	 *
	 *  @param w PrintWriter to write useless debugging message to
	 */
	public void dumpASCII(PrintWriter w) {
		w.print("<"+name+">");
	}

	/** Write HTML part of this non-terminal. Only calls dumpHTML(w, false).
	 *
	 *  @param w PrintWriter to write to
	 *  @see NonTerminal#dumpHTML(PrintWriter, boolean)
	 */
	public void dumpHTML(PrintWriter w) {
		dumpHTML(w, false);
	}

	/** Write DTD part of this non-terminal. Only calls dumpDTD(w, false).
	 *
	 *  @param w PrintWriter to write to
	 *  @see NonTerminal#dumpDTD(PrintWriter, boolean)
	 */
	public void dumpDTD(PrintWriter w) {
		dumpDTD(w, false);
	}

	/** Write HTML part of this non-terminal, for the definition part or the
	 *  cross-ref section.
	 *
	 *  @param w PrintWriter to write to
	 *  @param xref True, if cross-ref is to be written
	 */
	public void dumpHTML(PrintWriter w, boolean xref) {
		if (!xref) {
			//dumpHTMLComments(w);
			dumpHTMLNewLines(w);
		}
		Object arg[] = { name, getName() };
		if (defined) w.print(Bonfire.format("nontermdef", arg));
		else w.print(Bonfire.format("nontermnondef", name));
	}

	/** Write DTD part of this non-terminal, for the definition part or the
	 *  cross-ref section.
	 *
	 *  @param w PrintWriter to write to
	 *  @param xref True, if cross-ref is to be written
	 */
	public void dumpDTD(PrintWriter w, boolean xref) {
		Object arg[] = { name, getName() };
		if (defined) w.print(Bonfire.format("nontermdtd", arg));
		else w.print(Bonfire.format("nontermnondtd", name));
	}

	/** Write HTML cross-ref part of this non-terminal. Only calls
	 *  dumpHTML(w, true).
	 *
	 *  @param w PrintWriter to write to
	 *  @see NonTerminal#dumpHTML(PrintWriter, boolean)
	 */
	public void dumpXREF(PrintWriter w) {
		dumpHTML(w, true);
	}

	/** Write DTD cross-ref part of this non-terminal. Only calls
	 *  dumpDTD(w, true).
	 *
	 *  @param w PrintWriter to write to
	 *  @see NonTerminal#dumpDTD(PrintWriter, boolean)
	 */
	public void dumpDTDREF(PrintWriter w) {
		dumpDTD(w, true);
	}
	/** Return name of this non-terminal without the brackets.
	 *
	 *  @return Name of this non-terminal without the brackets
	 */
	public String getName() {
		if (specialtype == 0) return name;
		else return "%"+name;
	}

	/** Write parser part of this non-terminal.
	 *
	 *  @param w PrintWriter to write to
	 */
	public void dumpParser(PrintWriter w) {
		if ((attr != null) && ((attr instanceof ListAttribute) ||
							   (attr instanceof PassthroughAttribute) ||
							   (attr instanceof PointerAttribute)))
			w.print("n=");
		w.print(translateNonTerm(name)+"()");
		if (attr != null) {
			attr.dumpParseCode(w);
			//w.print("}");
		}
	}

	public void setMyIter(String it) {
		myiter = it;
	}

	public void maketoString(PrintWriter w, int env) {		
		if (attr.getTYPE() == PassthroughAttribute.TYPE) return;
		if ((env & (Expr.ONEORMORE_F | Expr.ZEROORMORE_F)) != 0) {
			if (myiter == null) {
				Object args[] = { attr.getName(),
								  Bonfire.getLangFileName() };
				w.print(Bonfire.format("nttsmoresingle", args));
			} else {
				Object args[] = { attr.getName(),
								  Bonfire.getLangFileName(),
								  myiter };
				w.print(Bonfire.format("nttsmoremulti", args));
			}
		} else if ((env & (Expr.OPTIONAL_F | Expr.OR_F)) != 0)
			w.print(Bonfire.format("nttsopt", attr.getName()));
		else w.print(Bonfire.format("ntts", attr.getName()));
	}

}
